///<reference path="orgAnice.dialogManager.d.ts"/>
var DialogCommandManager = (function () {
    // --- Commands -------------------------------------------------------
    // --- Constructor ----------------------------------------------------
    function DialogCommandManager(unsavedDialog) {
        // --- Events ---------------------------------------------------------
        this.DialogAdded = new orgAniceModule.orgAniceEvent();
        this.QueryFailed = new orgAniceModule.orgAniceEvent();
        this.QuerySent = new orgAniceModule.orgAniceEvent();
        this.QuerySucceeded = new orgAniceModule.orgAniceEvent();
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.Ready = new orgAniceModule.orgAniceEvent();
        this.SelectionChanged = new orgAniceModule.orgAniceEvent();
        // --- Fields ---------------------------------------------------------
        this.currentDialog = undefined;
        this.currentIsModified = false;
        this.currentName = "unknown";
        this.dialogs = {};
        this.isWarnUnsavedChanged = true;
        this.warnUnsavedDialog = unsavedDialog;
        setTimeout(function () {
        }, 5000);
        this.onReady();
    }
    Object.defineProperty(DialogCommandManager.prototype, "IsQueryOnAddDialog", {
        get: function () {
            return this.isQueryOnAddDialog;
        },
        set: function (value) {
            if (value === this.isQueryOnAddDialog)
                return;
            this.isQueryOnAddDialog = value;
            this.onPropertyChanged("IsQueryOnAddDialog");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "IsSessionStorageSupported", {
        get: function () {
            return this.isSessionStorageSupported;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "SearchControl", {
        get: function () {
            return this.searchControl;
        },
        set: function (searchControl) {
            this.searchControl = searchControl;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "SessionId", {
        get: function () {
            var session = this.GetSessionValue("__session__");
            if (session == undefined) {
                session = moment().valueOf().toString();
                this.SetSessionValue("__session__", session);
            }
            return session;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "WarnUnsavedDialog", {
        get: function () { return this.warnUnsavedDialog; },
        set: function (dialog) { this.warnUnsavedDialog = dialog; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    DialogCommandManager.prototype.AddDialogItem = function (dialog) {
        var _this = this;
        this.dialogs[dialog.Name] = dialog;
        dialog.Parent = this;
        dialog.PropertyChanged.add(function (name) {
            _this.onPropertyChanged(name);
        });
        dialog.QueryFailed.add(function (name) {
            _this.QueryFailed.trigger(name);
        });
        dialog.QuerySent.add(function (name) {
            _this.QuerySent.trigger(name);
        });
        dialog.QuerySucceeded.add(function (name) {
            _this.QuerySucceeded.trigger(name);
        });
        if (this.searchControl != undefined) {
            this.searchControl.IndexChanged.add(function (args) {
                var options = Array.isArray(args) ? args[0] : args;
                if (dialog.Controller != options.Controller)
                    return;
                dialog.IndexChanged(options);
            });
        }
        this.DialogAdded.trigger(this, dialog);
        if (this.isQueryOnAddDialog)
            dialog.Query("00000000-0000-0000-0000-000000000001"); // query for current entry
    };
    /// <summary>This command calls the ClearCommand of the current dialog.</summary>
    DialogCommandManager.prototype.Clear = function (name) {
        if (!this.validateSavedModifications)
            return;
        try {
            var dialog = (name != undefined && this.dialogs.hasOwnProperty(name))
                ? this.dialogs[name]
                : this.currentDialog;
            if (dialog == undefined)
                return;
            dialog.Clear();
        }
        catch (exception) {
        }
    };
    Object.defineProperty(DialogCommandManager.prototype, "CurrentDialog", {
        get: function () {
            if (this.currentDialog != undefined)
                return this.currentDialog;
            if (this.currentName != undefined && this.currentName != "unknown")
                this.Select(this.currentName);
            var keys = Object.keys(this.dialogs);
            if (keys.length > 0)
                this.Select(keys[0]);
            //			this.currentDialog = this.dialogs[keys[0]];
            return this.currentDialog;
        },
        enumerable: true,
        configurable: true
    });
    DialogCommandManager.prototype.GetDialogByName = function (name) {
        if (this.dialogs.hasOwnProperty(name))
            return this.dialogs[name];
        return undefined;
    };
    DialogCommandManager.prototype.GetSessionValue = function (name) {
        if (this.isSessionStorageSupported) {
            return window.sessionStorage.getItem(name.toLowerCase());
        }
        else {
            return $.cookie(window.orgAnice.MakeStorageKey(name));
        }
    };
    DialogCommandManager.prototype.HandleAjaxError = function (xhr) {
        if (xhr == undefined) {
            this.HandleAjaxErrorImpl(0);
            return;
        }
        this.HandleAjaxErrorImpl(xhr.status);
    };
    DialogCommandManager.prototype.HandleAjaxErrorImpl = function (reason) {
        var url = "/";
        var domain = $.cookie("orgAniceDomain");
        if (domain != undefined && domain !== "" && domain != "local")
            url += domain + "/";
        url += $.cookie("orgAniceProvider");
        if (reason != undefined) {
            if (typeof reason == "string") {
                url += "?reason=" + reason;
            }
            else if (typeof reason == "number") {
                switch (reason) {
                    case 401:
                        url += "?reason=SessionExpired";
                        break;
                    case 0:
                        url += "?reason=InternalError";
                        break;
                    default:
                        url += "?reason=SessionExpired";
                        break;
                }
            }
        }
        window.location.assign(url);
    };
    DialogCommandManager.prototype.IgnoreModified = function () { this.isCurrentModified = false; };
    Object.defineProperty(DialogCommandManager.prototype, "IsCurrentModified", {
        get: function () {
            if (this.currentDialog == undefined)
                return false;
            return this.currentDialog.IsModified;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "IsModified", {
        get: function () {
            var result = false;
            $.each(this.dialogs, function (index, dialog) {
                if (dialog.isModified)
                    result = result || true;
            });
            // ReSharper disable once ExpressionIsAlwaysConst
            return result;
        },
        set: function (value) {
            var _this = this;
            $.each(this.dialogs, function (index, dialog) {
                dialog.isModified = false;
                if (_this.currentDialog != undefined)
                    _this.currentDialog.IsModified = value;
            });
        },
        enumerable: true,
        configurable: true
    });
    DialogCommandManager.prototype.OnQuerySucceeded = function (dialog) {
        this.QuerySucceeded.trigger(dialog);
    };
    DialogCommandManager.prototype.Query = function (id) {
        if (this.currentDialog == undefined || this.currentDialog == null)
            return;
        this.currentDialog.Query(id);
    };
    DialogCommandManager.prototype.QueryDialog = function (name, id) {
        var dialog = this.dialogs.hasOwnProperty(name) ? this.dialogs[name] : undefined;
        if (dialog != undefined)
            dialog.Query(id);
    };
    DialogCommandManager.prototype.RequeryAll = function (exceptDialog) {
    };
    DialogCommandManager.prototype.Restore = function () {
        if (this.CurrentDialog != undefined)
            this.CurrentDialog.Restore();
        this.currentIsModified = false;
        this.onPropertyChanged("IsModified");
    };
    DialogCommandManager.prototype.Save = function (validate) {
        try {
            var d = this.currentDialog;
            //TODO: this should re-direct to the error page!
            if (d == undefined)
                return;
            this.currentDialog.Save(validate);
        }
        catch (exception) {
        }
    };
    DialogCommandManager.prototype.SaveIfModified = function (validate) {
        if (!this.IsModified || this.currentDialog == undefined)
            return true;
        return this.currentDialog.SaveIfModified(validate);
    };
    DialogCommandManager.prototype.Select = function (name) {
        if (typeof name == "number") {
            if (name >= Object.keys(this.dialogs).length)
                return null;
            var targetDialog = Object.keys(this.dialogs)[name];
            if (targetDialog == undefined)
                return null;
            return this.Select(targetDialog);
        }
        if (this.currentDialog != undefined && this.currentDialog != null) {
            if (this.currentDialog.Name === name)
                return this.currentDialog;
            if (this.currentDialog.IsModified || this.currentDialog.IsNew) {
                if (this.warnUnsavedDialog != undefined) {
                    if (this.warnUnsavedDialog.apply(null)) {
                        this.currentDialog.Save();
                    }
                    else {
                        this.currentDialog.Restore();
                    }
                }
            }
        }
        var caption;
        if (this.currentName != undefined && this.currentName !== "") {
            caption = $("#" + this.currentName + " .dialog");
            if (caption != undefined)
                caption.removeClass("active");
        }
        this.currentName = name;
        caption = $("#" + this.currentName + " .dialog");
        if (caption != undefined)
            caption.addClass("active");
        this.currentDialog = this.dialogs[name]; // set new dialog to current
        if (this.currentDialog != undefined) {
            this.currentDialog.IsModified = false; // set IsModified & IsNew to false
            this.currentDialog.IsNew = false;
        }
        this.onSelectionChanged(this.currentDialog);
        if (this.currentDialog != undefined && this.searchControl != undefined)
            this.currentDialog.QueryIndexes();
        return this.currentDialog;
    };
    DialogCommandManager.prototype.SelectByList = function (list) {
        var selector = this.CurrentDialog;
        if (selector.List != undefined && selector.List.Name === list.Name)
            return true;
        $.each(this.dialogs, function (index, value) {
            if (value.List != undefined && value.List.Name === list.Name) {
                selector = value;
                return false;
            }
            return true;
        });
        if (selector.List != undefined && selector.List.Name === list.Name) {
            this.Select(selector);
            return true;
        }
        return false;
    };
    DialogCommandManager.prototype.SetModified = function () { this.isCurrentModified = true; };
    DialogCommandManager.prototype.SetSessionValue = function (name, value) {
        if (this.isSessionStorageSupported) {
            window.sessionStorage.setItem(name.toLowerCase(), value);
        }
        else {
            // if sessionStorage is not supported by the browser, we use a cookie instead.
            // BUT! This cookie must be qualified somehow, as there might be many sessions open at least on a desktop!
            $.cookie(window.orgAnice.MakeStorageKey(name), value, { path: "/" });
        }
    };
    DialogCommandManager.prototype.Validate = function () {
        if (this.currentDialog == undefined)
            return true;
        return this.currentDialog.Validate();
    };
    DialogCommandManager.prototype.ValidateSessionIsAlive = function () {
        var isAlive = false;
        $.ajax({
            url: "/Account/IsAlive",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: "",
            type: "GET",
            async: false,
            cache: false
        })
            .done(function (result) {
            isAlive = result.Success != undefined && result.Success;
        })
            .fail(function () {
            isAlive = false;
        });
        // ReSharper disable once ExpressionIsAlwaysConst
        return isAlive;
    };
    Object.defineProperty(DialogCommandManager.prototype, "isCurrentModified", {
        // --- Implementation -------------------------------------------------
        set: function (value) {
            this.currentIsModified = value;
            this.onPropertyChanged("IsModified");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManager.prototype, "isSessionStorageSupported", {
        get: function () {
            if (this.isStorageSupported != undefined)
                return this.isStorageSupported;
            try {
                window.sessionStorage.setItem("Test", "Test");
                window.sessionStorage.removeItem("Test");
                this.isStorageSupported = true;
                return true;
            }
            catch (e) {
                this.isStorageSupported = false;
                return false;
            }
        },
        enumerable: true,
        configurable: true
    });
    DialogCommandManager.prototype.onPropertyChanged = function (propertyName) {
        propertyName = Array.isArray(propertyName) ? propertyName[0] : propertyName;
        this.PropertyChanged.trigger(propertyName);
    };
    DialogCommandManager.prototype.onReady = function () {
        this.isReadyFired = true;
        this.Ready.FireOnAdd(this.isReadyFired);
        this.Ready.trigger("");
    };
    DialogCommandManager.prototype.onSelectionChanged = function (dialog) {
        this.SelectionChanged.trigger(dialog);
    };
    Object.defineProperty(DialogCommandManager.prototype, "validateSavedModifications", {
        get: function () {
            return (this.currentDialog == undefined ||
                this.currentDialog == null ||
                !(this.currentDialog.IsModified || this.currentDialog.IsNew));
        },
        enumerable: true,
        configurable: true
    });
    // --- Fields ---------------------------------------------------------
    DialogCommandManager.StorageTag = "__";
    // --- Properties -----------------------------------------------------
    DialogCommandManager.EmptyRequestGuid = "00000000-0000-0000-0000-000000000001";
    return DialogCommandManager;
}());
var DialogCommandManagerItem = (function () {
    // --- Constructor ----------------------------------------------------
    function DialogCommandManagerItem(name, controller, dataEndpoint) {
        var _this = this;
        // --- Events ---------------------------------------------------------
        this.QueryFailed = new orgAniceModule.orgAniceEvent();
        this.QuerySent = new orgAniceModule.orgAniceEvent();
        this.QuerySucceeded = new orgAniceModule.orgAniceEvent();
        this.OnClearCalled = new TypedEvent();
        this.OnDeleted = new TypedEvent();
        this.OnQuery = new TypedEvent();
        this.OnSaved = new TypedEvent();
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.RelationsChanged = new orgAniceModule.orgAniceEvent();
        this.ValidationStateChanged = new orgAniceModule.orgAniceEvent();
        this.canCreate = true;
        this.canDelete = true;
        this.canSave = true;
        this.currentOid = "00000000-0000-0000-0000-000000000000";
        this.isEnabled = ko.observable(false);
        this.isFillMode = false;
        this.isModified = false;
        this.isNew = false;
        this.isPropertyChangedSuspended = false;
        this.validators = [];
        this.validationState = ko.observable(false);
        // --- Commands -------------------------------------------------------
        /// <summary>This command clears all fields in the dialog.</summary>
        this.ClearCommand = undefined;
        /// <summary>The command which deletes the entry given by its <c>id</c>.</summary>
        this.DeleteCommand = undefined;
        this.FillViewModelCommand = undefined;
        this.InitializeCommand = undefined;
        /// <summary>The query command, used to query a single entry by ID.</summary>
        this.QueryCommand = undefined;
        this.RestoreCommand = undefined;
        /// <summary>The save commands of the dialogs.</summary>
        this.SaveCommand = undefined;
        this.self = this;
        this.name = name;
        this.controller = controller;
        if (dataEndpoint != undefined && typeof dataEndpoint == "string")
            this.serviceEndpoint = dataEndpoint;
        this.validationState.subscribe(function (newValue) {
            _this.onValidationStateChanged(newValue);
        });
        var $this = this;
        this.toplevel = $("#" + name);
        if (this.toplevel != undefined) {
            ko.applyBindingsToNode(document.getElementById(this.name), {
                css: { 'disabled': $this.isEnabled }
            }, null);
        }
        //this.toplevel.focusout((ev: JQueryEventObject) => {
        //	if (!control.IsModified) return;
        //	var target = ev.relatedTarget;
        //	if (control.hasIgnoreFocusLost(target)) return;
        //	var doSave = true;
        //	if (control.Parent.WarnUnsavedDialog != undefined)
        //		doSave = control.Parent.WarnUnsavedDialog.apply(this);
        //	if (doSave)
        //		control.Save();
        //	else
        //		control.Restore();
        //});
    }
    Object.defineProperty(DialogCommandManagerItem.prototype, "DeleteAction", {
        // --- Fields ---------------------------------------------------------
        get: function () { return "/Delete"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IndexesAction", {
        get: function () { return "/Indexes"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "QueryAction", {
        get: function () { return "/GetEntry"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "QueryListAction", {
        get: function () { return "/GetList"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "QueryListShortcutsAction", {
        get: function () { return "/GetListShortcuts"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "SaveAction", {
        get: function () { return "/Save"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "CanCreate", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.canCreate; },
        set: function (value) {
            if (this.canCreate == value)
                return;
            this.canCreate = value;
            this.onPropertyChanged("CanCreate");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "CanDelete", {
        /// <summary>Returns a value indicating, whether this instance can delete its <see cref="ViewModel"/>. </summary>
        /// <returns><c>true</c>, if values can be deleted; otherwise, <c>false</c>. </returns>
        get: function () {
            return this.canDelete && this.DeleteCommand != undefined;
        },
        set: function (value) {
            if (this.canDelete === value)
                return;
            this.canDelete = value;
            this.onPropertyChanged("CanDelete");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "CanSave", {
        /// <summary>Returns a value indicating, whether this instance can save its <see cref="ViewModel"/>. </summary>
        /// <returns><c>true</c>, if values can be deleted; otherwise, <c>false</c>. </returns>
        get: function () {
            return this.canSave && this.SaveCommand != undefined && !this.validationState();
        },
        set: function (value) {
            if (this.canSave === value)
                return;
            this.canSave = value;
            this.onPropertyChanged("CanSave");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "Controller", {
        get: function () { return this.controller; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "CurrentOid", {
        /// <summary>Returns the OID of the current data object, which will be set by the Query method.< / summary >
        /// <returns>The OID of the current data object.</returns>
        get: function () { return this.currentOid; },
        set: function (value) { this.currentOid = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsEnabledObserved", {
        get: function () { return this.isEnabled; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsEnabled", {
        get: function () { return this.isEnabled(); },
        set: function (value) {
            if (this.isEnabled() === value)
                return;
            this.isEnabled(value);
            this.onPropertyChanged("IsEnabled");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsFillMode", {
        get: function () { return this.isFillMode; },
        set: function (value) {
            if (this.isFillMode === value)
                return;
            this.isFillMode = value;
            this.onPropertyChanged("IsFillMode");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsModified", {
        get: function () { return this.isModified; },
        set: function (value) {
            if (this.isModified === value)
                return;
            this.isModified = value;
            this.onPropertyChanged("IsModified");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsNew", {
        get: function () { return this.isNew; },
        set: function (value) { this.isNew = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsReadonly", {
        get: function () { return this.isReadonly; },
        set: function (value) { this.isReadonly = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "IsSelected", {
        get: function () {
            return (this.Parent.CurrentDialog.Name == this.Name);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "List", {
        get: function () { return this.list; },
        set: function (value) {
            if (this.list != undefined) {
                this.list.PropertyChanged.remove(this.onListPropertyChanged);
            }
            this.list = value;
            this.list.PropertyChanged.add(this.onListPropertyChanged);
            this.onPropertyChanged("List");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "Name", {
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "Parent", {
        get: function () { return this.parent || window.orgAnice.dialogManager; },
        set: function (parent) { this.parent = parent; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "ServiceEndpoint", {
        get: function () {
            if (this.serviceEndpoint == undefined) {
                this.serviceEndpoint = document.location.protocol + "//" + document.location.host + "/" + this.controller;
            }
            return this.serviceEndpoint;
        },
        set: function (value) {
            if (this.ServiceEndpoint === value)
                return;
            this.serviceEndpoint = value;
            this.onPropertyChanged("ServiceEndpoint");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "TypeValidator", {
        get: function () { return this.typeValidator; },
        set: function (value) { this.typeValidator = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "Validators", {
        get: function () {
            return this.validators;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "ValidationHasError", {
        get: function () {
            return this.validationState();
        },
        set: function (value) {
            if (this.validationState() == value)
                return;
            this.validationState(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogCommandManagerItem.prototype, "ViewModel", {
        get: function () { return this.viewModel; },
        set: function (value) { this.viewModel = value; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    DialogCommandManagerItem.prototype.ApplyValidations = function () {
        if (this.viewModel == undefined || this.typeValidator == undefined)
            return;
        var dialogItem = this;
        var viewModel = this.viewModel;
        var typeValidator = this.typeValidator;
        $.each(viewModel, function (index, value) {
            var memberValidator = undefined;
            $.each(typeValidator.MemberValidators, function (mindex, mvalue) {
                if (mvalue.Fieldname !== index)
                    return true; // continue
                memberValidator = mvalue;
                return false; // leave loop
            });
            // ReSharper disable once ConditionIsAlwaysConst
            // ReSharper disable HeuristicallyUnreachableCode
            // ReSharper does not detect that memberValidator was set in the previous $.each() loop
            if (memberValidator != undefined) {
                var fields = $("#" + dialogItem.Name + " #" + memberValidator.Fieldname);
                if (fields == undefined || fields.length == 0)
                    return true; // continue main loop
                $.each(fields, function (findex, hfield) {
                    var field = $(hfield);
                    if (field == undefined)
                        return true; // continue fields loop
                    var binding = field.attr("data-bind"), setBinding = false, setCss = false, setValueUpdate = false;
                    $.each(memberValidator.Validators, function (idx, validator) {
                        switch (validator.ValidationType) {
                            case 0:
                                if (validator.Length > 0)
                                    field.attr("maxlength", validator.Length);
                                break;
                            case 1:
                                if (validator.Length <= 0)
                                    break;
                                viewModel[memberValidator.Fieldname].extend({ minLength: validator.Length });
                                setCss = true;
                                setValueUpdate = true;
                                setBinding = true;
                                break;
                            case 2:
                                var observable = viewModel[memberValidator.Fieldname]; // pattern validation is applicable to strings only
                                if (observable == undefined || typeof observable != "function")
                                    return true; // no observable view model property found
                                if (validator.hasOwnProperty("IsDateTime") && validator.IsDateTime) {
                                    // use shadowDate
                                    viewModel[memberValidator.Fieldname].extend({
                                        shadowDate: {
                                            format: validator.Format,
                                            pattern: validator.Expression
                                        }
                                    });
                                    var computedName = memberValidator.Fieldname + ".shadowDate";
                                    binding = (binding != undefined && binding !== "")
                                        ? binding.replace(memberValidator.Fieldname, computedName)
                                        : computedName;
                                }
                                else {
                                    // use simple pattern validation
                                    observable.extend({ pattern: validator.Expression });
                                    binding = binding || memberValidator.Fieldname;
                                }
                                setCss = true;
                                setValueUpdate = true;
                                setBinding = true;
                                break;
                            case 3:
                                if (validator.BoolValue === true)
                                    field.attr("readonly", "readonly");
                                break;
                        }
                        return true; // continue
                    });
                    // ReSharper disable ConditionIsAlwaysConst
                    if (setValueUpdate)
                        binding += ", valueUpdate: 'afterkeydown'";
                    if (setCss)
                        binding += ", css: { 'invalid': " + memberValidator.Fieldname + ".hasError }";
                    if (setBinding)
                        field.attr("data-bind", binding);
                    // ReSharper restore ConditionIsAlwaysConst
                    return true; // continue fields loop
                });
            }
            // ReSharper restore HeuristicallyUnreachableCode
        });
    };
    DialogCommandManagerItem.prototype.CheckValidationState = function () {
        if (this.validators == undefined || this.validators.length == 0) {
            this.validationState(false);
            return true;
        }
        var manager = this;
        var hasError = false;
        $.each(this.validators, function (index, value) {
            if (!value.HasError)
                return true; // continue
            manager.ValidationHasError = true;
            return false;
        });
        // ReSharper disable ExpressionIsAlwaysConst
        this.validationState(hasError);
        return hasError;
        // ReSharper restore ExpressionIsAlwaysConst
    };
    DialogCommandManagerItem.prototype.Clear = function () {
        if (!this.ControllerIsAlive()) {
            this.Parent.HandleAjaxErrorImpl(401);
            return;
        }
        var clear = this.ClearCommand || this.clearViewModelFallback;
        clear.apply(this);
        this.currentOid = EmptyGuid;
        this.isEnabled(false);
    };
    DialogCommandManagerItem.prototype.ControllerIsAlive = function () {
        var _this = this;
        var isAlive = false;
        var endpoint = this.ServiceEndpoint + "/IsAlive";
        $.ajax({
            url: endpoint,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            type: "GET",
            async: false,
            cache: false
        })
            .done(function (result) {
            isAlive = result.Success != undefined && result.Success;
            console.log("IsAlive succeeded");
        })
            .fail(function (xhr) {
            isAlive = false;
            _this.Parent.HandleAjaxError(xhr);
        });
        // ReSharper disable once ExpressionIsAlwaysConst
        return isAlive;
    };
    DialogCommandManagerItem.prototype.Delete = function () {
        if (!this.ControllerIsAlive()) {
            this.Parent.HandleAjaxErrorImpl(401);
            return;
        }
        var options = {
            Oid: this.currentOid
        };
        if (this.CanDelete)
            this.DeleteCommand.apply(options);
    };
    DialogCommandManagerItem.prototype.FinishDeleted = function () {
        this.List.RemoveItem(this.currentOid);
        this.isModified = false;
        this.isNew = false;
        this.currentOid = this.List.SelectedItem["Oid"];
        if (this.currentOid != undefined)
            setTimeout(this.List.Select(this.currentOid, true), 10);
    };
    DialogCommandManagerItem.prototype.FillViewModel = function (data) {
        if (this.FillViewModelCommand == undefined)
            this.fillViewModelFallback(this.viewModel, data);
        else
            this.FillViewModelCommand(data);
    };
    DialogCommandManagerItem.prototype.GetSessionValue = function (name) {
        return this.Parent.GetSessionValue(name);
    };
    // ReSharper disable InconsistentNaming
    DialogCommandManagerItem.prototype.IndexChanged = function (args) {
        var options = arguments[0].length > 0 ? args[0] : args;
        if (options == undefined || this.tableName !== options.tableName)
            return; // ignore event, if tableName does not match
        if (this.list != undefined)
            this.list.IndexChanged(options);
    };
    // ReSharper restore InconsistentNaming
    DialogCommandManagerItem.prototype.Initialize = function () {
        if (this.InitializeCommand == undefined)
            return;
        this.InitializeCommand.apply(this);
    };
    DialogCommandManagerItem.prototype.Query = function (oid, ignoreModified) {
        var _this = this;
        if (this.isEmptyGuid(oid))
            return;
        this.currentOid = oid;
        var instance = this;
        var options = {
            "key": oid,
            "includeRelations": window.orgAnice.IsTouch
        };
        $.ajax({
            url: this.ServiceEndpoint + this.QueryAction,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: options,
            type: "GET",
            cache: false
        })
            .done(function (result) {
            if (result.hasOwnProperty("Success")) {
                var success = result.Success;
                if (!success) {
                    instance.Clear();
                    return;
                }
            }
            instance.FillViewModel(result.Data);
            instance.IsModified = false;
            instance.isNew = false;
            instance.isEnabled(true);
            if (instance.ViewModel.hasOwnProperty("Relations") && result.Relations != undefined && typeof result.Relations == "object") {
                var relations = [];
                $.each(result.Relations, function (index, value) {
                    relations.push(new RelationsResult(value));
                });
                var model = instance.ViewModel["Relations"];
                if (model != undefined && relations.length > 0)
                    model(relations);
                instance.onRelationsChanged(relations);
            }
            _this.QuerySucceeded.trigger(_this.name);
        })
            .fail(function (xhr) {
            instance.Parent.HandleAjaxError(xhr);
            window.location.assign("/Account/LogOff");
            //$(document).trigger("dialog_query_failed", { "id": oid, "status": textStatus, "error": error, "instance": instance.Name });
            _this.QueryFailed.trigger(instance.name);
        });
        this.QuerySent.trigger(this.name);
    };
    DialogCommandManagerItem.prototype.QueryIndexes = function () {
        var control = this.Parent.SearchControl;
        if (control == undefined)
            return;
        // the 1st call to ServiceEndpoint will set the private field
        if (this.ServiceEndpoint == undefined || typeof this.serviceEndpoint != "string" || this.serviceEndpoint == "") {
            control.ViewModel.Indexes([{ DisplayName: "???", Expression: "", Name: undefined, Title: undefined }]);
            return;
        }
        control.QueryIndexes(this.serviceEndpoint + this.IndexesAction, this.controller);
    };
    DialogCommandManagerItem.prototype.Restore = function () {
        if (!this.ControllerIsAlive()) {
            window.location.href = $.cookie("LoginUrl") + "?reason=SessionExpired";
            return false;
        }
        var command = this.RestoreCommand || this.restoreViewModelFallback;
        command.apply(this);
        this.IsModified = this.isNew = false;
        return true;
    };
    DialogCommandManagerItem.prototype.Save = function (validate) {
        if (!this.ControllerIsAlive()) {
            this.Parent.HandleAjaxErrorImpl(401);
            return;
        }
        var options = {};
        if (this.CanSave)
            this.SaveCommand.apply(options);
    };
    DialogCommandManagerItem.prototype.SaveIfModified = function (validate) {
        if (!this.IsModified)
            return true;
        var doSave = true;
        if (this.Parent.WarnUnsavedDialog != undefined) {
            doSave = this.Parent.WarnUnsavedDialog.apply(this);
        }
        if (doSave)
            this.Save(validate);
        else
            this.Restore();
        return doSave;
    };
    DialogCommandManagerItem.prototype.Search = function (options) {
        if (this.list != undefined) {
            this.list.Search(options);
        }
    };
    DialogCommandManagerItem.prototype.SetList = function (list) {
        this.list = list;
        list.DialogManager = this;
    };
    DialogCommandManagerItem.prototype.SetSessionValue = function (name, value) {
        if (this.Parent.IsSessionStorageSupported) {
            window.sessionStorage.setItem(name.toLowerCase(), value);
        }
        else {
            $.cookie(DialogCommandManager.StorageTag + name.toLowerCase(), value);
        }
    };
    DialogCommandManagerItem.prototype.SuspendPropertyChanged = function () { this.isPropertyChangedSuspended = true; };
    /// <summary>Returns this instance' properties as a plain object. </summary>
    /// <returns>This instance' properties converted to a plain model. </returns>
    DialogCommandManagerItem.prototype.ToViewModel = function () {
        return {
            name: this.name,
            currentOid: this.currentOid,
            canSave: this.CanSave ? "true" : "false",
            canDelete: this.CanDelete ? "true" : "false",
            viewModel: this.viewModel
        };
    };
    DialogCommandManagerItem.prototype.Validate = function () {
        if (this.viewModel == undefined || !this.viewModel.hasOwnProperty("Validations"))
            return true;
        var valid = true;
        $.each(this.viewModel["Validations"], function (name, value) {
            if (value() === true) {
                valid = false;
                return false;
            }
            return true;
        });
        // ReSharper disable once ExpressionIsAlwaysConst
        return valid;
    };
    DialogCommandManagerItem.prototype.ResumePropertyChanged = function () { this.isPropertyChangedSuspended = false; };
    // --- Implementation -------------------------------------------------
    DialogCommandManagerItem.prototype.clearViewModel = function () {
        if (this.ClearCommand == undefined) {
            this.clearViewModelFallback(this.viewModel);
        }
        else {
            this.ClearCommand(this.viewModel);
        }
        this.IsModified = false;
        this.isNew = false;
    };
    DialogCommandManagerItem.prototype.clearViewModelFallback = function (viewModel) {
        viewModel = viewModel || this.viewModel;
        if (viewModel == undefined)
            return;
        this.isFillMode = true;
        this.resetViewModel(viewModel);
        this.isFillMode = false;
        //var target = this.ServiceEndpoint + "/GetNewRecord";
        //$.ajax({
        //  url: target,
        //  contentType: "application/json; charset=utf-8",
        //  dataType: "json",
        //  data: null,
        //  type: "GET",
        //  async: true,
        //  cache: true
        // })
        // .done((result) => {
        //  if (result.Success) {
        //   this.FillViewModel(result.Data);
        //  } else {
        //   this.resetViewModel(viewModel);
        //  }
        //  this.isFillMode = false;
        // })
        // .fail(() => {
        //  // simply reset all fields
        //  this.resetViewModel(viewModel);
        //  this.isFillMode = false;
        // });
    };
    DialogCommandManagerItem.prototype.resetViewModel = function (viewModel) {
        $.each(viewModel, function (propName, prop) {
            if (propName == "Documents")
                return true;
            if (propName == "Validations")
                return true;
            if (prop.hasWriteFunction == undefined || prop.hasWriteFunction) {
                switch (typeof prop()) {
                    case "boolean":
                        prop(false);
                        break;
                    case "number":
                        prop(0);
                        break;
                    case "string":
                        prop("");
                        break;
                }
            }
            return true;
        });
    };
    DialogCommandManagerItem.prototype.fillViewModelFallback = function (viewModel, data) {
        viewModel = viewModel || this.viewModel;
        if (viewModel == undefined || data == undefined)
            return;
        this.isFillMode = true;
        $.each(viewModel, function (name, func) {
            var lname = name.toLowerCase();
            if (lname == "relations" || lname == "validations" || lname == "documents")
                return true; // keep loop running
            if (data.hasOwnProperty(name) && viewModel.hasOwnProperty(name)) {
                func(data[name]);
            }
            return true; // keep loop running
        });
        this.isFillMode = false;
    };
    DialogCommandManagerItem.prototype.hasIgnoreFocusLost = function (element) {
        var je = $(element);
        var navItem = RibbonManager.GetNavItem(je);
        return navItem != undefined && navItem.IgnoreFocusLost;
    };
    DialogCommandManagerItem.prototype.isEmptyGuid = function (oid) {
        return (oid == undefined || oid == "" || oid.indexOf("00000000-0000-0000-0000-000000000000") != -1);
    };
    DialogCommandManagerItem.prototype.onListPropertyChanged = function (caller, args) {
        //var dialog = this;
    };
    DialogCommandManagerItem.prototype.onPropertyChanged = function (propertyName) {
        if (this.isPropertyChangedSuspended)
            return;
        this.PropertyChanged.trigger(propertyName);
    };
    DialogCommandManagerItem.prototype.onRelationsChanged = function (relations) {
        var args = {
            Controller: this.controller,
            Relations: relations
        };
        this.RelationsChanged.trigger(args);
    };
    DialogCommandManagerItem.prototype.onValidationStateChanged = function (value) {
        this.ValidationStateChanged.trigger(this.self, value);
    };
    DialogCommandManagerItem.prototype.restoreViewModelFallback = function () {
        this.Query(this.currentOid, true);
    };
    return DialogCommandManagerItem;
}());
var DialogValidator = (function () {
    function DialogValidator(errorText, parent) {
        this.dialogManagerItem = parent;
        this.errorText = errorText;
    }
    Object.defineProperty(DialogValidator.prototype, "DialogManagerItem", {
        get: function () { return this.dialogManagerItem; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogValidator.prototype, "HasError", {
        get: function () { return this.hasError; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogValidator.prototype, "ErrorText", {
        get: function () { return this.errorText; },
        enumerable: true,
        configurable: true
    });
    return DialogValidator;
}());
// set the global dialog command manager
// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function (window) {
    //if (window.orgAnice.dialogManager == undefined) {
    //	window.orgAnice.dialogManager = new DialogCommandManager(() => confirm("Möchten Sie den geänderten Datensatz speichern?"));
    //}
})(window);
//# sourceMappingURL=orgAnice.dialogManager.js.map