﻿
///<reference path="orgAnice.dialogManager.d.ts"/>

class DialogCommandManager implements IDialogCommandManager {

    // --- Events ---------------------------------------------------------
	public DialogAdded: orgAniceModule.orgAniceEvent<IDialogCommandManagerItem> = new orgAniceModule.orgAniceEvent<IDialogCommandManagerItem>();

	public QueryFailed: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
	public QuerySent: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
	public QuerySucceeded: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

    public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
    public Ready: orgAniceModule.orgAniceEvent<void> = new orgAniceModule.orgAniceEvent<void>();
    public SelectionChanged: orgAniceModule.orgAniceEvent<IDialogCommandManagerItem> = new orgAniceModule.orgAniceEvent<IDialogCommandManagerItem>();

    // --- Fields ---------------------------------------------------------
    public static StorageTag: string = "__";
    // --- Fields ---------------------------------------------------------
    private currentDialog: IDialogCommandManagerItem = undefined;
    private currentIsModified: boolean = false;
    private currentName: string = "unknown";
    private dialogs: { [index: string]: IDialogCommandManagerItem } = {};
    private isQueryOnAddDialog: boolean;
	private isReadyFired: boolean;
	private isStorageSupported: boolean;
    private isWarnUnsavedChanged: boolean = true;
	private searchControl: ISearchControl;
	private warnUnsavedDialog: () => boolean;

    // --- Properties -----------------------------------------------------
	public static EmptyRequestGuid: string = "00000000-0000-0000-0000-000000000001";

	public get IsQueryOnAddDialog(): boolean {
        return this.isQueryOnAddDialog;
    }

    public set IsQueryOnAddDialog(value: boolean) {
        if (value === this.isQueryOnAddDialog) return;
        this.isQueryOnAddDialog = value;
        this.onPropertyChanged("IsQueryOnAddDialog");
	}

	public get IsSessionStorageSupported(): boolean {
		return this.isSessionStorageSupported;
	}

    public get SearchControl(): ISearchControl {
        return this.searchControl;
    }

    public set SearchControl(searchControl: ISearchControl) {
        this.searchControl = searchControl;
	}

	public get SessionId(): string {
		var session = this.GetSessionValue("__session__");
		if (session == undefined) {
			session = moment().valueOf().toString();
			this.SetSessionValue("__session__", session);
		}
		return session;
	}

	public get WarnUnsavedDialog(): () => boolean { return this.warnUnsavedDialog; }
	public set WarnUnsavedDialog(dialog: () => boolean) { this.warnUnsavedDialog = dialog; }

    // --- Methods --------------------------------------------------------
	public AddDialogItem(dialog: IDialogCommandManagerItem) {
        this.dialogs[dialog.Name] = dialog;
        dialog.Parent = this;

		dialog.PropertyChanged.add((name: string) => {
			this.onPropertyChanged(name);
		});
		dialog.QueryFailed.add((name: string) => {
			this.QueryFailed.trigger(name);
		});
		dialog.QuerySent.add((name: string) => {
			this.QuerySent.trigger(name);
		});
		dialog.QuerySucceeded.add((name: string) => {
			this.QuerySucceeded.trigger(name);
		});

		if (this.searchControl != undefined) {
			this.searchControl.IndexChanged.add((args: any) => {
				var options: IIndexChangedEventArgs = Array.isArray(args) ? args[0] : args;
				if (dialog.Controller != options.Controller) return;
				dialog.IndexChanged(options);
			});
		}

		this.DialogAdded.trigger(this, dialog);

	    if (this.isQueryOnAddDialog)
	        dialog.Query("00000000-0000-0000-0000-000000000001");    // query for current entry
	}

	/// <summary>This command calls the ClearCommand of the current dialog.</summary>
	public Clear(name?: string) {
		if (!this.validateSavedModifications) return;

		try {
			var dialog = (name != undefined && this.dialogs.hasOwnProperty(name))
				? this.dialogs[name]
				: this.currentDialog;

			if (dialog == undefined) return;

			dialog.Clear();
		} catch (exception) {
		}
	}

	public get CurrentDialog(): IDialogCommandManagerItem {
		if (this.currentDialog != undefined)
			return this.currentDialog;

		if (this.currentName != undefined && this.currentName != "unknown")
			this.Select(this.currentName);

		var keys = Object.keys(this.dialogs);
		if (keys.length > 0)
			this.Select(keys[0]);
//			this.currentDialog = this.dialogs[keys[0]];

		return this.currentDialog;
	}

	public GetDialogByName(name: string): IDialogCommandManagerItem {
		if (this.dialogs.hasOwnProperty(name))
			return this.dialogs[name];

		return undefined;
	}

	public GetSessionValue(name: string): string {
		if (this.isSessionStorageSupported) {
			return window.sessionStorage.getItem(name.toLowerCase());
		} else {
			return $.cookie(window.orgAnice.MakeStorageKey(name));
		}
	}

	public HandleAjaxError(xhr: JQueryXHR): void {
		if (xhr == undefined) {
			this.HandleAjaxErrorImpl(0);
			return;
		}

		this.HandleAjaxErrorImpl(xhr.status);
	}

	public HandleAjaxErrorImpl(reason: any): void {
		var url = "/";
		var domain = $.cookie("orgAniceDomain");
		if (domain != undefined && domain !== "" && domain != "local")
			url += domain + "/";
		url += $.cookie("orgAniceProvider");

		if (reason != undefined) {
			if (typeof reason == "string") {
				url += "?reason=" + reason;
			}
			else if (typeof reason == "number") {
				switch (reason) {
					case 401:	// unauthorized (expired)
						url += "?reason=SessionExpired";
						break;

					case 0:
						url += "?reason=InternalError";
						break;

					default:
						url += "?reason=SessionExpired";
						break;
				}
			}
		}
		window.location.assign(url);
	}

	public IgnoreModified() { this.isCurrentModified = false; }

	public get IsCurrentModified(): boolean {
		if (this.currentDialog == undefined) return false;
		return this.currentDialog.IsModified;
	}

	public get IsModified(): boolean {
		var result = false;
		$.each(this.dialogs, (index, dialog) => {
			if (dialog.isModified)
				result = result || true;
		});
		// ReSharper disable once ExpressionIsAlwaysConst
		return result;
	}

	public set IsModified(value: boolean) {
		$.each(this.dialogs, (index, dialog) => {
			dialog.isModified = false;
			if (this.currentDialog != undefined)
				this.currentDialog.IsModified = value;
		});
	}

	public OnQuerySucceeded(dialog: string): void {
		this.QuerySucceeded.trigger(dialog);
	}

	public Query(id: string) {
		if (this.currentDialog == undefined || this.currentDialog == null) return;

		this.currentDialog.Query(id);
	}

	public QueryDialog(name: string, id: string) {
		var dialog = this.dialogs.hasOwnProperty(name) ? this.dialogs[name] : undefined;

		if (dialog != undefined)
			dialog.Query(id);
    }

    public RequeryAll(exceptDialog: string): void {
    }

	public Restore(): void {
		if (this.CurrentDialog != undefined)
			this.CurrentDialog.Restore();

		this.currentIsModified = false;
		this.onPropertyChanged("IsModified");
	}

	public Save(validate?: boolean): void {
		try {
			var d = this.currentDialog;

			//TODO: this should re-direct to the error page!
			if (d == undefined)
				return;

			this.currentDialog.Save(validate);
		} catch (exception) {
		}
	}

	public SaveIfModified(validate?: boolean): boolean {
		if (!this.IsModified || this.currentDialog == undefined) return true;

		return this.currentDialog.SaveIfModified(validate);
	}

	public Select(name: any): IDialogCommandManagerItem {
	    if (typeof name == "number") {	// index was passed
			if (name >= Object.keys(this.dialogs).length) return null;
	        const targetDialog = Object.keys(this.dialogs)[name];
			if (targetDialog == undefined)
				return null;

			return this.Select(targetDialog);
		}

		if (this.currentDialog != undefined && this.currentDialog != null) {
			if (this.currentDialog.Name === name)
				return this.currentDialog;

			if (this.currentDialog.IsModified || this.currentDialog.IsNew) {
				if (this.warnUnsavedDialog != undefined) {
					if (this.warnUnsavedDialog.apply(null)) {
						this.currentDialog.Save();
					} else {
						this.currentDialog.Restore();
					}
				}
			}
		}

	    let caption: JQuery;
		if (this.currentName != undefined && this.currentName !== "") {
			caption = $(`#${this.currentName} .dialog`);
			if (caption != undefined)
				caption.removeClass("active");
		}

		this.currentName = name;

		caption = $(`#${this.currentName} .dialog`);
		if (caption != undefined)
			caption.addClass("active");

		this.currentDialog = this.dialogs[name];	// set new dialog to current
		if (this.currentDialog != undefined) {		// was just activated, so ->
			this.currentDialog.IsModified = false;	// set IsModified & IsNew to false
			this.currentDialog.IsNew = false;
		}
		this.onSelectionChanged(this.currentDialog);

        if (this.currentDialog != undefined && this.searchControl != undefined)
	        this.currentDialog.QueryIndexes();

		return this.currentDialog;
	}

	public SelectByList(list: IListControl): boolean {

		var selector = this.CurrentDialog;
		if (selector.List != undefined && selector.List.Name === list.Name)
			return true;

		$.each(this.dialogs, (index: number, value: IDialogCommandManagerItem) => {
			if (value.List != undefined && value.List.Name === list.Name) {
				selector = value;
				return false;
			}
			return true;
		});

		if (selector.List != undefined && selector.List.Name === list.Name) {
			this.Select(selector);
			return true;
		}

		return false;
	}

	public SetModified() { this.isCurrentModified = true; }

	public SetSessionValue(name: string, value: string): void {
		if (this.isSessionStorageSupported) {
			window.sessionStorage.setItem(name.toLowerCase(), value);
		} else {
			// if sessionStorage is not supported by the browser, we use a cookie instead.
			// BUT! This cookie must be qualified somehow, as there might be many sessions open at least on a desktop!
			$.cookie(window.orgAnice.MakeStorageKey(name), value, { path: "/" });
		}
	}

	public Validate(): boolean {
		if (this.currentDialog == undefined) return true;
		return this.currentDialog.Validate();
	}

	public ValidateSessionIsAlive(): boolean {
		var isAlive = false;

		$.ajax({
				url: "/Account/IsAlive",
				contentType: "application/json; charset=utf-8",
				dataType: "json",
				data: "",
				type: "GET",
				async: false,
				cache: false
			})
			.done((result) => {
				isAlive = result.Success != undefined && result.Success;
			})
			.fail(() => {
				isAlive = false;
			});

		// ReSharper disable once ExpressionIsAlwaysConst
		return isAlive;
	}

	// --- Commands -------------------------------------------------------
    // --- Constructor ----------------------------------------------------
	constructor(unsavedDialog: () => boolean) {
		this.warnUnsavedDialog = unsavedDialog;

		setTimeout(() => {
			
		}, 5000);

		this.onReady();
	}

    // --- Implementation -------------------------------------------------
	private set isCurrentModified(value: boolean) {
		this.currentIsModified = value;

		this.onPropertyChanged("IsModified");
	}

	private get isSessionStorageSupported(): boolean {
		if (this.isStorageSupported != undefined)
			return this.isStorageSupported;

		try {
			window.sessionStorage.setItem("Test", "Test");
			window.sessionStorage.removeItem("Test");
			this.isStorageSupported = true;
			return true;
		} catch (e) {
			this.isStorageSupported = false;
			return false;
		}
	}

	private onPropertyChanged(propertyName: string): void {
		propertyName = Array.isArray(propertyName) ? propertyName[0] : propertyName;
		this.PropertyChanged.trigger(propertyName);
	}

	private onReady() {
		this.isReadyFired = true;
		this.Ready.FireOnAdd(this.isReadyFired);
		this.Ready.trigger("");
	}

	private onSelectionChanged(dialog: IDialogCommandManagerItem) {
		this.SelectionChanged.trigger(dialog);
	}

	private get validateSavedModifications() {
		return (this.currentDialog == undefined ||
			this.currentDialog == null ||
			!(this.currentDialog.IsModified || this.currentDialog.IsNew));
	}
}

class DialogCommandManagerItem implements IDialogCommandManagerItem {

	// --- Events ---------------------------------------------------------
	public QueryFailed: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
	public QuerySent: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
	public QuerySucceeded: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	public OnClearCalled: IEvent = new TypedEvent();

	public OnDeleted: IEvent = new TypedEvent();

	public OnQuery: IEvent = new TypedEvent();

	public OnSaved: IEvent = new TypedEvent();

	public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	public RelationsChanged: orgAniceModule.orgAniceEvent<IRelationsChangedEventArgs> = new orgAniceModule.orgAniceEvent<IRelationsChangedEventArgs>();

	public ValidationStateChanged: orgAniceModule.orgAniceEvent<boolean> = new orgAniceModule.orgAniceEvent<boolean>();

	// --- Fields ---------------------------------------------------------
    public get DeleteAction(): string { return "/Delete"; }
    public get IndexesAction(): string { return "/Indexes"; }
    public get QueryAction(): string { return "/GetEntry"; }
    public get QueryListAction(): string { return "/GetList"; }
	public get QueryListShortcutsAction(): string { return "/GetListShortcuts"; }
	public get SaveAction(): string { return "/Save"; }

	private canCreate: boolean = true;
	private canDelete: boolean = true;
	private canSave: boolean = true;
	private controller: string;
	private currentOid: string = "00000000-0000-0000-0000-000000000000";
	private isEnabled: KnockoutObservable<boolean> = ko.observable<boolean>(false);
	private isFillMode: boolean = false;
	private isModified: boolean = false;
	private isNew: boolean = false;
	private isPropertyChangedSuspended: boolean = false;
	private isReadonly: boolean;
	private list: IListControl;
    private name: string;
	private parent: IDialogCommandManager;
	private self: IDialogCommandManagerItem;
	private serviceEndpoint: string;
	private toplevel: JQuery;
	private typeValidator: IOrgDataTypeValidator;
	private validators: IDialogValidator[] = [];
	private validationState: KnockoutObservable<boolean> = ko.observable<boolean>(false);
	private viewModel: {}

	// fields for search control
	private activeIndex: string;
	private indexes: string[];
	private tableName: string;

	// --- Properties -----------------------------------------------------

	public get CanCreate(): boolean { return this.canCreate; }

	public set CanCreate(value: boolean) {
		if (this.canCreate == value) return;
		this.canCreate = value;
		this.onPropertyChanged("CanCreate");
	}

	/// <summary>Returns a value indicating, whether this instance can delete its <see cref="ViewModel"/>. </summary>
	/// <returns><c>true</c>, if values can be deleted; otherwise, <c>false</c>. </returns>
	public get CanDelete() {
		return this.canDelete && this.DeleteCommand != undefined;
	}

	public set CanDelete(value: boolean) {
		if (this.canDelete === value) return;
		this.canDelete = value;
		this.onPropertyChanged("CanDelete");
	}

	/// <summary>Returns a value indicating, whether this instance can save its <see cref="ViewModel"/>. </summary>
	/// <returns><c>true</c>, if values can be deleted; otherwise, <c>false</c>. </returns>
	public get CanSave() {
		return this.canSave && this.SaveCommand != undefined && !this.validationState();
	}

	public set CanSave(value: boolean) {
		if (this.canSave === value) return;
		this.canSave = value;
		this.onPropertyChanged("CanSave");
	}

	public get Controller(): string { return this.controller; }

	/// <summary>Returns the OID of the current data object, which will be set by the Query method.< / summary >
	/// <returns>The OID of the current data object.</returns>
	public get CurrentOid() { return this.currentOid; }

	public set CurrentOid(value: string) { this.currentOid = value; }

	public get IsEnabledObserved(): KnockoutObservable<boolean> { return this.isEnabled; }

	public get IsEnabled(): boolean { return this.isEnabled(); }

	public set IsEnabled(value: boolean) {
		if (this.isEnabled() === value) return;
		this.isEnabled(value);
		this.onPropertyChanged("IsEnabled");
	}

	public get IsFillMode(): boolean { return this.isFillMode; }

	public set IsFillMode(value: boolean) {
		if (this.isFillMode === value) return;
		this.isFillMode = value;
		this.onPropertyChanged("IsFillMode");
	}

	public get IsModified() { return this.isModified; }

	public set IsModified(value: boolean) {
		if (this.isModified === value) return;
		this.isModified = value;
		this.onPropertyChanged("IsModified");
	}

	public get IsNew() { return this.isNew; }

	public set IsNew(value: boolean) { this.isNew = value; }

	public get IsReadonly() { return this.isReadonly; }

	public set IsReadonly(value: boolean) { this.isReadonly = value; }

	public get IsSelected(): boolean {
		return (this.Parent.CurrentDialog.Name == this.Name);
	}

	public get List(): IListControl { return this.list; }

	public set List(value: IListControl) {
		if (this.list != undefined) {
			this.list.PropertyChanged.remove(this.onListPropertyChanged);
		}
		this.list = value;
		this.list.PropertyChanged.add(this.onListPropertyChanged);

		this.onPropertyChanged("List");
	}

    public get Name() { return this.name; }

    public get Parent(): IDialogCommandManager { return this.parent || window.orgAnice.dialogManager; }

    public set Parent(parent: IDialogCommandManager) { this.parent = parent; }

	public get ServiceEndpoint() {
		if (this.serviceEndpoint == undefined) {
			this.serviceEndpoint = document.location.protocol + "//" + document.location.host + "/" + this.controller;
		}
		return this.serviceEndpoint;
	}

    public set ServiceEndpoint(value: string) {
        if (this.ServiceEndpoint === value) return;
        this.serviceEndpoint = value;
	    this.onPropertyChanged("ServiceEndpoint");
	}

	public get TypeValidator(): IOrgDataTypeValidator { return this.typeValidator; }

	public set TypeValidator(value: IOrgDataTypeValidator) { this.typeValidator = value; }

	public get Validators(): IDialogValidator[] {
		return this.validators;
	}

	public get ValidationHasError(): boolean {
		return this.validationState();
	}

	public set ValidationHasError(value: boolean) {
		if (this.validationState() == value) return;
		this.validationState(value);
	}

	public get ViewModel() { return this.viewModel; }

	public set ViewModel(value: {}) { this.viewModel = value; }

	// --- Methods --------------------------------------------------------
	public ApplyValidations(): void {
		if (this.viewModel == undefined || this.typeValidator == undefined) return;

		var dialogItem = this;
		var viewModel = this.viewModel;
		var typeValidator = this.typeValidator;

		$.each(viewModel, (index: string, value: KnockoutObservableStatic) => {

			var memberValidator: IOrgDataMemberValidator = undefined;

			$.each(typeValidator.MemberValidators, (mindex: number, mvalue: IOrgDataMemberValidator) => {
				if (mvalue.Fieldname !== index) return true; // continue

				memberValidator = mvalue;
				return false; // leave loop
			});

			// ReSharper disable once ConditionIsAlwaysConst
			// ReSharper disable HeuristicallyUnreachableCode
			// ReSharper does not detect that memberValidator was set in the previous $.each() loop
			if (memberValidator != undefined) {

				var fields = $("#" + dialogItem.Name + " #" + memberValidator.Fieldname);
				if (fields == undefined || fields.length == 0) return true;	// continue main loop

				$.each(fields, (findex: number, hfield: HTMLElement) => {

					var field = $(hfield);
					if (field == undefined) return true;	// continue fields loop

					var binding = field.attr("data-bind"),
						setBinding = false,
						setCss = false,
						setValueUpdate = false;

					$.each(memberValidator.Validators, (idx, validator: IOrgDataValdator) => {

						switch (validator.ValidationType) {
						case 0: // max length
							if (validator.Length > 0)
								field.attr("maxlength", validator.Length);
							break;
							case 1:	// min length
								if (validator.Length <= 0) break;
								viewModel[memberValidator.Fieldname].extend({ minLength: validator.Length });
								setCss = true;
								setValueUpdate = true;
								setBinding = true;
							break;
						case 2: // pattern validation
							var observable: KnockoutObservable<string> = viewModel[memberValidator.Fieldname]; // pattern validation is applicable to strings only
							if (observable == undefined || typeof observable != "function") return true; // no observable view model property found


							if (validator.hasOwnProperty("IsDateTime") && validator.IsDateTime) {
								// use shadowDate
								viewModel[memberValidator.Fieldname].extend({
									shadowDate: {
										format: validator.Format,
										pattern: validator.Expression
									}
								});

								var computedName = memberValidator.Fieldname + ".shadowDate";
								binding = (binding != undefined && binding !== "")
									? binding.replace(memberValidator.Fieldname, computedName)
									: computedName;

							} else {
								// use simple pattern validation
								observable.extend({ pattern: validator.Expression });

								binding = binding || memberValidator.Fieldname;
							}
							setCss = true;
							setValueUpdate = true;
							setBinding = true;

							break;
						case 3: // readonly field
							if (validator.BoolValue === true)
								field.attr("readonly", "readonly");
							break;
						}
						return true;	// continue
					});
					// ReSharper disable ConditionIsAlwaysConst
					if (setValueUpdate)
						binding += ", valueUpdate: 'afterkeydown'";
					if (setCss)
						binding += ", css: { 'invalid': " + memberValidator.Fieldname + ".hasError }";
					if (setBinding)
						field.attr("data-bind", binding);
					// ReSharper restore ConditionIsAlwaysConst

					return true;	// continue fields loop
				});
			}
			// ReSharper restore HeuristicallyUnreachableCode
		});
	}

	public CheckValidationState(): boolean {
		if (this.validators == undefined || this.validators.length == 0) {
			this.validationState(false);
			return true;
		}

		var manager = this;
		var hasError = false;
		$.each(this.validators, (index: number, value: IDialogValidator) => {
			if (!value.HasError) return true; // continue

			manager.ValidationHasError = true;
			return false;
		});

		// ReSharper disable ExpressionIsAlwaysConst
		this.validationState(hasError);
		return hasError;
		// ReSharper restore ExpressionIsAlwaysConst
	}

	public Clear() {
		if (!this.ControllerIsAlive()) {
			this.Parent.HandleAjaxErrorImpl(401);
			return;
		}

		var clear = this.ClearCommand || this.clearViewModelFallback;

		clear.apply(this);

		this.currentOid = EmptyGuid;
		this.isEnabled(false);
	}

	public ControllerIsAlive(): boolean {
		var isAlive = false;
		var endpoint = this.ServiceEndpoint + "/IsAlive";

		$.ajax({
				url: endpoint,
				contentType: "application/json; charset=utf-8",
				dataType: "json",
				type: "GET",
				async: false,
				cache: false
			})
			.done((result) => {
				isAlive = result.Success != undefined && result.Success;
				console.log("IsAlive succeeded");
			})
			.fail((xhr) => {
				isAlive = false;
				this.Parent.HandleAjaxError(xhr);
		});

		// ReSharper disable once ExpressionIsAlwaysConst
		return isAlive;
	}

	public Delete(): void {
		if (!this.ControllerIsAlive()) {
			this.Parent.HandleAjaxErrorImpl(401);
			return;
		}

		let options = {
			Oid: this.currentOid
		};

		if (this.CanDelete)
			this.DeleteCommand.apply(options);
	}

	public FinishDeleted(): void {
		this.List.RemoveItem(this.currentOid);
		this.isModified = false;
		this.isNew = false;

		this.currentOid = this.List.SelectedItem["Oid"];
		if (this.currentOid != undefined)
			setTimeout(this.List.Select(this.currentOid, true), 10);
	}

	public FillViewModel(data: {}): void {
		if (this.FillViewModelCommand == undefined)
			this.fillViewModelFallback(this.viewModel, data);
		else
			this.FillViewModelCommand(data);
	}

	public GetSessionValue(name: string): string {
		return this.Parent.GetSessionValue(name);
	}

	// ReSharper disable InconsistentNaming
	public IndexChanged(args: IIndexChangedEventArgs): void {
		var options = arguments[0].length > 0 ? args[0] : args;
		if (options == undefined || this.tableName !== options.tableName) return;	// ignore event, if tableName does not match

		if (this.list != undefined)
			this.list.IndexChanged(options);
	}
	// ReSharper restore InconsistentNaming
	
	public Initialize(): void {
		if (this.InitializeCommand == undefined) return;

		this.InitializeCommand.apply(this);
	}

	public Query(oid: string, ignoreModified?: boolean): void {
		if (this.isEmptyGuid(oid)) return;

		this.currentOid = oid;
		var instance = this;
	    var options = {
		    "key": oid,
		    "includeRelations": window.orgAnice.IsTouch
	    };

		$.ajax({
				url: this.ServiceEndpoint + this.QueryAction,
				contentType: "application/json; charset=utf-8",
				dataType: "json",
				data: options,
				type: "GET",
				cache: false
			})
			.done((result) => {
				if (result.hasOwnProperty("Success")) {
					var success = result.Success;
					if (!success) {
						instance.Clear();
						return;
					}
				}
				instance.FillViewModel(result.Data);
				instance.IsModified = false;
				instance.isNew = false;
				instance.isEnabled(true);

				if (instance.ViewModel.hasOwnProperty("Relations") && result.Relations != undefined && typeof result.Relations == "object") {
					var relations = [];
					$.each(result.Relations, (index, value: IRelationsResult) => {
						relations.push(new RelationsResult(value));
					});
					var model = instance.ViewModel["Relations"];
					if (model != undefined && relations.length > 0)
						model(relations);

					instance.onRelationsChanged(relations);
				}

				this.QuerySucceeded.trigger(this.name);
			})
			.fail((xhr) => {

				instance.Parent.HandleAjaxError(xhr);

				window.location.assign("/Account/LogOff");

				//$(document).trigger("dialog_query_failed", { "id": oid, "status": textStatus, "error": error, "instance": instance.Name });

				this.QueryFailed.trigger(instance.name);
			});

	    this.QuerySent.trigger(this.name);
    }

    public QueryIndexes(): void {
        var control = this.Parent.SearchControl;
        if (control == undefined) return;

		// the 1st call to ServiceEndpoint will set the private field
        if (this.ServiceEndpoint == undefined || typeof this.serviceEndpoint != "string" || this.serviceEndpoint == "") {
			control.ViewModel.Indexes([{ DisplayName: "???", Expression: "", Name: undefined, Title: undefined }]);
            return;
        }

        control.QueryIndexes(this.serviceEndpoint + this.IndexesAction, this.controller);
    }

	public Restore() {
		if (!this.ControllerIsAlive()) {
			window.location.href = $.cookie("LoginUrl") + "?reason=SessionExpired";
			return false;
		}

		var command = this.RestoreCommand || this.restoreViewModelFallback;

		command.apply(this);

		this.IsModified = this.isNew = false;

		return true;
	}

	public Save(validate?: boolean): void {
		if (!this.ControllerIsAlive()) {
			this.Parent.HandleAjaxErrorImpl(401);
			return;
		}

		var options = {};

		if (this.CanSave)
			this.SaveCommand.apply(options);
	}

	public SaveIfModified(validate?: boolean): boolean {
		if (!this.IsModified) return true;

		var doSave = true;
		if (this.Parent.WarnUnsavedDialog != undefined) {
			doSave = this.Parent.WarnUnsavedDialog.apply(this);
		}
		if (doSave)
			this.Save(validate);
		else
			this.Restore();

		return doSave;
	}

	public Search(options: ISearchArgs): void {
		if (this.list != undefined) {
			this.list.Search(options);
		}
	}

	public SetList(list: IListControl): void {
		this.list = list;
		list.DialogManager = this;
	}

	public SetSessionValue(name: string, value: string): void {
		if (this.Parent.IsSessionStorageSupported) {
			window.sessionStorage.setItem(name.toLowerCase(), value);
		} else {
			$.cookie(DialogCommandManager.StorageTag + name.toLowerCase(), value);
		}
	}

	public SuspendPropertyChanged(): void { this.isPropertyChangedSuspended = true; }

	/// <summary>Returns this instance' properties as a plain object. </summary>
	/// <returns>This instance' properties converted to a plain model. </returns>
	public ToViewModel() {
		return {
			name: this.name,
			currentOid: this.currentOid,
			canSave: this.CanSave ? "true" : "false",
			canDelete: this.CanDelete ? "true" : "false",
			viewModel: this.viewModel
		};
	}

	public Validate(): boolean {
		if (this.viewModel == undefined || !this.viewModel.hasOwnProperty("Validations")) return true;
		var valid = true;
		$.each(this.viewModel["Validations"], (name, value) => {
			if (value() === true) {
				valid = false;
				return false;
			}
			return true;
		});
		// ReSharper disable once ExpressionIsAlwaysConst
		return valid;
	}

	// --- Commands -------------------------------------------------------
	/// <summary>This command clears all fields in the dialog.</summary>
	public ClearCommand: (options: any) => void = undefined;

	/// <summary>The command which deletes the entry given by its <c>id</c>.</summary>
	public DeleteCommand: (id: string) => void = undefined;

	public FillViewModelCommand: (data: {}) => void = undefined;

	public InitializeCommand: () => void = undefined;

	/// <summary>The query command, used to query a single entry by ID.</summary>
	public QueryCommand: (id: string) => void = undefined;

	public RestoreCommand: () => void = undefined;

	public ResumePropertyChanged(): void { this.isPropertyChangedSuspended = false; }

	/// <summary>The save commands of the dialogs.</summary>
	public SaveCommand: (options: {}) => boolean = undefined;

    // --- Constructor ----------------------------------------------------
	constructor(name: string, controller: string, dataEndpoint?: string) {
		this.self = this;
		this.name = name;
		this.controller = controller;

		if (dataEndpoint != undefined && typeof dataEndpoint == "string")
			this.serviceEndpoint = dataEndpoint;

		this.validationState.subscribe((newValue) => {
			this.onValidationStateChanged(newValue);
		});

		var $this = this;
		this.toplevel = $("#" + name);
		if (this.toplevel != undefined) {
			ko.applyBindingsToNode(document.getElementById(this.name),
			{
				css: { 'disabled': $this.isEnabled }
			}, null);
		}
		//this.toplevel.focusout((ev: JQueryEventObject) => {
		//	if (!control.IsModified) return;

		//	var target = ev.relatedTarget;
		//	if (control.hasIgnoreFocusLost(target)) return;

		//	var doSave = true;
		//	if (control.Parent.WarnUnsavedDialog != undefined)
		//		doSave = control.Parent.WarnUnsavedDialog.apply(this);

		//	if (doSave)
		//		control.Save();
		//	else
		//		control.Restore();
		//});
	}

	// --- Implementation -------------------------------------------------
    private clearViewModel() {
        if (this.ClearCommand == undefined) {
            this.clearViewModelFallback(this.viewModel);
        } else {
            this.ClearCommand(this.viewModel);
        }
        this.IsModified = false;
        this.isNew = false;
    }

    private clearViewModelFallback(viewModel: {}) {
		viewModel = viewModel || this.viewModel;
		if (viewModel == undefined) return;

		this.isFillMode = true;
		this.resetViewModel(viewModel);
		this.isFillMode = false;

	    //var target = this.ServiceEndpoint + "/GetNewRecord";
	    //$.ajax({
			  //  url: target,
			  //  contentType: "application/json; charset=utf-8",
			  //  dataType: "json",
			  //  data: null,
			  //  type: "GET",
			  //  async: true,
			  //  cache: true
		   // })
		   // .done((result) => {
			  //  if (result.Success) {
				 //   this.FillViewModel(result.Data);
			  //  } else {
				 //   this.resetViewModel(viewModel);
			  //  }
			  //  this.isFillMode = false;
		   // })
		   // .fail(() => {
			  //  // simply reset all fields
			  //  this.resetViewModel(viewModel);
			  //  this.isFillMode = false;
		   // });
    }

	private resetViewModel(viewModel: {}) {
		$.each(viewModel, (propName, prop) => {
			if (propName == "Documents") return true;
			if (propName == "Validations") return true;
			if (prop.hasWriteFunction == undefined || prop.hasWriteFunction) {
				switch (typeof prop()) {
					case "boolean":
						prop(false);
						break;
					case "number":
						prop(0);
						break;
					case "string":
						prop("");
						break;
				}
			}
			return true;
		});

	}

	private fillViewModelFallback(viewModel: {}, data: {}): void {
		viewModel = viewModel || this.viewModel;
		if (viewModel == undefined || data == undefined) return;

		this.isFillMode = true;

		$.each(viewModel, (name: string, func) => {
			var lname = name.toLowerCase();
			if (lname == "relations" || lname == "validations" || lname == "documents") return true;	// keep loop running
			if (data.hasOwnProperty(name) && viewModel.hasOwnProperty(name)) {
				func(data[name]);
			}
			return true;	// keep loop running
		});

		this.isFillMode = false;
	}

	private hasIgnoreFocusLost(element: Element): boolean {
		var je = $(element);
		var navItem: IRibbonItem = RibbonManager.GetNavItem(je);
		return navItem != undefined && navItem.IgnoreFocusLost;
	}

	private isEmptyGuid(oid: string): boolean {
		return (oid == undefined || oid == "" || oid.indexOf("00000000-0000-0000-0000-000000000000") != -1);
	}

	private onListPropertyChanged(caller: any, args?: string): void {
		//var dialog = this;
	}

	private onPropertyChanged(propertyName: string): void {
		if (this.isPropertyChangedSuspended) return;
		this.PropertyChanged.trigger(propertyName);
	}

	private onRelationsChanged(relations) {
		var args: IRelationsChangedEventArgs = {
			Controller: this.controller,
			Relations: relations
		}
		this.RelationsChanged.trigger(args);
	}

	private onValidationStateChanged(value: boolean) {
		this.ValidationStateChanged.trigger(this.self, value);
	}

	private restoreViewModelFallback() {
		this.Query(this.currentOid, true);
	}
}

class DialogValidator implements IDialogValidator {
	private dialogManagerItem: IDialogCommandManagerItem;
	private hasError: boolean;
	private errorText: string;

	public get DialogManagerItem(): IDialogCommandManagerItem { return this.dialogManagerItem; }

	public get HasError(): boolean { return this.hasError; }

	public get ErrorText(): string { return this.errorText; }

	constructor(errorText: string, parent: IDialogCommandManagerItem) {
		this.dialogManagerItem = parent;
		this.errorText = errorText;
	}
}

// set the global dialog command manager
// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function (window) {

	//if (window.orgAnice.dialogManager == undefined) {
	//	window.orgAnice.dialogManager = new DialogCommandManager(() => confirm("Möchten Sie den geänderten Datensatz speichern?"));
	//}
})(window);
