///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />
/// <summary>A controller for handling frames. </summary>
/// <seealso cref="T:IFrameController"/>
/// <remarks>This controller must be attached to a DIV element which surrounds a control that pubishes
///			 an <c>invalidateLayout</c> event to its outermost element. The controller will check its
///			 immediate sibling for whether it is a <b>GridSplitter</b>, and if itar find it, attaches
///			 itself to the <b>PositionChanged</b> event. When that event is fired, the controller will
///			 trigger the <c>invalidateLayout</c> it found.
/// </remarks>
var FrameController = (function () {
    // --- Constructor ----------------------------------------------------
    function FrameController(element, gridSplitterId, options) {
        // --- Events ---------------------------------------------------------
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        if (element == undefined)
            throw new Error("No toplevel element defined!");
        this.toplevel = $(element);
        this.gridSplitter = $("#" + gridSplitterId);
        if (this.gridSplitter == undefined)
            return;
        var s = this.gridSplitter;
        var data = this.gridSplitter.gridSplitter();
        var list = this.toplevel.find(".baselist.control-frame");
        if (list == undefined || list.length == 0)
            return;
        if (data != undefined) {
            data.PositionChanged.add(function () {
                list.trigger("invalidateLayout");
            });
        }
    }
    Object.defineProperty(FrameController.prototype, "Toplevel", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.toplevel; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    FrameController.prototype.Initialize = function () {
    };
    // --- Implementation -------------------------------------------------
    FrameController.prototype.onPropertyChanged = function (propertyName) {
        this.PropertyChanged.trigger(this, propertyName);
    };
    return FrameController;
}());
// make it a jQuery plug-in
// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function ($) {
    $.fn.frameController = function (arg1, arg2, arg3) {
        var length = arguments.length;
        return this.each(function () {
            var instance = this;
            var $instance = $(instance);
            var control = $instance.data("frameController");
            switch (length) {
                case 2:
                    if (control != undefined)
                        return control;
                    control = new FrameController(instance, arg2);
                    $instance.data["framecontroller"] = control;
                    return control;
            }
            return control;
        });
    };
})(jQuery || $);
//# sourceMappingURL=orgAnice.frameController.js.map