﻿
///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />

interface IFrameController {
	// --- Events ---------------------------------------------------------
	PropertyChanged: orgAniceModule.orgAniceEvent<string>;

	// --- Properties -----------------------------------------------------
	Toplevel: JQuery;

	// --- Methods --------------------------------------------------------
	Initialize();
}

interface JQuery {
	gridSplitter: () => IGridSplitter;
}

/// <summary>A controller for handling frames. </summary>
/// <seealso cref="T:IFrameController"/>
/// <remarks>This controller must be attached to a DIV element which surrounds a control that pubishes
///			 an <c>invalidateLayout</c> event to its outermost element. The controller will check its
///			 immediate sibling for whether it is a <b>GridSplitter</b>, and if itar find it, attaches
///			 itself to the <b>PositionChanged</b> event. When that event is fired, the controller will
///			 trigger the <c>invalidateLayout</c> it found.
/// </remarks>
class FrameController implements IFrameController {
	// --- Events ---------------------------------------------------------
	public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	// --- Fields ---------------------------------------------------------
	private toplevel: JQuery;
	private dialogFrame: JQuery;
	private gridSplitter: JQuery;
	private listControl: JQuery;

	// --- Properties -----------------------------------------------------
	public get Toplevel(): JQuery { return this.toplevel; }

	// --- Methods --------------------------------------------------------
	public Initialize(): void {
		
	}

	// --- Constructor ----------------------------------------------------
	constructor(element: any, gridSplitterId: string, options?: {}) {
		if (element == undefined)
			throw new Error("No toplevel element defined!");

		this.toplevel = $(element);
		this.gridSplitter = $("#" + gridSplitterId);

		if (this.gridSplitter == undefined) return;

		var s: any = this.gridSplitter;

		var data: IGridSplitter = this.gridSplitter.gridSplitter();

		var list = this.toplevel.find(".baselist.control-frame");
		if (list == undefined || list.length == 0) return;

		if (data != undefined) {
			data.PositionChanged.add(() => {
				list.trigger("invalidateLayout");
			});
		}
	}

	// --- Implementation -------------------------------------------------
	private onPropertyChanged(propertyName: string): void {
		this.PropertyChanged.trigger(this, propertyName);
	}
}

// make it a jQuery plug-in
// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function($) {
	$.fn.frameController = function (arg1?: any, arg2?: string, arg3?: any) {
		var length = arguments.length;

		return this.each(function () {
			var instance = this;
			var $instance = $(instance);
			var control: IFrameController = $instance.data("frameController");

			switch (length) {
				case 2:
					if (control != undefined)
						return control;
					control = new FrameController(instance, arg2);
					$instance.data["framecontroller"] = control;
					return control;
			}
			return control;
		});
	}
})(jQuery || $);