///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />
var GridSplitter = (function () {
    // --- Constructor(s) -------------------------------------------------
    function GridSplitter(splitter, el1, el2, orientation, isMobile) {
        // --- Events ---------------------------------------------------------
        this.PositionChanged = new orgAniceModule.orgAniceEvent();
        this.element1MinHeight = 105; // the minimum height of the 1st element
        this.element1MinWidth = 105; // the minimum width of the 1st element
        this.element2MinHeight = 40; // the minimum height of the 1st element
        this.element2MinWidth = 40; // the minimum width of the 1st element
        this.splitterThickness = 4; // the thickness (height or width) of the splitter element, NOT the splitter's handle
        // --- public fields --------------------------------------------------
        this.Splitterwidth = 3;
        this.ColumnMinWidthDesktop = 20;
        this.isMobile = isMobile;
        this.isActive = false;
        this.splitter = splitter;
        this.id = splitter.attr("id");
        this.workspace = this.id.split("_")[0];
        this.el1 = el1;
        this.el2 = el2;
        var control = this;
        if (isMobile) {
            $(window).resize(function () { return control.onResizeMobile(); });
        }
        else {
            this.orientation = orientation;
            $(window).resize(function () { return control.onResizeDesktop(); });
        }
        this.init();
        this.attachEvents(el1, el2);
    }
    // --- Methods --------------------------------------------------------
    GridSplitter.prototype.ResizeNext = function (splitter, el1, el2, event, initialLength) {
        //if (el2.length == 0) {
        //    return;
        //}
        var index = initialLength - (el2.length - 1);
        if (event.pageX >= ($(window).width() - (this.ColumnMinWidthDesktop * initialLength) - 10)) {
            return;
        }
        var firstColumnWidth = event.pageX - el1.offset().left;
        el1.width(firstColumnWidth);
        el1.css("max-width", firstColumnWidth);
        el1.children("div").css("width", firstColumnWidth);
        var lastcolumn = $(el2[el2.length - 1]);
        if (lastcolumn.offset() != undefined &&
            (lastcolumn.offset().left + lastcolumn.width()) >= $(window).width() - 10 - (20 * (index - 1))) {
            var lastColumnWidth = $(window).width() - event.pageX - (lastcolumn.offset().left - event.pageX);
            lastColumnWidth -= 10 + (20 * (index - 1)); //Sicherheitsabstand
            if (lastcolumn.width() <= 20) {
                this.ResizeNext(splitter, el1, el2.slice(0, -1), event, initialLength);
            }
            else {
                lastcolumn.css("max-width", lastColumnWidth);
                lastcolumn.children("div").css("width", lastColumnWidth);
            }
        }
    };
    // --- Implementation -------------------------------------------------
    GridSplitter.prototype.attachEvents = function (el1, el2) {
        var downEvent, moveEvent, upEvent;
        var verticalLimit;
        if (this.isMobile) {
            downEvent = "touchstart mousedown";
            moveEvent = "touchmove mousemove";
            upEvent = "touchend mouseup";
        }
        else {
            downEvent = "mousedown";
            moveEvent = "mousemove";
            upEvent = "mouseup";
            verticalLimit = $(window).height() - 30;
        }
        var $this = this;
        this.splitter.bind(downEvent, function (eDown) {
            $this.activateSplitters();
            var d = new Date().getTime();
            $(window).bind(moveEvent, function (eMove) {
                // TODO: prevent Move event from being propagated => the entire surface moves on certain devices
                //eMove.stopImmediatePropagation();
                //eMove.stopPropagation();
                if (new Date().getTime() < d + 30) {
                    return;
                }
                d = new Date().getTime();
                if (!$this.isMobile) {
                    $this.moveSplitterDesktop(el1, el2, eMove, verticalLimit);
                }
                else {
                    $this.moveSplitterMobile(el1, el2, eMove);
                }
            });
        });
        $(window).bind(upEvent, function (eUp) {
            if (!$this.isActive)
                return;
            $(window).unbind(moveEvent);
            $this.savePosition();
            $this.deactivateSplitters();
        });
    };
    GridSplitter.prototype.activateSplitters = function () {
        this.isActive = true; //current dragged splitter is always active
        var orientation = this.getOrientation();
        if (window.orgAnice.IsDesktop()) {
            var cursor = orientation == "horizontal" ? "ns-resize" : "ew-resize";
            $("html,body").css("cursor", cursor);
            // set all next vertical splitters active because they might be moving as well
            if (orientation == "vertical") {
                $.each(this.splitter.nextAll(".grid-splitter"), function () {
                    if ($(this).data("gridSplitter") != undefined)
                        $(this).data("gridSplitter").isActive = true;
                });
            }
        }
        this.splitter.parent().css("user-select", "none"); // disable marking text while dragging (blue squares)
    };
    GridSplitter.prototype.deactivateSplitters = function () {
        this.isActive = false;
        // TODO: $("*") is a most expensive operation! try to save time and memory cost
        this.splitter.parent().css("user-select", "");
        // normal cursor on desktop
        if (window.orgAnice.IsDesktop()) {
            $("html,body").css("cursor", "default");
        }
    };
    GridSplitter.prototype.getOrientation = function () {
        return this.orientation == undefined
            ? ($(window).width() / $(window).height() < 13 / 9) ? "horizontal" : "vertical"
            : this.orientation;
    };
    GridSplitter.prototype.getPosition = function () {
        var position;
        if ($.cookie(this.id) != undefined) {
            position = $.cookie(this.id);
        }
        else {
            position = (this.getOrientation() === "horizontal") ? this.splitter.offset().top : this.splitter.offset().left;
            $.cookie(this.id, position, { expires: 30, path: '/' });
        }
        return Math.round(position);
    };
    GridSplitter.prototype.init = function () {
        var _this = this;
        var orientation = this.getOrientation();
        var position;
        if ($.cookie(this.id) != undefined) {
            position = $.cookie(this.id);
        }
        else {
            position = (orientation == "horizontal") ? this.splitter.offset().top : this.splitter.offset().left;
        }
        //var minValue: number;
        var firstColumnWidth;
        if (window.orgAnice.IsDesktop() && ($.cookie(this.id) != undefined)) {
            if (orientation === "horizontal") {
                this.el1.height(position - this.el1.offset().top);
                this.el2.height($(window).height() - this.el2.offset().top);
            }
            else {
                firstColumnWidth = position - this.el1.offset().left;
                this.el1.css("max-width", firstColumnWidth);
                this.el1.width(firstColumnWidth);
                this.el1.children("div").css("width", firstColumnWidth);
            }
        }
        else if (window.orgAnice.IsTablet()) {
            var offset;
            if (orientation === "horizontal") {
                // TODO: calculate min-heigth or min-width for elements 1 & 2 from css(min-height/width)
                //// min el1 height
                //var value = this.el1.css("min-height");
                //if (value != undefined) {
                //	minValue = window.orgAnice.ConvertUnits(value, this.el1[0]);
                //	if (minValue != undefined && !isNaN(minValue))
                //		this.element1MinHeight = minValue;
                //}
                //// min el2 height
                //value = this.el2.css("min-height");
                //if (value != undefined) {
                //	minValue = window.orgAnice.ConvertUnits(value, this.el2[0]);
                //	if (minValue != undefined && !isNaN(minValue))
                //		this.element2MinHeight = minValue;
                //}
                offset = {
                    left: $(window).width() / 2 - this.splitter.width() / 2,
                    top: position - (this.splitterThickness + this.splitter.height())
                };
                this.el1.height(position - this.el1.offset().top);
                this.el2.height($(window).height() - this.el2.offset().top);
            }
            else {
                offset = {
                    left: position - this.splitter.width() / 2,
                    top: $(window).height() / 2 - this.splitter.height() / 2
                };
            }
            this.splitter.offset(offset);
            setTimeout(function () { return _this.onResizeMobile(); }, 0);
        }
    };
    GridSplitter.prototype.moveSplitterDesktop = function (el1, el2, event, verticalLimit) {
        if (this.getOrientation() == "vertical") {
            var initialLength = el2.length;
            this.ResizeNext(this.splitter, el1, el2, event, initialLength);
        }
        else {
            if (event.pageY >= verticalLimit) {
                el2.height(0);
                el1.height($(window).height() - el1.offset().top - this.splitter.height() - 25);
                return;
            }
            el1.height(event.pageY - el1.offset().top - (this.splitter.height() / 2));
            el2.height($(window).height() - el2.offset().top);
        }
        this.onPositionChanged({ left: event.pageX, top: event.pageY });
    };
    GridSplitter.prototype.moveSplitterMobile = function (el1, el2, event) {
        var orientation = this.getOrientation();
        var currentCoords = this.splitter.offset();
        var newCoords = { left: 0, top: 0 };
        var position;
        if (event.originalEvent.touches != undefined) {
            if (orientation == "vertical") {
                newCoords.left = event.originalEvent.touches[0].pageX - this.splitter.width() / 2;
                newCoords.top = currentCoords.top;
                el1.width(event.originalEvent.touches[0].pageX + 10);
                el2.width($(window).width() - event.originalEvent.touches[0].pageX - 10);
            }
            else {
                newCoords.top = event.originalEvent.touches[0].pageY - this.splitter.height();
                newCoords.left = currentCoords.left;
                if (event.originalEvent.touches[0].pageY < 105 || event.originalEvent.touches[0].pageY > $(window).height() - 40)
                    return;
                el1.height(event.originalEvent.touches[0].pageY - el1.offset().top - (this.splitter.height() / 2));
                position = $(window).height() - el2.offset().top - 52;
                el2.height(position);
            }
        }
        else {
            if (orientation == "vertical") {
                newCoords.left = event.pageX - this.splitter.width() / 2;
                newCoords.top = currentCoords.top;
                el1.width(event.pageX + 10);
                el2.width($(window).width() - event.pageX - 10);
            }
            else {
                if (event.pageY < 105 || event.pageY > $(window).height() - 40)
                    return;
                newCoords.left = currentCoords.left;
                newCoords.top = event.pageY - this.splitter.height();
                el1.height(event.pageY - el1.offset().top - (this.splitter.height() / 2));
                position = $(window).height() - el2.offset().top - 52;
                el2.height(position);
            }
        }
        this.splitter.offset(newCoords);
        this.onPositionChanged(newCoords);
    };
    GridSplitter.prototype.onPositionChanged = function (coords) {
        this.PositionChanged.trigger(this, coords);
        if (this.el2 != undefined) {
            if (this.el2.length == 1) {
                this.el2.trigger("invalidateLayout");
            }
            else {
                $.each(this.el2, function (index, element) {
                    $(element).trigger("invalidateLayout");
                });
            }
        }
    };
    GridSplitter.prototype.onResizeDesktop = function () {
        var firstColumnWidth;
        if (this.orientation === "horizontal") {
            this.el1.height(this.getPosition() - this.el1.offset().top);
            this.el2.height($(window).height() - this.el2.offset().top);
        }
        else {
            firstColumnWidth = this.getPosition() - this.el1.offset().left;
            this.el1.css("max-width", firstColumnWidth);
            this.el1.width(firstColumnWidth);
            this.el1.children("div").css("width", firstColumnWidth);
        }
    };
    GridSplitter.prototype.onResizeMobile = function () {
        //if (windowWidth != undefined && $(window).width() == windowWidth) return; //suppress weird resize-event calls on iPad
        //windowWidth = $(window).width();
        var orientation = this.getOrientation();
        var totalHeight = $(window).height();
        var totalWidth = $(window).width();
        var bottomNavHeight = Math.floor($("#orgAnicePageNavigation").is(":visible") ? $("#orgAnicePageNavigation").height() : 0);
        var topNavHeight = Math.floor($("#topNavigation").is(":visible") ? $("#topNavigation").height() + $(".table h2").height() : 0);
        var caption = $("#addressPage .caption");
        var innerHeight = totalHeight - bottomNavHeight - topNavHeight;
        var dialogHeight = totalHeight - bottomNavHeight;
        var position;
        var offset;
        this.el2.scrollTop(0);
        if (orientation === "horizontal") {
            offset = {
                left: $(window).width() / 2 - this.splitter.width() / 2,
                top: this.getPosition() - (this.splitterThickness + this.splitter.height())
            };
            this.el1.height(this.getPosition() - this.el1.offset().top - (this.splitter.height() / 2));
            position = $(window).height() - this.el2.offset().top - 52;
            this.el2.height(position);
            this.el1.width($(window).width());
            this.el2.width($(window).width());
            if (this.el2 != undefined) {
                if (this.el2.length == 1) {
                    this.el2.trigger("invalidateLayout");
                }
                else {
                    $.each(this.el2, function (index, element) {
                        $(element).trigger("invalidateLayout");
                    });
                }
            }
        }
        else {
            position = this.getPosition();
            offset = {
                left: position,
                top: $(window).height() / 2 - this.splitter.height() / 2
            };
            this.el1.height(dialogHeight);
            this.el2.height(innerHeight);
            this.el1.width(Math.floor(position + this.splitter.width()));
            this.el2.width(Math.floor($(window).width() - position - this.splitter.width() - 1));
        }
        this.splitter.offset(offset);
    };
    GridSplitter.prototype.savePosition = function () {
        var orientation = this.getOrientation();
        var position;
        if (orientation == "horizontal") {
            position = this.splitter.offset().top;
            $.cookie(this.id, position, { expires: 30, path: '/' });
        }
        else {
            position = this.splitter.offset().left;
            $.cookie(this.id, position, { expires: 30, path: '/' });
        }
        //var options = {
        //    Id: this.id,
        //    Workspace: this.workspace,
        //    Orientation: this.orientation,
        //    Position: position
        //};
        //$.ajax({
        //	url: "/Account/SaveGridSplitter",
        //	contentType: "application/json; charset=utf-8",
        //	dataType: "json",
        //	data: JSON.stringify(options),
        //	type: "POST"
        //})
        //.done(data => {
        //})
        //.fail((xhr, status, error) => {
        //	alert("fail");
        //});
    };
    GridSplitter.prototype.triggerLayout = function (element, dimension) {
        if (element.length == 1)
            element.trigger("invalidateLayout", dimension);
        else {
            $.each(element, function (index, el) {
                $(el).trigger("invalidateLayout");
            });
        }
    };
    return GridSplitter;
}());
// make it a jQuery plug-in
// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
    $.fn.gridSplitter = function () {
        if (this.length <= 0) {
            return this.data("gridSplitter");
        }
        var element1, element2, splittedId = this.attr("id").split("_"), orientation = splittedId[splittedId.length - 1];
        // NOTE: smartphone does not use gridsplitters, yet
        var isMobile = window.orgAnice.IsTablet() || window.orgAnice.IsSmartphone();
        if (orientation == "horizontal" || isMobile) {
            element1 = this.prev();
            element2 = this.next();
        }
        else {
            element1 = this.prev(".control-frame");
            element2 = this.nextAll(".workspace-column");
        }
        var gs = new GridSplitter(this, element1, element2, orientation, isMobile);
        this.data("gridSplitter", gs);
        return gs;
    };
})(jQuery || $);
// ReSharper restore JsFunctionCanBeConvertedToLambda 
//# sourceMappingURL=orgAnice.gridSplitter.js.map