﻿
///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="../typings/moment.d.ts"/>
///<reference path="./orgAnice.d.ts"/>

//import orgAnice = orgAniceModule.orgAnice;

// ReSharper disable InconsistentNaming
interface KnockoutExtenders {
	date: (target, format) => KnockoutComputed<{}>;
	minLength: (target, minvalue: number) => KnockoutComputed<{}>;
	numeric: (target, precision: number) => KnockoutComputed<{}>;
	pattern: (target, pattern: string) => KnockoutComputed<{}>;
	//required: (target, value) => KnockoutComputed<{}>;
	shadowDate: (target, args: {}) => KnockoutComputed<{}>;
	time: (target, format) => KnockoutComputed<{}>;
}
// ReSharper restore InconsistentNaming

// extend 'normal' KnockoutObservable for validation
interface KnockoutObservable<T> {
	hasError: KnockoutObservable<boolean>;
	errorText: KnockoutObservable<string>;
}


ko.extenders.date = (target, format) => {

	var orgAniceMinDateIso = orgAniceModule.orgAnice.minDate; // "1899-12-30", as defined in orgAnice.core
	var orgAniceMinDateJson = "/Date(" + orgAniceMinDateIso.valueOf() + ")/";

	// we consider the value held in target() to be always valid; that is
	// an ISO-8601 value ("YYYY-MM-DDThh:mm:ss.sss+/-offset"), which is the same as orgAnice.CurrentCulture.DateTimeFormat.SortableDateTimePattern
	// basically, this will always be saved as a string
	var result = ko.pureComputed({
		read: () => {
			var current = target();
			return (current == undefined || orgAniceMinDateIso.valueOf() == moment(current).valueOf()) ? "" : moment(current).format(format);
		},
		write: (newValue: any) => {
			var value, current, valueToWrite;
			if (newValue == undefined || newValue === "")
				valueToWrite = orgAniceMinDateIso.format();
			else {
				value = moment(newValue);
				if (!value.isValid())
					value = moment(newValue, format);
				if (!value.isValid())
					value = moment(orgAniceMinDateJson);

				valueToWrite = value.format();
			}

			current = target();

			if (valueToWrite != current) {
				target(valueToWrite);
			} else {
				if (newValue !== current)
					target.notifySubscribers(valueToWrite);
			}
		}
	}).extend({ notify: "always" });

	result(target());

	return result;
}


// creates and maintains a shadow property which will be bound to the input field
// target: the view model property to be shadowed
// args: a plain object which receives the format and expression for validation
// args.expression: a regex expression, against which the user input will be validated
// args.format: the formatting expression that will be used to convert the original values
ko.extenders.shadowDate = (target: KnockoutObservable<string>, args: {}) => {

	if (!target.hasOwnProperty("hasError")) {
		target.hasError = ko.observable<boolean>(false);
		target.errorText = ko.observable("");
	}

	var format: string = args["format"] || window.orgAnice.CurrentCulture.DateTimeFormat.ShortDatePattern.replace(/yyyy/, "yy").toUpperCase();
	var expression: string = args["pattern"] || "";
	var suppress = false;

	target["format"] = format;
	target["expression"] = expression;

	var orgAniceMinDateIso = orgAniceModule.orgAnice.minDate; // "1899-12-30", as defined in orgAnice.core

	// if target changes -> transfer formatted value into shadow
	target.subscribe((newValue) => {
		if (newValue == undefined || newValue === "") {
			newValue = orgAniceMinDateIso.format(format);
		}
		var val = moment(newValue);
		if (val.isValid()) {
			suppress = true;
			shadow(val.format(format));
			suppress = false;
		}
	});

	// we consider the value held in target() to be always valid; that is an ISO-8601 value ("YYYY-MM-DDThh:mm:ss.sss+/-offset"),
	// which is the same as orgAnice.CurrentCulture.DateTimeFormat.SortableDateTimePattern;
	// basically, this will always be saved as a string
	var shadow = ko.pureComputed({
		read: () => {
			var current = target();
			var val = (current == undefined || current === "" ||
					orgAniceMinDateIso.valueOf() == moment(current).valueOf()) || // "hidden" date
					moment(current).year() == 1 // c# DateTime.MinDate
				? ""
				: moment(current).format(format);
			return val;
		},
		write: (newValue: any) => {
			var current = target(),
				value = moment(current);

			if (newValue == undefined || newValue === "") {
				target(orgAniceMinDateIso.format());
				target.hasError(false); // TODO: this is NOT valid, if the field is mandatory
			} else {

				var hasError;
				if (expression !== "") {
					hasError = !newValue.match(expression);
					if (!hasError)
						value = moment(newValue, format);
				} else {
					value = moment(newValue, format);
				}
				hasError = hasError || !value.isValid();

				target.hasError(hasError);

				if (!hasError && !suppress) {
					target(value.format()); // assign new local-specific value to the observed member
				}
			}
		}
	}).extend({ notify: "always" });

	target["shadowDate"] = shadow;

	shadow(target());

	return shadow;
}

ko.extenders.time = (target, format) => {

	var orgAniceMinDateIso = orgAniceModule.orgAnice.minDate; // "1899-12-30", as defined in orgAnice.core
	var orgAniceMinDateJson = "/Date(" + orgAniceMinDateIso.valueOf() + ")/";

	// we consider the value held in target() to be always valid; that is
	// an ISO-8601 value ("YYYY-MM-DD hh:mm:ss sss +/-offset");
	// basically, this will always be saved as a string
	var result = ko.pureComputed({
		read: () => {
			var current = target();
			return (current == undefined || orgAniceMinDateIso.valueOf() == moment(current).valueOf()) ? "" : moment(current).format(format);
		},
		write: (newValue: any) => {
			var value, current, valueToWrite;
			if (newValue == undefined)
				valueToWrite = orgAniceMinDateIso.format();
			else {
				value = moment(newValue);
				if (!value.isValid())
					value = moment(newValue, format);
				if (!value.isValid())
					value = moment(orgAniceMinDateJson);

				valueToWrite = value.format();
			}

			current = target();

			if (valueToWrite != current) {
				target(valueToWrite);
			} else {
				if (newValue !== current)
					target.notifySubscribers(valueToWrite);
			}
		}
	}).extend({ notify: "always" });

	result(target());

	return result;
}

ko.extenders.numeric = (target, precision) => {
	//create a writable computed observable to intercept writes to our observable
	var result = ko.pureComputed({
		read: target, //always return the original observables value
		write: (newValue: any) => {
			var current = target(),
			    roundingMultiplier = Math.pow(10, precision),
			    newValueAsNum = isNaN(newValue) ? 10 : parseFloat(newValue),
			    valueToWrite = Math.round(newValueAsNum * roundingMultiplier) / roundingMultiplier;

			//only write if it changed
			if (valueToWrite !== current) {
				target(valueToWrite);
			} else {
				//	//if the rounded value is the same, but a different value was written, force a notification for the current field
				if (newValue !== current) {
					target.notifySubscribers(valueToWrite);
				}
			}
		}
	}).extend({ notify: 'always' });

	//initialize with current value to make sure it is rounded appropriately
	result(target());

	//return the new computed observable
	return result;
};

ko.observableArray.prototype.clear = (target) => {
	while (target.length > 0) {
		target.pop();
	}
}

ko.extenders.minLength = (target, minvalue) => {
	if (!target.hasOwnProperty("hasError")) {
		target.hasError = ko.observable<boolean>(false);
		target.errorText = ko.observable("");
	}

	function validate(newValue) {
		var value = (typeof newValue == "function") ? newValue() : newValue;
		if (value == undefined || typeof value != "string") {
			target.hasError(true);
			return;
		}
		value = value.trim();
		target.hasError(value.length < minvalue);
	}

	validate(target);

	target.subscribe(validate);

	return target;
}

ko.extenders.pattern = (target, pattern: string) => {
	if (!target.hasOwnProperty("hasError")) {
		target.hasError = ko.observable<boolean>(false);
		target.hasError.subscribe((newValue) => {
			var dummy = newValue;
		});
		target.errorText = ko.observable("");
	}

	function validate(newValue) {
		var value = typeof newValue == "function" ? newValue() : newValue;
		target.hasError(pattern != undefined && pattern !== "" && value != undefined && value !== "" && !value.match(pattern));
	}

	validate(target);

	if (target.rules == undefined)
		target.rules = {};
	target.rules.pattern = target.subscribe(validate);

	return target;
}

//ko.extenders.required = (target, value) => {
//	target.isRequired = ko.observable();

//	validate: (newValue) => {
//		target.isRequired(newValue == undefined || newValue.length == undefined || newValue.length < 1);
//	}

//	this.validate(target);

//	target.subscribe(this.validate);

//	return target;
//}