﻿
///<reference path="orgAnice.data.d.ts"/>

interface JQuery {
	listControl(data?: any, options?: any): JQuery;
	listControlItem(): JQueryTmplItem;
	template(name?: string): () => any;
}

interface JQueryStatic {
	listControl(template: string, data?: any, options?: any): JQuery;
	listControl(template: (data: any) => string, data?: any, options?: any): JQuery;
	listControlItem(element: JQuery): JQueryTmplItem;
	listControlItem(element: HTMLElement): JQueryTmplItem;
	template(name: string, template: any): (data: any) => string[];
	template(template: any): JQueryTemplateDelegate;
}

interface JQueryTemplateDelegate {
	(jQuery: JQueryStatic, data: any): string[];
}

interface JQueryTmplItem {
	data: any;
	nodes: HTMLElement[];
	key: number;
	parent: JQueryTmplItem;
}

interface KnockoutBindingHandlers {
	// Extensions for ListControl
	imgFromList: KnockoutBindingHandler;
	formattedDate: KnockoutBindingHandler;
}

interface IListControlQueryOptions {
	key: string;
	count: number;
	index: string;
	includeRelations: boolean;
	requery: boolean;
	shortcut: string;
}

interface IListControl {

    // --- Events ---------------------------------------------------------
	Activated: orgAniceModule.orgAniceEvent<IListActivatedEventArgs>;

	Cleared: orgAniceModule.orgAniceEvent<boolean>;

    PropertyChanged: orgAniceModule.orgAniceEvent<string>;

    Ready: orgAniceModule.orgAniceEvent<IListControl>;

    /// <summary>Fired, if the currently selected entry in the list changes.</summary>
    SelectionChanged: orgAniceModule.orgAniceEvent<ISelectionChangedEventArgs>;

    /// <summary>Fired, if a shortcut has been clicked.</summary>
	ShortcutChangedEvent: orgAniceModule.orgAniceEvent<string>;

	RelationsChanged: orgAniceModule.orgAniceEvent<IRelationsChangedEventArgs>;

	UpdateRelations: orgAniceModule.orgAniceEvent<IRelationsResultModel[]>;

    // --- Properties -----------------------------------------------------
	Columns: IListControlColumn[];
	CurrentShortcut: string;

	/// <summary>The dialog manager the instance is assigned to.</summary>
	/// <remarks>Each list is assigned to a dialog via its dialog manager.</remarks>
	DialogManager: IDialogCommandManagerItem;

	/// <summary>Enables/disables whether to check for modifications to be saved before selecting a new entry
	/// 		 from the list. The default is <c>true</c>.</summary>
	EnsureCanChange: boolean;

    Formatters: { [index: string]: IListItemFormatter };
    Headers: string[];
	Height: number;
	IsActivationSuppressed: boolean;
	IsAutoShow: boolean;
	IsPagingEnabled: boolean;
	KeyColumn: string;
	Name: string;
	PageSize: number;
	PagingControl: IPagingControl;
	PagingPlacement: Placement;
	SelectedItem: {};

	/// <summary>The service endpoint defines the entire connection to the service which will process all server-side actions
	///			 of the control. It may contain schema, host, and path information, or parts of these, but without the
	///			 method to be invoked. In WCF terms, this is the address of th service. </summary>
	ServiceEndpoint: string;

    ShortcutPlacement: Placement;
	ShowShortcuts: boolean;

    ViewModel: IListControlViewModel;
    Width: number;

    // --- Methods --------------------------------------------------------
	AddColumn(args: { Name: string; Header: string; IsKey?: boolean; IsVisible?: boolean }): void;
	AddColumn(name: string, text: string, isKey?: boolean, isVisible?: boolean): void;

    AddOrChangeItem(item: {}): {}

	Clear(): void;

	FillViewModel(args: {}[]): void;

	IndexChanged(args: IIndexChangedEventArgs): void;

	OnAction(action: string, args: any): void;

	Query(args?: IListControlQueryOptions): void;

	QueryByShortcut(key: string, index?: string): void;

	/// <summary>Requests a search action on the server.</summary>
	/// <param name="options">Search parameters as determined by the search controller. </param>
	/// <remarks>The results of this operation must be executed asynchronously! Thus, this method calls
	/// 		 into the search controller to update its display, after the AJAX call returns.</remarks>
	Search(options: ISearchArgs): void;

	/// <summary>Removes the item identified by the given oid.</summary>
	/// <param name="oid">The item's oid. </param>
	RemoveItem(oid: string): void;

	ResumePropertyChanged();

	ResumeSelectionChanged();

	/// <summary>Scrolls the currently selected entry into view.</summary>
	ScrollIntoView(): void;

	Select(oid: string, fireChanged?: boolean): void;

	Show(): void;

	SuspendPropertyChanged(): void;

	SuspendSelectionChanged(): void;
}

interface IShortcutViewModel {
	/// <summary>Gets or sets the current shortcut; must be one of the entries from the shortcuts ViewModel.</summary>
	currentShortcut: KnockoutObservable<string>;
	shortcuts: KnockoutObservableArray<string>;
	onClick: (data: any, event: any) => void;
}

interface IListControlOptions {
    // UpperCamelCase definitions for compatibility to TypeScript and C# style guidelines
    AutoOpen: boolean;
    Columns: IListControlColumn[];
    Formatters?: IListItemFormatter[];
	Height: number;
	IsPagingEnabled: boolean;
	Key: string;
	NoFrames: boolean;
	PageSize: number;
	/// <summary>The number of pages at which a shortcut will be selected automatically (system default is 20). </summary>
	PagesToAutoShortcut: number;
	PagingControl: IPagingControl;
	PagingPlacement: Placement;
	ServiceEndpoint: string;
	ShortcutControlOptions: IShortcutControlOptions;
	ShortcutPlacement: Placement;
	/// <summary>Suppresses list activation, that is, selecting the currently active record, which would
	///			 force the control to switch into dialog mode on a smartphone device. </summary>
	SuppressActivation: boolean;
    Width: number;

    // lowerCamelCase definitions for compatibility to javascript guidelines
    // ReSharper disable InconsistentNaming
    autoOpen: boolean;
    columns: IListControlColumn[];
    formatters?: IListItemFormatter[];
	height: number;
	isPagingEnabled: boolean;
	key: string;
	noFrames: boolean;
	pageSize: number;
	pagesToAutoShortcut: number;
	pagingControl: IPagingControl;
	pagingPlacement: Placement;
	serviceEndpoint: string;
	shortcutControlOptions: IShortcutControlOptions;
	shortcutPlacement: Placement;
	suppressActivation: boolean;
    width: number;
    // ReSharper restore InconsistentNaming
}

interface IListControlColumn {
    CssFormatter: (item: ListControlModelItem, column: IListControlColumn) => string;

    ColumnName: () => string;

    /// <summary>Gets the text, which will be displayed as the header.</summary>
    /// <returns>A string.</returns>
    Header: string;

    /// <summary>Query if this ListControlColumn is the key column; the default is <c>false</c>.</summary>
    /// <returns><c>true</c> if this is the key column, <c>false</c> if not.</returns>
    IsKey: boolean;

    /// <summary>Query if this ListControlColumn is visible; the default is <c>true</c>.</summary>
    /// <returns><c>true</c> if this column is visible, <c>false</c> if not.</returns>
    IsVisible: boolean;

    /// <summary>Gets the name of this column; the name will also be used to index into the values of the parent's viewmodel.</summary>
    /// <returns>A string.</returns>
    Name: string;

    /// <summary>The name of the template, which is created during initialization. </summary>
    Template: string;

    Type: number;
}

interface IListControlViewModel {
    Columns: IListControlColumn[];
    ColumnTemplates: { [index: string]: string };
    Formatters: { [index: string]: IListItemFormatter };
    KeyColumn: string;
    IsVisible: boolean;
    Items: KnockoutObservableArray<ListControlModelItem>;
    OnClick: Function;
	SelectTemplate: (column: IListControlColumn) => string;
	Relations: KnockoutObservableArray<IRelationsResultModel>;
}
