///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="../typings/knockout.mapping.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />
///<reference path="orgAnice.listControl.d.ts"/> />
///<reference path="orgAnice.pagingControl.d.ts"/> />
///<reference path="orgAnice.statusBar.d.ts"/> />
/*
 * css-rules for
 * - scrollFrame
 *		-- align-items: determines how to adjust the list-content, if the frame is wider than the list the content:
 *			+ center: will adjust the list in the center and ignore the space around
 *			+ stretch: will adjust the list over the full width of the area, distributing the available space to all columns
 *			+ flex-end: will adjust the list to the left, the last header will fill all remaining space to the right
 *			+ flex-start: will adjust the list to the left, ignoring the space to the right
 */
var Placement;
(function (Placement) {
    Placement[Placement["None"] = 0] = "None";
    Placement[Placement["Top"] = 1] = "Top";
    Placement[Placement["Bottom"] = 2] = "Bottom";
    Placement[Placement["Left"] = 4] = "Left";
    Placement[Placement["Right"] = 8] = "Right";
})(Placement || (Placement = {}));
// ReSharper disable InconsistentNaming
var Alignment;
(function (Alignment) {
    Alignment[Alignment["baseline"] = 0] = "baseline";
    Alignment[Alignment["center"] = 1] = "center";
    Alignment[Alignment["flex-end"] = 2] = "flex-end";
    Alignment[Alignment["flex-start"] = 3] = "flex-start";
    Alignment[Alignment["stretch"] = 4] = "stretch";
})(Alignment || (Alignment = {}));
var ListItemFormatter = (function () {
    // --- Constructor ----------------------------------------------------
    function ListItemFormatter(name, formatter) {
        this.format = function (value) { return value.toString(); };
        this.name = name;
        if (formatter != undefined && typeof formatter == "function")
            this.format = formatter;
    }
    Object.defineProperty(ListItemFormatter.prototype, "Name", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListItemFormatter.prototype, "Format", {
        get: function () { return this.format; },
        set: function (f) {
            this.format = f;
        },
        enumerable: true,
        configurable: true
    });
    return ListItemFormatter;
}());
var ListControl = (function () {
    // --- Constructor ----------------------------------------------------
    function ListControl(name, topLevel, options) {
        var _this = this;
        // --- Events ---------------------------------------------------------
        this.Activated = new orgAniceModule.orgAniceEvent();
        this.Cleared = new orgAniceModule.orgAniceEvent();
        /// <summary>Fired, if a property of the control has changed.</summary>
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.Ready = new orgAniceModule.orgAniceEvent();
        /// <summary>Fired, if the current collection in the list view changes.</summary>
        this.SelectionChanged = new orgAniceModule.orgAniceEvent();
        /// <summary>Fired, if a shortcut has been clicked.</summary>
        this.ShortcutChangedEvent = new orgAniceModule.orgAniceEvent();
        this.RelationsChanged = new orgAniceModule.orgAniceEvent();
        this.UpdateRelations = new orgAniceModule.orgAniceEvent();
        // --- Fields ---------------------------------------------------------
        this.selectedClass = "active";
        this.hasData = false;
        this.hasShortcuts = false;
        this.height = 500;
        this.isActivationSuppressed = false;
        this.isAutoShow = false;
        this.isPagingEnabled = true;
        this.isReadyFired = false;
        // --- paging ---------------------------------
        this.pageSize = 5; // -1;	// number of entries per page, if paging is enabled
        this.pagingPlacement = Placement.Bottom;
        this.selectedItem = {};
        this.selectedValue = ko.observable();
        this.shortcutPlacement = Placement.None;
        // --------------------------------------------
        this.suspendPropertyChanged = false;
        this.suspendSelectionChanged = false;
        this.ensureCanChange = true;
        this.viewModel = {
            Columns: [],
            ColumnTemplates: {},
            Formatters: {},
            KeyColumn: "",
            IsVisible: true,
            Items: ko.observableArray([]),
            OnClick: function () { },
            SelectTemplate: undefined,
            Relations: ko.observableArray()
        };
        if (name == undefined)
            throw "ListControl can not be created - name is missing!";
        if (topLevel == undefined)
            throw "ListCOntrol can not be created - top level element is missing!";
        var control = this;
        var pagesize = options.pageSize || options.PageSize || 50;
        if (isNaN(pagesize))
            pagesize = 50;
        this.pageSize = pagesize;
        this.pagingControl = options.pagingControl || options.PagingControl;
        this.pagingPlacement = options.pagingPlacement || options.PagingPlacement;
        if (typeof this.pagingControl === "object") {
            if (options.pageSize != undefined)
                this.pagingControl.PageSize = options.pageSize;
        }
        this.isActivationSuppressed = options.suppressActivation || options.SuppressActivation;
        this.name = name;
        this.topLevel = topLevel; // the top level element; the control attaches to this
        this.height = $(window).height() - this.topLevel.offset().top;
        this.topLevel.height(this.height);
        this.topLevel.css("display", "block").addClass("control-frame");
        this.topLevel.bind("invalidateLayout", function (e) {
            _this.height = $(e.target).height();
            _this.onControlResized(Array.isArray(e) && e["length"] >= 2 ? e[1] : e);
        });
        // data, event: these are supplied by knockout.js click binding
        // data: the data; event: the javascript event
        this.viewModel.OnClick = function (data, evt) {
            var oldOid = control.selectedValue();
            var oid = data.hasOwnProperty("Oid")
                ? data["Oid"]()
                : $(evt.currentTarget).attr("id");
            if (oid != undefined)
                oid = oid.toLowerCase();
            // validate that changing the current entry is allowed (dialog may be modified)
            var canChange = control.validateCanChange();
            if (!canChange) {
                control.SuspendSelectionChanged();
                control.selectedValue(oldOid);
                control.ResumeSelectionChanged();
                return;
            }
            control.selectedValue(oid);
            control.onActivated(oid != oldOid);
            control.onUpdateRelations(oid);
        };
        this.selectedValue.subscribe(function (newValue) {
            if (newValue == undefined)
                return;
            control.scrollFrame.find("ul").removeClass("active");
            var value = newValue.toLowerCase();
            var selected = control.scrollFrame.find("#" + value);
            if (selected == undefined || selected.length == 0) {
                selected = control.scrollFrame.find("#" + value);
            }
            if (selected == undefined || selected.length == 0) {
                selected = control.scrollFrame.find("#" + value.toUpperCase());
            }
            if (selected != undefined && selected.length > 0) {
                selected.addClass("active");
            }
            control.setSelectedItem(selected);
            control.onSelectionChanged(value);
        });
        this.selectedValue.extend({ notify: 'always' });
        this.viewModel.Relations.subscribe(function (relations) {
            _this.setSessionValue("relations", JSON.stringify(relations)); // update session storage
            _this.onRelationsChanged(relations);
        });
        this.serviceEndpoint = options.serviceEndpoint || options.ServiceEndpoint;
        this.isAutoShow = options.autoOpen || options.AutoOpen || false;
        this.shortcutPlacement = options.shortcutPlacement || options.ShortcutPlacement || this.shortcutPlacement;
        var formatters = options.formatters || options.Formatters;
        if (formatters != undefined && formatters.length > 0) {
            $.each(formatters, function (index, value) {
                control.ViewModel.Formatters[value.Name] = value;
            });
        }
        this.create(options); // create the control
        this.layout(); // layout the control
        this.attachPagingCommands();
        if (!this.IsAutoShow) {
            this.topLevel.css("display", "none");
        }
        if (this.shortcutControl != undefined) {
            this.shortcutControl.QueryShortcutsFinished.add(function () {
                control.layout("height");
            });
        }
        this.PropertyChanged.add(function (property) {
            if (property == undefined)
                return;
            if (typeof property == "object")
                property = property[0];
            if (property == undefined)
                return;
            switch (property.toLowerCase()) {
                case "height":
                case "width":
                    _this.layout(property);
                    break;
            }
        });
        $(window).on("resize", function () {
            _this.layout();
        });
    }
    Object.defineProperty(ListControl.prototype, "Alignment", {
        // --- Properties -----------------------------------------------------
        /// <summary>Gets the alignment of the columns in the ListView.</summary>
        /// <returns>An Alignment.</returns>
        get: function () { return this.alignment || Alignment["flex-end"]; },
        /// <summary>Sets the alignments in the ListView.</summary>
        /// <param name="value" type="Alignment">The new alignment value. </param>
        set: function (value) {
            this.alignment = value;
            this.onPropertyChanged("Alignment");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Columns", {
        get: function () { return this.viewModel.Columns; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "CurrentShortcut", {
        get: function () { return this.shortcutControl == undefined ? undefined : this.shortcutControl.CurrentShortcut; },
        set: function (value) {
            if (this.shortcutControl == undefined)
                return;
            this.shortcutControl.CurrentShortcut = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "DialogManager", {
        get: function () { return this.dialogManager; },
        set: function (value) {
            this.dialogManager = value;
            this.onPropertyChanged("DialogManager");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "EnsureCanChange", {
        get: function () { return this.ensureCanChange; },
        set: function (value) {
            if (this.ensureCanChange === value)
                return;
            this.ensureCanChange = value;
            this.onPropertyChanged("ValidateBeforeChange");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Formatters", {
        get: function () { return this.ViewModel.Formatters; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Headers", {
        get: function () {
            var headers = [];
            $.each(this.viewModel.Columns, function (index, value) { headers.push(value); });
            return headers;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Height", {
        get: function () { return this.height; },
        set: function (value) {
            if (value == undefined || isNaN(value) || value == this.height)
                return;
            this.height = value;
            this.onPropertyChanged("Height");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "IsActivationSuppressed", {
        get: function () { return this.isActivationSuppressed; },
        set: function (value) {
            if (this.isActivationSuppressed == value)
                return;
            this.isActivationSuppressed = value;
            this.onPropertyChanged("IsActivationSuppressed");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "IsAutoShow", {
        get: function () { return this.isAutoShow || false; },
        set: function (value) { this.isAutoShow = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "IsPagingEnabled", {
        get: function () { return this.isPagingEnabled; },
        set: function (value) {
            if (this.pagingControl == undefined)
                return;
            this.isPagingEnabled = value;
            if (value) {
                this.pagingControl.TopLevel.css("display", "block");
            }
            else {
                this.pagingControl.TopLevel.css("display", "none");
            }
            this.layout();
            this.onPropertyChanged("IsPagingEnabled");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "IsPropertyChangedSuspended", {
        get: function () { return this.suspendPropertyChanged; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "KeyColumn", {
        get: function () { return this.keyColumn || ""; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Name", {
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "PageSize", {
        get: function () { return this.pageSize; },
        set: function (value) {
            if (value <= 0)
                value = -1;
            this.pageSize = value;
            this.onPropertyChanged("PageSize");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "PagingControl", {
        get: function () { return this.pagingControl; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "PagingPlacement", {
        get: function () { return this.pagingPlacement; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "SelectedItem", {
        get: function () {
            return this.selectedItem;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "SelectedValue", {
        get: function () { return this.selectedValue(); },
        set: function (value) {
            this.selectedValue(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "ServiceEndpoint", {
        get: function () {
            return this.serviceEndpoint;
        },
        set: function (value) {
            this.serviceEndpoint = value;
            this.onPropertyChanged("ServiceEndpoint");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "ShortcutPlacement", {
        get: function () { return this.shortcutPlacement || Placement.Bottom; },
        set: function (value) {
            this.shortcutPlacement = value;
            this.onPropertyChanged("ShortcutPlacement");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "ShowShortcuts", {
        get: function () {
            return this.showShortcuts;
        },
        set: function (value) {
            this.showShortcuts = value;
            this.onPropertyChanged("ShowShortcuts");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "TopLevel", {
        get: function () { return this.topLevel; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "ViewModel", {
        get: function () { return this.viewModel; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "Width", {
        get: function () { return this.width; },
        set: function (value) {
            if (value == undefined || isNaN(value) || value == this.width)
                return;
            this.width = value;
            this.onPropertyChanged("Width");
        },
        enumerable: true,
        configurable: true
    });
    ListControl.prototype.AddColumn = function (args) {
        var name, header, type = 0, isKey = false, isVisible = true;
        if (typeof arguments[0] == "object") {
            if (!(args.hasOwnProperty("Name") || args.hasOwnProperty("name")))
                throw "AddColumns expects a plain object containing the name of the column";
            if (!(args.hasOwnProperty("Header") || args.hasOwnProperty("header")))
                throw "AddColumns expects a plain object containing the header of the column";
            name = args.Name;
            header = args.Header;
            isKey = args.hasOwnProperty("IsKey") ? args.IsKey : false;
            isVisible = args.hasOwnProperty("IsVisible") ? args.IsVisible : true;
            type = args.hasOwnProperty("Type") ? args.Type : "String";
        }
        else if (arguments.length >= 2) {
            name = arguments[0];
            header = arguments[1];
            if (arguments.length >= 3)
                isKey = arguments[2];
            if (arguments.length >= 4)
                isVisible = arguments[3];
            if (arguments.length >= 5)
                type = arguments[4];
        }
        else {
            throw "Incorrect request: number or type of arguments!";
        }
        var column = new ListControlColumn(name, header, isKey, isVisible, type);
        this.viewModel.Columns.push(column);
        if (isKey != undefined && isKey)
            this.keyColumn = name;
        this.viewModelItem = this.viewModelItem || new ListControlModelItem();
        this.viewModelItem.AddProperty(name);
    };
    ListControl.prototype.AddOrChangeItem = function (args) {
        if (!args.hasOwnProperty("Oid"))
            return null;
        var found = ko.utils.arrayFirst(this.viewModel.Items(), function (item) {
            return item.hasOwnProperty("Oid") && item["Oid"]() == args["Oid"];
        });
        var formatters = this.Formatters;
        // if a match was found, than 'found' will now have an Oid
        if (found != undefined) {
            $.each(found, function (key, value) {
                var val = args[key];
                if (args.hasOwnProperty(key))
                    found[key](formatters != undefined && formatters.hasOwnProperty(key) ? formatters[key].Format(val) : val);
            });
            this.viewModel.Items.valueHasMutated();
        }
        else {
            var newValue = new ListControlModelItem(args, formatters);
            var oid = args.hasOwnProperty("Oid") ? args["Oid"] : undefined;
            this.viewModel.Items.push(ko.mapping.fromJS(newValue));
            if (oid != undefined) {
                this.selectedValue(oid);
                this.ScrollIntoView();
            }
            found = newValue;
        }
        return found;
    };
    ListControl.prototype.Clear = function () {
        this.viewModel.Items([]);
        this.Cleared.trigger(this, true);
        this.selectedValue(orgAniceModule.orgAnice.EmptyGuid);
    };
    ListControl.prototype.FillViewModel = function (args) {
        this.updateList(args);
    };
    ListControl.prototype.ChangeIndex = function (args) {
        var manager = window.orgAnice.dialogManager;
        manager.SelectByList(this);
        manager.SearchControl.SelectIndex(args.Index, false);
    };
    ListControl.prototype.IndexChanged = function (args, syncWithIndexSelector) {
        if (args.Index == undefined || args.Controller == undefined)
            return;
        args.Key = args.Key || "00000000-0000-0000-0000-000000000001";
        this.activeIndex = args.Index;
        var control = this;
        var options = {
            index: args.Index,
            key: args.Key,
            count: (this.isPagingEnabled && this.hasPaging()) ? this.pageSize : -1,
            limit: this.hasPaging() ? this.PageSize : -1,
            includeRelations: (args.hasOwnProperty("includeRelations") && args["includeRelations"]) || false
        };
        var selector = args.Key;
        if (selector === "00000000-0000-0000-0000-000000000001")
            selector = this.selectedValue();
        $.ajax({
            url: this.serviceEndpoint + "/ChangeIndex",
            cache: false,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: JSON.stringify(options),
            type: "POST"
        })
            .done(function (result) {
            var data = result.Data;
            if (result.Success && data != undefined) {
                control.updateList(data.List);
                control.updatePaging(control, data);
                control.updateShortcutsAsync(result.Dictionary["shortcuts"], result.Dictionary["currentShortcut"]); // fire (and forget) update of shortcuts
                if (selector == undefined)
                    selector = data.Oid;
                if (selector != undefined) {
                    if (control.isActivationSuppressed)
                        control.SuspendSelectionChanged();
                    control.SelectedValue = selector;
                    if (control.isActivationSuppressed)
                        control.ResumeSelectionChanged();
                    control.ScrollIntoView();
                    if (control.ViewModel.hasOwnProperty("Relations") && result.Relations != undefined) {
                        var relations = [];
                        $.each(result.Relations, function (index, value) {
                            relations.push(new RelationsResult(value));
                        });
                        control.ViewModel.Relations(relations);
                    }
                }
            }
            else {
                $(document).statusBar().AddMessage("Index konnte nicht gewechselt werden!", StatusBarMessageSeverity.Error);
            }
            if (syncWithIndexSelector != undefined && syncWithIndexSelector) {
                window.orgAnice.dialogManager.SelectByList(control);
                window.orgAnice.dialogManager.SearchControl.SelectIndex(args.Index, true);
            }
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
            $(document).statusBar().AddMessage("Index konnte nicht gewechselt werden!", StatusBarMessageSeverity.Error);
        });
    };
    ListControl.prototype.OnAction = function (action, args) {
        if (typeof action !== "string")
            return;
        switch (action.toLowerCase()) {
            case "addcolumn":
                this.AddColumn(args);
                break;
            case "clear":
                this.Clear();
                break;
            case "select":
                this.select(args);
                break;
            case "position":
                this.setPosition(args);
                break;
            case "query":
                this.Query(args);
                break;
            case "queryindex":
                this.Query(args);
                break;
            case "resize":
                this.layout();
                break;
            case "resumepropertychanged":
                this.ResumePropertyChanged();
                break;
            case "scrollintoview":
                this.ScrollIntoView();
                break;
            case "show":
                this.Show();
                break;
            case "suspendpropertychanged":
                this.SuspendPropertyChanged();
                break;
        }
    };
    ListControl.prototype.Query = function (args) {
        var _this = this;
        if (this.serviceEndpoint == undefined || this.serviceEndpoint == "")
            throw new Error("No data endpoint defined for list '" + this.name + "'");
        if (args == undefined)
            args = "00000000-0000-0000-0000-000000000001";
        var options = {
            key: "00000000-0000-0000-0000-000000000001",
            count: this.pageSize,
            index: undefined,
            includeRelations: args.hasOwnProperty("includeRelations") && args["includeRelations"] === true,
            requery: false,
            suppressActivation: (args.hasOwnProperty("suppressActivation") && args["suppressActivation"] === true) || false
        };
        var selector;
        var keyname = this.getKeyName();
        if (typeof args == "object") {
            if (args.hasOwnProperty("index")) {
                options.index = args["index"];
            }
            if (args.hasOwnProperty("key")) {
                options[keyname] = args["key"];
            }
            else if (args.hasOwnProperty("id")) {
                options[keyname] = args["id"];
            }
            if (args.hasOwnProperty("select")) {
                var s = args["select"];
                if (s != undefined && typeof s == "string")
                    s = s.toLowerCase();
                selector = s;
                options[keyname] = s;
            }
            if (args.hasOwnProperty("requery")) {
                options.requery = args["requery"];
            }
        }
        else if (typeof args == "string" && orgAniceModule.IsValidGuid(args)) {
            options[keyname] = String(args);
        }
        if (options[keyname] != undefined)
            options[keyname] = options[keyname].toLowerCase();
        if (args.hasOwnProperty("ignoreIfSelected") && args["ignoreIfSelected"] === true) {
            if (this.hasSelectedValue &&
                options[keyname] == this.selectedValue().toLowerCase())
                return;
        }
        var found = false;
        if (selector != undefined) {
            if (!options.requery) {
                $.each(this.viewModel.Items(), function (index, item) {
                    if (item.hasOwnProperty("Oid") && item["Oid"] == selector) {
                        _this.selectedValue(selector);
                        _this.ScrollIntoView();
                        found = true;
                        return false;
                    }
                });
            }
        }
        else if (orgAniceModule.IsValidGuid(options[keyname]) &&
            !(orgAniceModule.IsEmptyGuid(options[keyname]) || orgAniceModule.IsEmptyRequestGuid(options[keyname]))) {
            selector = options[keyname];
        }
        if (found)
            return;
        var propagate = true;
        if (args.hasOwnProperty("propagate") && args["propagate"] === false) {
            propagate = false;
        }
        this.executeAjaxGet(this.serviceEndpoint + "/GetList", options, selector, false, true);
        if (this.shortcutControl != undefined) {
            this.shortcutControl.QueryShortcuts();
        }
    };
    ListControl.prototype.QueryByShortcut = function (key, index) {
        var options = {
            count: (this.hasPaging() && (this.isPagingEnabled || key === "")) ? this.pageSize : -1,
            index: this.shortcutControl == undefined ? "" : this.shortcutControl.ShortcutColumn,
            key: key,
            includeRelations: false,
            delayRelations: true
        };
        var endpoint = "/GetListForShortcut";
        this.executeQuery(endpoint, options);
    };
    ListControl.prototype.RemoveItem = function (oid) {
        if (oid == undefined || !orgAniceModule.IsValidGuid(oid))
            return;
        if (this.viewModel.Items().length == 0)
            return;
        var itemToDelete = undefined;
        $.each(this.viewModel.Items(), function (index, item) {
            if (item.hasOwnProperty("Oid") && item["Oid"]() == oid) {
                itemToDelete = item;
                return false;
            }
            else
                return true;
        });
        // ReSharper disable ConditionIsAlwaysConst
        // ReSharper disable once HeuristicallyUnreachableCode
        if (itemToDelete != undefined) {
            this.viewModel.Items.remove(itemToDelete);
            if (this.viewModel.Items().length > 0) {
                var top = this.viewModel.Items()[0]["Oid"]();
                if (top != undefined)
                    this.selectedValue(top);
            }
            else {
                this.selectedValue(orgAniceModule.orgAnice.EmptyGuid);
            }
        }
        // ReSharper restore ConditionIsAlwaysConst
    };
    ListControl.prototype.ResumePropertyChanged = function () {
        this.suspendPropertyChanged = false;
    };
    ListControl.prototype.ResumeSelectionChanged = function () {
        this.suspendSelectionChanged = false;
    };
    ListControl.prototype.ScrollIntoView = function () {
        var value = this.selectedValue();
        if (value == undefined)
            return;
        // TODO: test this in all browsers, extend to scroll to middle of view, not to top
        var selected = this.scrollFrame.find("#" + value.toLowerCase());
        if (selected != undefined && selected.length > 0) {
            var offsetTop = selected.offset().top;
            var offsetHeight = this.Height / 2;
            var delta = offsetTop - offsetHeight;
            window.scrollTo(0, delta);
        }
    };
    ListControl.prototype.Search = function (options) {
        var _this = this;
        var dataEndpoint = this.serviceEndpoint + "/Search";
        var control = this;
        if (this.shortcutControl != undefined && this.shortcutControl.CurrentShortcut != undefined && this.shortcutControl.CurrentShortcut != "") {
            var shortCut = options.value[0].toUpperCase();
            this.shortcutControl.CurrentShortcut = shortCut;
            this.onShortcutChanged(shortCut);
        }
        options.count = this.pageSize;
        $.ajax({
            url: dataEndpoint,
            cache: false,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: options,
            type: "GET"
        })
            .done(function (result) {
            if (result.Success && result.Data.Success) {
                var data = result.Data;
                data.Oid = data.Oid || "00000000-0000-0000-0000-000000000001";
                if (data.List != undefined) {
                    _this.updateList(data.List);
                    _this.updatePaging(control, result.Data);
                }
                control.activeIndex = data.Index;
                control.SelectedValue = data.Oid;
                control.ScrollIntoView();
                control.onUpdateRelations(data.Oid);
                window.orgAnice.SearchControl.SetSearchResult({ success: true, index: data.Index });
            }
            else {
                window.orgAnice.SearchControl.SetSearchResult({ success: false, index: undefined });
            }
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
            $(document).statusBar().AddMessage("Suche fehlgeschlagen", StatusBarMessageSeverity.Error);
        });
    };
    ListControl.prototype.Select = function (oid, fireChanged) {
        if (fireChanged == undefined)
            fireChanged = false;
        if (oid == undefined || oid === "")
            return;
        this.select({ oid: oid, fireChanged: fireChanged });
    };
    ListControl.prototype.Show = function () {
        this.layout();
        this.topLevel.css("display", "block");
    };
    ListControl.prototype.SuspendPropertyChanged = function () {
        this.suspendPropertyChanged = true;
    };
    ListControl.prototype.SuspendSelectionChanged = function () {
        this.suspendSelectionChanged = true;
    };
    ListControl.prototype.validateCanChange = function () {
        if (!this.ensureCanChange)
            return true;
        var canChange = true;
        if (window.orgAnice.dialogManager.IsModified) {
            canChange = window.orgAnice.dialogManager.SaveIfModified();
        }
        //if (this.dialogManager != undefined) {
        //	if (!this.dialogManager.IsModified) return true;
        //	canChange = this.dialogManager.SaveIfModified(true);
        //}
        return canChange;
    };
    // --- Implementation -------------------------------------------------
    ListControl.prototype.addColumn = function (column) {
        var newColumn = new ListControlColumn(column.Name, column.Header, column.IsKey, column.IsVisible);
        newColumn.Type = column.Type || 0;
        this.viewModel.Columns.push(newColumn);
    };
    ListControl.prototype.applyShortcut = function (control, value) {
        control.QueryByShortcut(value);
    };
    ListControl.prototype.attachPagingCommands = function () {
        var _this = this;
        if (this.pagingControl == undefined)
            return;
        this.pagingControl.GotoPositionCommand = function (skipPages, count, oid) {
            var endpoint = _this.ServiceEndpoint + "/GetListByPosition";
            var options = {
                skipPages: skipPages,
                oid: oid,
                count: count,
                includeRelations: true,
                suppressActivation: _this.isActivationSuppressed
            };
            _this.executeAjaxGet(endpoint, options);
        };
    };
    ListControl.prototype.create = function (options) {
        var _this = this;
        if (this.isCreated)
            return;
        if (this.topLevel == undefined)
            this.topLevel = $("#" + this.name + "_list");
        if (this.topLevel == undefined)
            throw Error("TopLevel control " + this.name + " could not be found");
        var control = this;
        if (options.height != undefined && !isNaN(options.height))
            this.height = options.height;
        var columns = options.columns || options.Columns;
        $.each(columns, function (index, value) {
            _this.AddColumn(value);
        });
        this.viewModel.SelectTemplate = function (column) {
            // TODO: return default template if none is found
            return control.ViewModel.ColumnTemplates.hasOwnProperty(column.Name)
                ? control.ViewModel.ColumnTemplates[column.Name]
                : "";
        };
        this.topLevel.empty(); // remove all children
        //this.topLevel.addClass("baselist");
        if (window.orgAnice.IsSmartphone()) {
            this.createSmartphoneLayout(options);
            return;
        }
        else if (window.orgAnice.IsTablet()) {
            this.createSmartphoneLayout(options);
            return;
        }
        var noFrames = !window.orgAnice.IsDesktop();
        var frame;
        if (noFrames) {
            frame = this.topLevel;
        }
        else {
            frame = $(document.createElement("div"));
            frame.addClass("frame");
            var outerBorder = $(document.createElement('div'));
            outerBorder.addClass("outer-border");
            this.topLevel.append(outerBorder);
            var innerBorder = $(document.createElement('div'));
            innerBorder.addClass("inner-border");
            outerBorder.append(innerBorder);
            innerBorder.append(frame);
        }
        if (this.hasShortcut(Placement.Top)) {
            this.paneTop = this.createShortcutPane(options.shortcutControlOptions, "scTop", "horizontal-pane top-pane");
            frame.append(this.paneTop);
        }
        if (this.hasPaging(Placement.Top)) {
            this.pagingPaneTop = this.pagingControl.TopLevel;
            frame.append(this.pagingPaneTop);
        }
        this.paneHeaders = $(document.createElement("div")).attr("id", "listheader").addClass("list-header");
        //this.paneHeaders.css("display", "table-header-group");
        //if (window.orgAnice.IsSmartphone()) {
        //	this.paneHeaders.insertBefore(frame);
        //} else {
        //	frame.append(this.paneHeaders);
        //}
        var centerFrame = $(document.createElement("div"));
        centerFrame.addClass("table");
        centerFrame.css("width", "100%");
        frame.append(centerFrame);
        if (this.hasShortcut(Placement.Bottom) && this.shortcutControl == undefined) {
            this.paneBottom = this.createShortcutPane(options.shortcutControlOptions, "scBottom", "horizontal-pane scBottom-pane");
            frame.append(this.paneBottom);
        }
        if (this.hasPaging(Placement.Bottom)) {
            this.pagingPaneBottom = this.PagingControl.TopLevel;
            frame.append(this.pagingPaneBottom);
        }
        if (this.hasShortcut(Placement.Left) && this.shortcutControl == undefined) {
            this.paneLeft = this.createShortcutPane(options.shortcutControlOptions, "scLeft", "vertical-pane left-pane");
            centerFrame.append(this.paneLeft);
        }
        this.scrollFrame = $(document.createElement("div"))
            .attr("id", "scrollFrame")
            .addClass("scrollFrame");
        centerFrame.append(this.scrollFrame);
        if (this.hasShortcut(Placement.Right) && this.shortcutControl == undefined) {
            this.paneRight = this.createShortcutPane(options.shortcutControlOptions, "scRight", "vertical-pane right-pane");
            centerFrame.append(this.paneRight);
        }
        var table = $(document.createElement("div"));
        table.addClass("table");
        var alignment = this.scrollFrame.css("align-items") || "baseline";
        var flextStart = false;
        var flexEnd = false;
        switch (alignment) {
            case "center":
                table.css("margin-left", "auto");
                table.css("margin-right", "auto");
                break;
            case "stretch":
                table.css("width", "100%");
                break;
            case "flex-start":
                flextStart = true;
                break;
            case "flex-end":
                flexEnd = true;
                break;
            default:
                break;
        }
        this.createScrollFrame(table);
        table.prepend(this.paneHeaders);
        var firstHeader = this.firstVisibleColumn;
        var lastHeader = this.lastVisibleColumn;
        $.each(this.Columns, function (index, value) {
            if (value.IsVisible) {
                var header = $(document.createElement("div"));
                header.html(value.Header);
                header.attr("id", value.Name.toLowerCase());
                header.addClass("cell listheader").addClass(value.Name.toLowerCase());
                if (flextStart && index == firstHeader)
                    header.css("width", "100%");
                else if (flexEnd && index == lastHeader)
                    header.css("width", "100%");
                _this.paneHeaders.append(header);
            }
        });
        this.scrollFrame.append(table);
        this.isCreated = true;
        this.layout();
        this.onReady();
    };
    ListControl.prototype.createScrollFrame = function (container) {
        // create column-templates
        var control = this;
        var listname = this.name;
        $.each(this.viewModel.Columns, function (index, column) {
            column.Template = listname + "-column-template-" + column.Name;
            control.ViewModel.ColumnTemplates[column.Name] = name;
            var template = $(document.createElement("script"))
                .attr("type", "text/html")
                .attr("id", column.Template);
            var item = $(document.createElement("li"))
                .addClass(column.Name.toLowerCase())
                .css("color", "White");
            template.append(item);
            $("body").append(template);
        });
        // the (main) row loop - some knockout-template magic :)
        container.attr("data-bind", "foreach: { data: Items, as: 'Item' }");
        var keyColumn = this.KeyColumn;
        var ul = $(document.createElement("ul"))
            .addClass("row")
            .attr("data-bind", "attr: { id: $data[\"" + keyColumn + "\"] }," +
            "click: $parent.OnClick," +
            "foreach: { data: $parent.Columns, as: 'Column' }");
        var li = $(document.createElement("li"))
            .addClass("cell listentry")
            .attr("data-bind", "css: Column.Name.toLowerCase()")
            .attr("data-bind", "text: Item[Name], visible: IsVisible, css: Column.CssFormatter(Item), attr: { 'data-column-name': Column.Name }");
        ul.append(li);
        container.append(ul);
        ko.applyBindings(this.viewModel, container[0]);
    };
    ListControl.prototype.createShortcutPane = function (options, name, classes) {
        var _this = this;
        var shortcutControl = ShortcutControl.Create($.extend(options, { serviceEndpoint: this.serviceEndpoint }));
        var pane = shortcutControl.Toplevel;
        pane.attr("id", name);
        pane.addClass(classes);
        this.shortcutControl = shortcutControl;
        this.shortcutControl.ShortcutChangedEvent.add(function (args) {
            _this.onShortcutChanged(args);
        });
        return pane;
    };
    ListControl.prototype.createSmartphoneLayout = function (options) {
        //this.paneHeaders = $(document.createElement("div"))
        //	.attr("id", "listheader")
        //	.addClass("list-header")
        //	.appendTo(this.topLevel);
        var _this = this;
        var centerFrame = $(document.createElement("div"))
            .addClass("table")
            .css("width", "100%")
            .appendTo(this.topLevel);
        if (this.pagingControl != undefined) {
            this.pagingPaneBottom = this.PagingControl.TopLevel;
            this.pagingPaneBottom.appendTo(this.topLevel);
        }
        this.scrollFrame = $(document.createElement("div"))
            .attr("id", "scrollFrame")
            .addClass("scrollFrame");
        centerFrame.append(this.scrollFrame);
        if (options.shortcutControlOptions != undefined)
            this.paneRight = this.createShortcutPane(options.shortcutControlOptions, "scRight", "vertical-pane right-pane");
        centerFrame.append(this.paneRight);
        var table = $(document.createElement("div"));
        table.addClass("table");
        this.createScrollFrame(table);
        this.paneHeaders = $(document.createElement("div"))
            .attr("id", "listheader")
            .addClass("list-header")
            .prependTo(table);
        $.each(this.Columns, function (index, value) {
            if (value.IsVisible) {
                var header = $(document.createElement("div"));
                header.html(value.Header);
                header.attr("id", value.Name.toLowerCase());
                header.addClass("cell listheader").addClass(value.Name.toLowerCase());
                _this.paneHeaders.append(header);
            }
        });
        this.scrollFrame.append(table);
        this.isCreated = true;
        this.layout();
        this.onReady();
    };
    ListControl.prototype.convertUnits = function (valueToConvert, element) {
        if (valueToConvert == undefined) {
            return 0;
        }
        var units = valueToConvert.replace(/[0-9]*/, '');
        var value = parseFloat(valueToConvert);
        switch (units) {
            case "px":
                return value;
            case "em":
                return value * this.getEmSize(element);
            default:
                return value;
        }
    };
    ListControl.prototype.executeAjaxGet = function (endpoint, options, selector, useCache, doAsync) {
        var _this = this;
        useCache = useCache || false;
        doAsync = doAsync || false;
        var suppressActivation = (options.hasOwnProperty("suppressActivation") && options["suppressActivation"]) || false;
        var control = this;
        $.ajax({
            url: endpoint,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: options,
            cache: useCache,
            async: doAsync,
            type: "GET"
        })
            .done(function (result) {
            var listQueryResult = result.Data;
            control.updatePagingAsync(listQueryResult);
            if (result.Dictionary != undefined && result.Dictionary.hasOwnProperty("shortcuts")) {
                control.updateShortcutsAsync(result.Dictionary["shortcuts"]);
            }
            var entries = [];
            if (listQueryResult != undefined) {
                entries = listQueryResult.List;
            }
            if (entries == undefined || entries.length == 0) {
                control.Clear();
                control.selectedValue(undefined);
                entries = []; // set empty list to avoid errors
            }
            control.updateList(entries);
            //control.updatePaging(control, listQueryResult);
            if (listQueryResult != undefined) {
                if (selector == undefined) {
                    selector = listQueryResult.Oid;
                }
            }
            if (selector == undefined && entries.length > 0 && entries[0].hasOwnProperty("Oid")) {
                selector = entries[0]["Oid"]; // set to topOfList, if no selector defined
            }
            if (selector != undefined) {
                if (suppressActivation)
                    control.SuspendSelectionChanged();
                control.SelectedValue = selector;
                control.ScrollIntoView();
                if (suppressActivation)
                    control.ResumeSelectionChanged();
            }
            if (control.ViewModel.hasOwnProperty("Relations") && result.Relations != undefined) {
                var relations = [];
                $.each(result.Relations, function (index, value) {
                    relations.push(new RelationsResult(value));
                });
                control.ViewModel.Relations(relations);
            }
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
            _this.Clear();
        });
    };
    ListControl.prototype.executeQuery = function (endpoint, options, selector) {
        var _this = this;
        if (endpoint[0] != "/")
            endpoint = "/" + endpoint;
        var control = this;
        var includeRelations = (options.hasOwnProperty("includeRelations") && options["includeRelations"]) || false;
        var delayRelations = (options.hasOwnProperty("delayRelations") && options["delayRelations"]) || false;
        $.ajax({
            url: this.serviceEndpoint + endpoint,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: options,
            cache: false,
            type: "GET"
        })
            .done(function (result) {
            if (result.Success && result.Data != undefined) {
                var data = result.Data;
                var entries;
                // schedule async updates
                //control.updateShortcutsAsync(data);
                control.updatePagingAsync(data);
                if ((entries = data.List) == undefined) {
                    control.Clear();
                    control.selectedValue(undefined);
                    $(document).statusBar().AddMessage("Keine Listenwerte erhalten!", StatusBarMessageSeverity.Error);
                    return;
                }
                else {
                    if (entries.length == 0) {
                        $(document).statusBar().AddMessage("Die Liste ist leer - möglicherweise reichen Ihre Rechte nicht aus um die Daten zu sehen.", StatusBarMessageSeverity.Warning);
                    }
                    control.updateList(entries);
                }
                if (selector == undefined && entries.length > 0 && entries[0].hasOwnProperty("Oid")) {
                    selector = entries[0]["Oid"]; // set to topOfList, if no selector is defined
                }
                if (selector != undefined) {
                    if (!control.IsActivationSuppressed)
                        control.SelectedValue = selector;
                    control.ScrollIntoView();
                }
                if (control.ViewModel.hasOwnProperty("Relations")) {
                    if (delayRelations) {
                        control.queryRelations();
                    }
                    else if (includeRelations && result.Relations != undefined) {
                        var relations = [];
                        $.each(result.Relations, function (index, value) {
                            relations.push(new RelationsResult(value));
                        });
                        control.ViewModel.Relations(relations);
                    }
                }
            }
            else {
                $(document).statusBar().AddMessage("Keine Listenwerte erhalten!", StatusBarMessageSeverity.Error);
            }
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
            _this.Clear();
        });
    };
    Object.defineProperty(ListControl.prototype, "firstVisibleColumn", {
        get: function () {
            $.each(this.Columns, function (index, column) {
                if (column.IsVisible)
                    return index;
            });
            return -1;
        },
        enumerable: true,
        configurable: true
    });
    ListControl.prototype.getEmSize = function (element) {
        if (element == undefined) {
            element = document.documentElement;
        }
        var cfs = getComputedStyle(element, 'fontSize');
        var fontSize = cfs.height;
        return parseFloat(fontSize) || 16;
    };
    ListControl.prototype.getKeyName = function () {
        var keyname = "key";
        $.each(this.Columns, function (index, value) {
            if (value != undefined && value.IsKey) {
                return value.Name;
            }
        });
        return keyname;
    };
    ListControl.prototype.getToplevelHeight = function () {
        //if (this.height == undefined) {
        this.height = this.TopLevel.height();
        var bottomMargin = this.TopLevel.css("margin-bottom");
        if (bottomMargin != undefined && bottomMargin !== "") {
            var value = this.convertUnits(bottomMargin);
            this.height -= value;
        }
        //}
        return this.height;
    };
    ListControl.prototype.getToplevelWidth = function () {
        var topLevelWidth = this.TopLevel.width();
        if (this.width == undefined || this.width != topLevelWidth) {
            this.width = this.TopLevel.width();
        }
        return this.width;
    };
    ListControl.prototype.hasPaging = function (which) {
        return this.pagingControl != undefined &&
            (which == undefined || this.pagingPlacement === which);
    };
    ListControl.prototype.hasShortcut = function (which) { return ((this.shortcutPlacement & which) === which); };
    Object.defineProperty(ListControl.prototype, "hasSelectedValue", {
        get: function () {
            return orgAniceModule.IsValidGuid(this.selectedValue());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "isSessionStorageSupported", {
        get: function () {
            if (this.isStorageSupported != undefined)
                return this.isStorageSupported;
            try {
                window.sessionStorage.setItem("Test", "Test");
                window.sessionStorage.removeItem("Test");
                this.isStorageSupported = true;
                return true;
            }
            catch (e) {
                this.isStorageSupported = false;
                return false;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControl.prototype, "lastVisibleColumn", {
        get: function () {
            var num = -1;
            $.each(this.Columns, function (index, column) {
                if (column.IsVisible)
                    num = index;
            });
            return num;
        },
        enumerable: true,
        configurable: true
    });
    ListControl.prototype.layout = function (property) {
        var doHeight = true, doWidth = true;
        if (property != undefined) {
            if (property === "width")
                doHeight = false;
            if (property === "height")
                doWidth = false;
        }
        switch (window.orgAnice.deviceType) {
            case orgAniceModule.DeviceType.Smartphone:
                this.layoutSmartphone(doHeight, doWidth);
                return;
            case orgAniceModule.DeviceType.Tablet:
                this.layoutTablet(doHeight, doWidth);
                return;
        }
        if (doHeight) {
            var height = this.getToplevelHeight() - 2;
            if (this.paneTop != undefined && this.paneTop.is(":visible")) {
                height -= this.paneTop.height();
            }
            if (this.pagingPaneTop != undefined && this.isPagingEnabled) {
                height -= this.pagingPaneTop.height();
            }
            if (this.paneHeaders != undefined && this.paneHeaders.is(":visible")) {
                height -= this.paneHeaders.height();
            }
            if (this.paneBottom != undefined && this.paneBottom.is(":visible")) {
                height -= this.paneBottom.height();
            }
            if (this.pagingPaneBottom != undefined && this.isPagingEnabled) {
                height -= this.pagingPaneBottom.height();
            }
            this.scrollFrame.height(height - 3);
            if (this.shortcutControl != undefined &&
                (this.shortcutPlacement == Placement.Left || this.shortcutPlacement == Placement.Right)) {
                this.shortcutControl.Height = height - 80;
            }
        }
        if (doWidth) {
            var width = this.getToplevelWidth() - 2;
            if (this.paneLeft != undefined && this.paneLeft.is(":visible")) {
                width -= this.paneLeft.width();
            }
            if (this.paneRight != undefined && this.paneRight.is(":visible")) {
                width -= this.paneRight.width();
            }
            if (width < 0)
                width = 0;
            this.scrollFrame.width(width);
            if (this.shortcutControl != undefined &&
                (this.shortcutPlacement == Placement.Top || this.shortcutPlacement == Placement.Bottom)) {
                this.shortcutControl.Width = width;
            }
        }
        //TODO: uncomment/change this, when fixing the header and aking the columns resizable
        //try {
        //	var firstRow = this.scrollFrame.find(".table").children().first();
        //	if (firstRow != undefined) {
        //		this.paneHeaders.children().each((index, child) => {
        //			var element = $(child);
        //			var name = element.attr("id");
        //			if (name != undefined && typeof name == "string") {
        //				var column = firstRow.find("." + name);
        //				if (column != undefined) {
        //					width = column.width();
        //					if (!isNaN(width))
        //						element.width(width);
        //				}
        //			}
        //		});
        //	}
        //} catch (e) {}
    };
    ListControl.prototype.layoutSmartphone = function (doHeight, doWidth) {
        if (doHeight) {
        }
    };
    ListControl.prototype.layoutTablet = function (doHeight, doWidth) {
        var width = this.getToplevelWidth() - 2;
        var scHeight;
        if (doHeight) {
            if (this.shortcutControl != undefined) {
                scHeight = this.topLevel.height();
                if (this.pagingControl != undefined) {
                    scHeight -= this.pagingControl.TopLevel.height();
                }
                this.shortcutControl.Height = Math.floor(scHeight);
                this.shortcutControl.Toplevel.css("top", this.TopLevel.offset().top);
            }
        }
        if (doWidth) {
            this.paneHeaders.width("100%");
            if (this.shortcutControl != undefined) {
                scHeight = this.topLevel.height();
                if (this.pagingControl != undefined) {
                    scHeight -= this.pagingControl.TopLevel.height();
                }
                this.shortcutControl.Height = Math.floor(scHeight);
                width -= this.paneRight.width();
            }
            this.scrollFrame.width(width);
            if (this.pagingControl != undefined)
                this.pagingControl.Width = this.TopLevel.width();
        }
    };
    ListControl.prototype.select = function (args) {
        this.scrollFrame.each(function () {
            $(this).removeClass("active");
        });
        if (args == undefined || typeof args != "object" || !args.hasOwnProperty("oid"))
            return;
        var fireChanged = (args.hasOwnProperty("fireChanged") && args["fireChanged"] === true);
        var oid = args["oid"];
        var active = this.scrollFrame.find("#" + oid);
        if (active != undefined) {
            active.addClass("active");
            if (!fireChanged) {
                this.SuspendSelectionChanged();
            }
            this.selectedValue(oid);
            if (!fireChanged) {
                this.ResumeSelectionChanged();
            }
        }
    };
    ListControl.prototype.setPosition = function (args) {
        if (this.scrollFrame == undefined)
            return;
        var elements = $("#" + this.name + " #scrollFrame ul"), selected;
        switch (args.toLowerCase()) {
            case "top":
                selected = elements.first();
                break;
            case "last":
                selected = elements.last();
                break;
        }
        if (selected == undefined)
            return;
        this.select(selected.attr("id"));
    };
    ListControl.prototype.setSelectedItem = function (target) {
        this.selectedItem = {};
        if (target == undefined || $(target) == undefined) {
            target = this.scrollFrame.find("ul .active");
            if (target == undefined)
                return;
        }
        var children = target.children();
        if (children == undefined || children.length == undefined || children.length == 0)
            return;
        var $this = this;
        $.each(children, function (index, child) {
            var element = $(child);
            if (element == undefined)
                return true; // continue loop
            var colname = element.attr("data-column-name");
            if (colname == undefined)
                return true; // continue loop
            $this.selectedItem[colname] = element.text();
            return true;
        });
    };
    ListControl.prototype.setSessionValue = function (name, value) {
        if (this.isSessionStorageSupported) {
            window.sessionStorage.setItem(name.toLowerCase(), value);
        }
        else {
            $.cookie(window.orgAnice.MakeStorageKey(name), value);
        }
    };
    ListControl.prototype.onActivated = function (oidChanged) {
        var args = {
            Activated: true,
            Value: this.selectedValue()
        };
        this.Activated.trigger(args);
    };
    ListControl.prototype.onControlResized = function (dimension) {
        var _this = this;
        if (this.layoutDeferralTimer != undefined) {
            clearTimeout(this.layoutDeferralTimer);
            this.layoutDeferralTimer = undefined;
        }
        var control = this;
        var dim = dimension;
        this.layoutDeferralTimer = setTimeout(function () {
            _this.layoutDeferralTimer = undefined;
            control.layout(dim);
        }, 100);
    };
    ListControl.prototype.onPropertyChanged = function (property) {
        if (this.suspendPropertyChanged)
            return;
        this.PropertyChanged.trigger(property);
    };
    ListControl.prototype.onReady = function () {
        this.isReadyFired = true;
        this.Ready.FireOnAdd(this.isReadyFired);
        this.Ready.trigger(this);
    };
    ListControl.prototype.onRelationsChanged = function (relations) {
        this.RelationsChanged.trigger(relations);
        this.UpdateRelations.trigger(relations);
    };
    ListControl.prototype.onSelectionChanged = function (value) {
        if (this.suspendSelectionChanged)
            return;
        var args = {
            NewValue: value,
            Origin: this.name
        };
        this.SelectionChanged.trigger(args);
    };
    ListControl.prototype.onUpdateRelations = function (value) {
        this.queryRelations();
    };
    ListControl.prototype.updateList = function (newContent) {
        var results = [];
        var columns = this.Columns;
        var formatters = this.viewModel.Formatters;
        var hasFormatters = formatters != undefined;
        if (this.hasPaging()) {
            this.pagingControl.FirstOid = newContent == undefined || newContent.length == 0 ? undefined : newContent[0]["Oid"];
            this.pagingControl.LastOid = newContent == undefined || newContent.length <= 0 ? undefined : newContent[newContent.length - 1]["Oid"];
        }
        if (newContent != undefined) {
            $.each(newContent, function (index, sourceItem) {
                if (sourceItem == undefined)
                    return true; // continue
                var result = ko.mapping.fromJS(sourceItem);
                //var result = new ListControlModelItem;
                $.each(columns, function (idx, column) {
                    var name = column.Name;
                    if (sourceItem.hasOwnProperty(name)) {
                        var value = sourceItem[name];
                        //result[name] = (hasFormatters && formatters.hasOwnProperty(name))
                        //? formatters[name].Format(value)
                        //: value;
                        result[name]((hasFormatters && formatters.hasOwnProperty(name)) ? formatters[name].Format(value) : value);
                    }
                });
                results.push(result);
            });
        }
        this.viewModel.Items(results);
    };
    ListControl.prototype.updatePagingAsync = function (info) {
        if (info == undefined || !this.hasPaging())
            return;
        setTimeout(this.updatePaging(this, info), 0);
    };
    ListControl.prototype.updatePaging = function (control, info) {
        if (info == undefined || !this.hasPaging())
            return;
        this.pagingControl.Update(info.CountRecords, info.CurrentRecord);
        this.IsPagingEnabled = this.pagingControl.TotalPages > 1;
    };
    ListControl.prototype.updateShortcuts = function (shortcuts, currentShortcut) {
        if (this.shortcutControl != undefined) {
            this.shortcutControl.UpdateShortcuts(shortcuts);
        }
    };
    ListControl.prototype.updateShortcutsAsync = function (shortcuts, currentShortcut) {
        var control = this;
        setTimeout(control.updateShortcuts(shortcuts, currentShortcut), 0);
    };
    ListControl.prototype.onShortcutChanged = function (args) {
        this.currentShortcut = args == undefined ? undefined : (args.hasOwnProperty("length") ? args[0] : args);
        this.QueryByShortcut(this.currentShortcut, this.activeIndex);
        this.ShortcutChangedEvent.trigger(this, this.currentShortcut);
    };
    ListControl.prototype.queryRelations = function () {
        var _this = this;
        if (this.viewModel.Relations == undefined)
            return;
        var options = {
            key: this.selectedValue()
        };
        var control = this;
        $.ajax({
            url: this.serviceEndpoint + "/GetRelations",
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: options,
            //data: JSON.stringify(options),
            cache: false,
            async: false,
            type: "GET"
        })
            .done(function (result) {
            var relations = result.Relations;
            if (relations == undefined || relations.length == 0) {
                _this.ViewModel.Relations([]);
                return;
            }
            control.ViewModel.Relations(relations);
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
        });
    };
    return ListControl;
}());
var ListControlColumn = (function () {
    // --- Methods --------------------------------------------------------
    // --- Constructor ----------------------------------------------------
    function ListControlColumn(name, text, isKey, isVisible, type) {
        var _this = this;
        // --- Events ---------------------------------------------------------
        // --- Fields ---------------------------------------------------------
        this.cssFormatter = function (item) {
            var result = _this.name.toLowerCase();
            if (_this.type == 2 && item != undefined && item.hasOwnProperty(_this.name)) {
                result += (item[_this.name]() ? " booltrue" : " boolfalse");
            }
            return result;
        };
        this.type = 0;
        this.name = name;
        if (text != undefined)
            this.text = text;
        if (isKey != undefined)
            this.isKey = isKey;
        if (isVisible != undefined)
            this.isVisible = isVisible;
        if (type != undefined)
            this.type = type;
    }
    Object.defineProperty(ListControlColumn.prototype, "CssFormatter", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.cssFormatter; },
        enumerable: true,
        configurable: true
    });
    ListControlColumn.prototype.ColumnName = function () { return this.Name; };
    Object.defineProperty(ListControlColumn.prototype, "IsKey", {
        /// <summary>Query if this ListControlColumn is the key column; the default is <c>false</c>.</summary>
        /// <returns><c>true</c> if this is the key column, <c>false</c> if not.</returns>
        get: function () { return this.isKey != undefined && this.isKey; },
        set: function (value) { this.isKey = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControlColumn.prototype, "IsVisible", {
        /// <summary>Query if this ListControlColumn is visible; the default is <c>true</c>.</summary>
        /// <returns><c>true</c> if this column is visible, <c>false</c> if not.</returns>
        get: function () { return this.isVisible == undefined || this.isVisible; },
        set: function (value) { this.isVisible = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControlColumn.prototype, "Name", {
        /// <summary>Gets the name of this column; the name will also be used to index into the values of the parent's viewmodel.</summary>
        /// <returns>A string.</returns>
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControlColumn.prototype, "Header", {
        /// <summary>Gets the text, which will be displayed as the header.</summary>
        /// <returns>A string.</returns>
        get: function () { return this.text == undefined ? this.name : this.text; },
        set: function (value) { this.text = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControlColumn.prototype, "Template", {
        get: function () { return this.template; },
        set: function (value) { this.template = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ListControlColumn.prototype, "Type", {
        get: function () { return this.type; },
        set: function (value) { this.type = value; },
        enumerable: true,
        configurable: true
    });
    return ListControlColumn;
}());
var ListControlModelItem = (function () {
    // --- Constructor ----------------------------------------------------
    function ListControlModelItem(args, formatters) {
        if (args == undefined)
            return;
        var newItem = this;
        $.each(args, function (key, value) {
            newItem.AddProperty(key);
            newItem[key] = formatters != undefined && formatters.hasOwnProperty(key) ? formatters[key].Format(value) : value;
        });
    }
    // --- Properties -----------------------------------------------------
    // --- Methods --------------------------------------------------------
    ListControlModelItem.prototype.AddProperty = function (name) {
        var property = {};
        property[name] = "";
        this.values = this.values || [];
        this.values.push(property);
    };
    return ListControlModelItem;
}());
var ListControlShortcutPanel = (function () {
    // --- Properties -----------------------------------------------------
    // --- Methods --------------------------------------------------------
    // --- Constructor ----------------------------------------------------
    function ListControlShortcutPanel(position, placeholder) {
        // --- Events ---------------------------------------------------------
        // --- Fields ---------------------------------------------------------
        this.position = Placement.Top;
        this.position = position;
        this.placeholder = placeholder;
    }
    // --- Implementation -------------------------------------------------
    ListControlShortcutPanel.prototype.create = function (placement, viewModel) {
        if (this.placeholder == undefined)
            throw Error("The placeholder for this ListControlShortcutPanel element is undefined!");
        var selector = $(this.placeholder);
        if (selector == undefined)
            throw Error("The placeholder for this ListControlShortcutPanel element cannot be found!");
        var horizontal = this.position & Placement.Bottom || this.position & Placement.Top;
        var element = $(this.placeholder);
        element.remove("*");
        var container = $("<ul class='shortcuts'></ul>");
        if (!horizontal)
            container.addClass("vertical");
        element.add(container);
        $.each(viewModel.shortcuts, function (index, value) {
            var scut = $("<li class='shortcut'></li>");
            if (!horizontal)
                scut.addClass("vertical");
            var span = $("<span></span>");
            scut.add(span);
            span.attr("text", value);
            container.add(scut);
        });
    };
    return ListControlShortcutPanel;
}());
// make it a jQuery plug-in
// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function ($) {
    $.fn.listControl = function (arg1, arg2, arg3) {
        if (!this.length)
            return this;
        var defaults = {};
        var count = arguments.length;
        var result = this;
        if (arg1 == undefined) {
            return $(this).data("listControl");
        }
        return this.each(function () {
            var instance = this;
            var $instance = $(instance);
            var name = $instance[0].id;
            var options;
            var control = $instance.data("listControl");
            if (count > 1) {
                var target = arg1;
                var selector = arg2;
                var value = arg3;
                if (target == undefined)
                    throw "target undefined for assignment to '" + name + "'";
                if (control == undefined)
                    throw "Control '" + name + "' must be initialized before actions can be performed on it";
                switch (target.toLowerCase()) {
                    case "action":
                        control.OnAction(selector, value);
                        break;
                    case "control":
                        return (result = control);
                    case "event":
                        if (!control.hasOwnProperty(selector))
                            return $instance;
                        control[selector].add(arg3);
                        return $instance;
                    case "property":
                        if (arg3 == undefined)
                            return (result = selector == undefined ? control : control[selector]);
                        else {
                            control[selector] = arg3;
                        }
                        return $instance;
                }
            }
            else
                options = arg1;
            // duplicate initialization guard
            if ($instance.data("listControl") != null)
                return this;
            var opts = $.extend({}, defaults, options); // combine defaults & options
            var lc = new ListControl(name, $instance, opts); // create & initialize new instance of ListControl
            lc.SuspendPropertyChanged();
            $instance.data("listControl", lc);
            if (lc.IsAutoShow) {
                lc.Show();
            }
            lc.ResumePropertyChanged();
            return result;
        });
    };
})(jQuery || $);
//# sourceMappingURL=orgAnice.listControl.js.map