///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.d.ts"/>
var NavigationControl = (function () {
    // --- Constructor ----------------------------------------------------
    function NavigationControl(placeholder, options) {
        // --- Events ---------------------------------------------------------
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.Ready = new orgAniceModule.orgAniceEvent();
        // --- Fields ---------------------------------------------------------
        this.isReadyFired = false;
        this.orientation = orgAniceModule.Orientation.Horizontal;
        this.propertyChangedSuspended = false;
        this.viewModel = {
            Items: [],
            Orientation: orgAniceModule.Orientation.Horizontal
        };
        if (placeholder == undefined)
            throw "The toplevel element (placeholder in HTML) must be assigned!";
        if (placeholder.length == 0)
            throw "The toplevel element is not a valid jQuery selector!";
        this.topLevel = $(placeholder);
        this.create(options);
    }
    Object.defineProperty(NavigationControl.prototype, "IsReady", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.isReadyFired; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationControl.prototype, "Orientation", {
        get: function () { return this.orientation; },
        set: function (value) {
            this.orientation = value;
            this.onPropertyChanged("Orientation");
        },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    NavigationControl.prototype.Initialize = function (options) {
        this.create(options);
    };
    NavigationControl.prototype.FindByName = function (name) {
        for (var idx = 0; idx < this.viewModel.Items.length; ++idx) {
            var item = this.viewModel.Items[idx];
            if (item != undefined && item.Name == name)
                return item;
        }
        return undefined;
    };
    NavigationControl.prototype.ResumePropertyChanged = function () {
        this.propertyChangedSuspended = false;
    };
    NavigationControl.prototype.SuspendPropertyChanged = function () {
        this.propertyChangedSuspended = true;
    };
    // --- Implementation -------------------------------------------------
    NavigationControl.prototype.create = function (options) {
        var _this = this;
        this.viewModel.Orientation = options.Orientation;
        // protect against early calls, like attaching events and so on
        if (options.Items == undefined || options.Items.length == 0)
            return;
        var container = this.topLevel;
        container.empty();
        this.topLevel.addClass(this.viewModel.Orientation == orgAniceModule.Orientation.Horizontal ? "horizontal-pane" : "vertical-pane");
        var outerBorder = $(document.createElement("div"));
        outerBorder.addClass("outer-border");
        var innerBorder = $(document.createElement("div"));
        innerBorder.addClass("inner-border");
        var frame = $(document.createElement("div"));
        frame.addClass("nav-pane");
        innerBorder.append(frame);
        outerBorder.append(innerBorder);
        container.append(outerBorder);
        $.each(options.Items, function (index, value) {
            var navItem = new NavigationItem(value.Name, value);
            _this.viewModel.Items.push(navItem);
            frame.append(navItem.DomElement);
        });
        this.onReady();
    };
    NavigationControl.prototype.onPropertyChanged = function (property) {
        if (this.propertyChangedSuspended)
            return;
        this.PropertyChanged.trigger(this, property);
    };
    NavigationControl.prototype.onReady = function () {
        this.isReadyFired = true;
        this.Ready.FireOnAdd(this.isReadyFired);
        this.Ready.trigger("");
    };
    return NavigationControl;
}());
var NavigationItem = (function () {
    // --- Constructor ----------------------------------------------------
    function NavigationItem(name, options) {
        var _this = this;
        // --- Events ---------------------------------------------------------
        this.CheckedChanged = new orgAniceModule.orgAniceEvent();
        this.Click = new orgAniceModule.orgAniceEvent();
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.supendEvents = true; // suspend until constructor is called
        this.viewModel = {
            DisplayText: ko.observable(),
            ImagePath: ko.observable(),
            IsVisible: ko.observable(),
            Name: ko.observable(this.name),
            OnClick: function () {
                _this.onClick();
            }
        };
        this.name = name;
        this.eventType = options.EventType;
        this.imagePath = options.Img;
        this.imagePathActive = options.ImgActive;
        this.imagePathDisabled = options.ImgDisabled;
        this.isChecked = options.IsChecked || false;
        this.isEnabled = options.IsEnabled || true;
        this.isToggle = options.IsToggle || false;
        this.IsVisible = options.IsVisible || true;
        this.viewModel.Name(this.name);
        this.viewModel.DisplayText(options.DisplayText == undefined ? this.name : options.DisplayText);
        this.domElement = this.create(options);
        ko.applyBindings(this.viewModel, this.domElement[0]);
        this.onPropertyChangedInternal("IsChecked");
        this.onPropertyChangedInternal("IsEnabled");
        this.supendEvents = false;
    }
    Object.defineProperty(NavigationItem.prototype, "DisplayText", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.viewModel.DisplayText(); },
        set: function (value) {
            this.viewModel.DisplayText(value);
            this.onPropertyChanged("DisplayText");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "DomElement", {
        get: function () { return this.domElement; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "ImagePath", {
        get: function () { return this.imagePath; },
        set: function (value) {
            this.imagePath = value;
            this.onPropertyChanged("ImagePath");
            if (this.IsEnabled && this.viewModel.IsVisible())
                this.viewModel.ImagePath(this.imagePath);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "ImagePathActive", {
        get: function () { return this.imagePathActive; },
        set: function (value) {
            this.imagePathActive = value;
            this.onPropertyChanged("ImagePathActive");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "ImagePathDisabled", {
        get: function () { return this.imagePathDisabled; },
        set: function (value) {
            this.imagePathDisabled = value;
            this.onPropertyChanged("ImagePathDisabled");
            if (!this.isEnabled)
                this.viewModel.ImagePath(this.imagePathDisabled);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "IsChecked", {
        get: function () { return this.isChecked; },
        set: function (value) {
            if (this.isChecked == value)
                return;
            this.isChecked = value;
            this.toggleVisualState(this.isChecked, "active", this.imagePathActive, false);
            this.onPropertyChanged("IsChecked");
            this.onCheckChanged(this.isChecked);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "IsEnabled", {
        get: function () { return this.isEnabled; },
        set: function (value) {
            this.isEnabled = value;
            this.toggleVisualState(this.isEnabled, "disabled", this.imagePathDisabled, true);
            this.onPropertyChanged("IsEnabled");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "IsToggleButton", {
        get: function () { return this.isToggle; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "IsVisible", {
        get: function () { return this.viewModel.IsVisible(); },
        set: function (value) {
            this.viewModel.IsVisible(value);
            this.onPropertyChanged("IsVisible");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "Name", {
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationItem.prototype, "ValidateAction", {
        get: function () { return this.validation; },
        set: function (value) { this.validation = value; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    NavigationItem.prototype.SetEvent = function (options) {
        switch (options.EventName.toLowerCase()) {
            case "click":
                if (typeof options.AddRemove == "boolean" && options.AddRemove)
                    this.Click.add(options.EventFunc);
                else
                    this.Click.remove(options.EventFunc);
                break;
            case "checkchanged":
                if (typeof options.AddRemove == "boolean" && options.AddRemove)
                    this.CheckedChanged.add(options.EventFunc);
                else
                    this.CheckedChanged.remove(options.EventFunc);
                break;
        }
    };
    NavigationItem.prototype.SetProperty = function (options) {
        var instance = this;
        $.each(options, function (index, value) {
            if (instance.viewModel.hasOwnProperty(index)) {
                if (typeof instance.viewModel[index] == "function")
                    instance.viewModel[index].call(instance, value);
                else
                    instance.viewModel[index] = value;
            }
            else if (instance.hasOwnProperty(index)) {
                instance[index] = value;
                instance.onPropertyChanged(index);
            }
            else {
                switch (index.toLowerCase()) {
                    case "displaytext":
                        instance.DisplayText = value;
                        break;
                    case "ischecked":
                        instance.IsChecked = value;
                        break;
                    case "isenabled":
                        instance.IsEnabled = value;
                        break;
                    case "isvisible":
                        instance.IsVisible = value;
                        break;
                    case "name":
                        instance.viewModel.Name(value);
                        break;
                }
            }
        });
    };
    // --- Implementation -------------------------------------------------
    NavigationItem.prototype.create = function (options) {
        var container = $(document.createElement("div"));
        container.addClass("nav-item");
        container.attr("data-bind", "visible: IsVisible, attr: { id: Name }, click: OnClick");
        var imgdiv = $(document.createElement("div"));
        imgdiv.addClass("imgdiv");
        var txt = $(document.createElement("div"));
        txt.addClass("txtdiv");
        txt.attr("data-bind", "text: DisplayText");
        container.append(imgdiv);
        container.append(txt);
        return container;
    };
    NavigationItem.prototype.onCheckChanged = function (value) {
        if (this.supendEvents)
            return;
        this.CheckedChanged.trigger(value);
    };
    NavigationItem.prototype.onClick = function () {
        if (this.supendEvents || !this.isEnabled)
            return;
        // if a validation is assigned, but fails -> do nothing
        if (this.validation != undefined && !this.validation())
            return;
        if (this.IsToggleButton) {
            this.IsChecked = !this.isChecked;
        }
        if (this.eventType != undefined)
            eval("orgAnice.trigger(this.eventType);");
        //TODO: "eval()" is not the way to go! Changed this to a reference to the "orgAnice" namespace
    };
    NavigationItem.prototype.onPropertyChanged = function (property) {
        if (this.supendEvents)
            return;
        this.onPropertyChangedInternal(property);
        this.PropertyChanged.trigger(this, property);
    };
    NavigationItem.prototype.onPropertyChangedInternal = function (property) {
        var imagePath;
        switch (property) {
            case "ImagePathActive":
                break;
            case "IsEnabled":
                imagePath = this.isEnabled
                    ? (this.isToggle && this.isChecked && this.imagePathActive != undefined ? this.imagePathActive : this.imagePath)
                    : (this.imagePathDisabled != undefined ? this.imagePathDisabled : this.imagePath);
                this.viewModel.ImagePath(imagePath);
                break;
            case "IsChecked":
                imagePath = this.isEnabled
                    ? (this.isChecked && this.imagePathActive != undefined ? this.imagePathActive : this.imagePath)
                    : (this.imagePathDisabled != undefined ? this.imagePathDisabled : this.imagePath);
                this.viewModel.ImagePath(imagePath);
                if (this.isChecked)
                    this.domElement.addClass("checked");
                else
                    this.domElement.removeClass("checked");
                break;
        }
    };
    NavigationItem.prototype.toggleVisualState = function (toggle, selector, alternateImage, reverseProperty) {
        this.viewModel.ImagePath(toggle && alternateImage != undefined ? alternateImage : this.imagePath);
        if ((toggle && !reverseProperty) || (!toggle && reverseProperty)) {
            this.domElement.addClass(selector);
            this.domElement.find(".imgdiv").addClass(selector);
            this.domElement.find(".txtdiv").addClass(selector);
        }
        else {
            this.domElement.removeClass(selector);
            this.domElement.find(".imgdiv").removeClass(selector);
            this.domElement.find(".txtdiv").removeClass(selector);
        }
    };
    return NavigationItem;
}());
var NavigationViewModel = (function () {
    // --- Constructor ----------------------------------------------------
    function NavigationViewModel(json) {
        // --- Fields ---------------------------------------------------------
        this.homeOid = ko.observable("");
        this.backOid = ko.observable("");
        this.nextOid = ko.observable("");
        this.currentOid = ko.observable("");
        if (json == undefined)
            return;
        this.backLink = json["BackLink"];
        this.backText = json["BackText"] || "Zurück";
        this.nextLink = json["NextLink"];
        this.nextText = json["NextText"] || "Weiter";
        this.homeLink = json["HomeLink"];
        this.homeText = json["HomeText"];
        this.nextCookieName = json["NextCookieName"] || "NextOid";
        var userCookie = this.UserPrefix;
        var cookie = $.cookie(userCookie + NavigationViewModel.HomeCookieOid);
        this.homeOid(cookie == undefined ? NavigationViewModel.EmptyRequestGuid : cookie);
        cookie = $.cookie(userCookie + NavigationViewModel.CurrentCookieOid);
        this.currentOid(cookie == undefined ? NavigationViewModel.EmptyRequestGuid : cookie);
        cookie = $.cookie(userCookie + NavigationViewModel.BackCookieOid);
        this.backOid(cookie == undefined ? NavigationViewModel.EmptyRequestGuid : cookie);
        cookie = $.cookie(userCookie + NavigationViewModel.NextCookieOid);
        this.nextOid(cookie == undefined ? NavigationViewModel.EmptyRequestGuid : cookie);
        this.homeOid.subscribe(function (oid) {
            $.cookie(userCookie + NavigationViewModel.HomeCookieOid, oid, { path: '/', expires: 7 });
        });
        this.isBackVisible = this.getBoolFromJson("IsBackVisible", true, json);
        this.isHomeVisible = this.getBoolFromJson("IsHomeVisible", true, json);
        this.isNextVisible = this.getBoolFromJson("IsNextVisible", true, json);
    }
    Object.defineProperty(NavigationViewModel.prototype, "CurrentOid", {
        // --- Properties -----------------------------------------------------
        /// <summary>The Oid of the current entity in the active display.</summary>
        /// <returns>A KnockoutObservable&lt;string&gt;</returns>
        get: function () { return this.currentOid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HomeOid", {
        /// <summary>The Oid of the entity, which shall be loaded by the page, where the <i>Home</i> button points to.</summary>
        /// <returns>A KnockoutObservable&lt;string&gt;</returns>
        /// <remarks>Currently ,this is usually the <i>Address</i> page.</remarks>
        get: function () { return this.homeOid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "BackOid", {
        /// <summary>The Oid of the entity, which shall be loaded by the page, where the <i>Back</i> button points to.</summary>
        /// <returns>A KnockoutObservable&lt;string&gt;</returns>
        get: function () { return this.backOid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "NextOid", {
        /// <summary>The Oid of the entity, which shall be loaded by the page, where the <i>Next</i> button points to.</summary>
        /// <returns>A KnockoutObservable&lt;string&gt;</returns>
        get: function () { return this.nextOid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "BackLink", {
        get: function () { return this.backLink; },
        set: function (value) { this.backLink = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "BackText", {
        get: function () { return this.backText; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HomeLink", {
        get: function () { return this.homeLink; },
        set: function (value) { this.homeLink = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HomeText", {
        get: function () { return this.homeText; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "IsBackVisible", {
        get: function () { return this.isBackVisible; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "IsHomeVisible", {
        get: function () { return this.isHomeVisible; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "IsNextVisible", {
        get: function () { return this.isNextVisible; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "NextLink", {
        get: function () { return this.nextLink; },
        set: function (value) { this.nextLink = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "NextText", {
        get: function () { return this.nextText; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "TargetCookieName", {
        get: function () { return this.nextCookieName; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "BackUrl", {
        get: function () {
            return (this.HasBackLink && this.HasBackOid)
                ? this.BackLink // + "/" + this.BackOid()
                : "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HomeUrl", {
        get: function () {
            return (this.HasHomeLink && this.HasHomeOid)
                ? this.HomeLink // + "/" + this.HomeOid()
                : "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "NextUrl", {
        get: function () {
            return (this.HasNextLink && this.HasNextOid)
                ? this.NextLink // + "/" + this.NextOid()
                : "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasBackLink", {
        get: function () { return this.backLink != undefined && this.backLink != ""; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasBackUrl", {
        get: function () {
            return this.HasBackLink && this.HasBackOid && this.BackOid() != NavigationViewModel.EmptyRequestGuid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasBackOid", {
        get: function () {
            return this.backOid() != undefined && this.backOid().length > 0 && this.backOid() != NavigationViewModel.EmptyRequestGuid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasCurrentOid", {
        get: function () {
            return orgAniceModule.IsValidGuid(this.currentOid()) && !orgAniceModule.IsEmptyGuid(this.currentOid());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasHomeLink", {
        get: function () { return this.homeLink != undefined && this.homeLink != ""; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasHomeOid", {
        get: function () { return this.homeOid() != undefined && this.homeOid().length > 0; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasHomeUrl", {
        get: function () {
            return this.HasHomeOid && this.HasHomeLink && this.homeOid() != NavigationViewModel.EmptyRequestGuid;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasNextLink", {
        get: function () { return this.nextLink != undefined && this.nextLink != ""; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasNextUrl", {
        get: function () { return this.HasNextLink && this.HasNextOid; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "HasNextOid", {
        get: function () {
            return !this.IsEmptyGuid(this.nextOid());
            //return this.nextOid() != undefined && this.nextOid().length > 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NavigationViewModel.prototype, "UserPrefix", {
        get: function () {
            var cookie = $.cookie("orgAniceUser");
            return cookie == undefined ? "unknown" : cookie;
        },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    NavigationViewModel.prototype.IsEmptyGuid = function (value) {
        return value == undefined || value === "" || value === NavigationViewModel.EmptyGuid;
    };
    // --- Implementation -------------------------------------------------
    NavigationViewModel.prototype.getBoolFromJson = function (name, fallback, json) {
        var value = json[name];
        return (value != undefined) ? value : fallback;
    };
    // --- Constant Values ------------------------------------------------
    NavigationViewModel.EmptyGuid = "00000000-0000-0000-0000-000000000000";
    NavigationViewModel.EmptyRequestGuid = "00000000-0000-0000-0000-000000000001";
    NavigationViewModel.HomeCookieOid = "addressOid";
    NavigationViewModel.BackCookieOid = "sourceOid";
    NavigationViewModel.NextCookieOid = "targetOid";
    NavigationViewModel.CurrentCookieOid = "currentOid";
    return NavigationViewModel;
}());
// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
    $.fn.orgAniceNavigation = function (arg1, arg2, arg3) {
        if (!this.length)
            return this;
        var defaults = {
            Orientation: orgAniceModule.Orientation.Horizontal
        };
        var count = arguments.length;
        return this.each(function () {
            var instance = this;
            var $instance = $(instance);
            var options;
            var control;
            switch (count) {
                case 0:
                    return;
                case 1:
                    options = $.extend({}, arg1, defaults);
                    control = $instance.data["orgAniceNavigation"];
                    if (control == undefined) {
                        control = new NavigationControl(this, options);
                        $instance.data["orgAniceNavigation"] = control;
                    }
                    else {
                        control.Initialize(options);
                    }
                    break;
                default:
                    var target = arg1;
                    var selector = arg2;
                    var value = arg3;
                    var item;
                    control = $instance.data["orgAniceNavigation"];
                    if (control == undefined) {
                        control = new NavigationControl(this, defaults);
                        $instance.data["orgAniceNavigation"] = control;
                    }
                    switch (target.toLowerCase()) {
                        case "event":
                            if (selector.toLowerCase() == "ready") {
                                control.Ready.add(value);
                                break;
                            }
                            item = control.FindByName(selector);
                            if (item == undefined)
                                return;
                            item.SetEvent(value);
                            break;
                        case "property":
                            selector = selector.replace(/\{/, '').replace(/\}/, '');
                            var items = selector.split(/[ \,\;]/);
                            $.each(items, function (index, navItem) {
                                item = control.FindByName(navItem);
                                if (item != undefined)
                                    item.SetProperty(value);
                            });
                            break;
                    }
                    break;
            }
        });
    };
})(jQuery || $);
// ReSharper restore JsFunctionCanBeConvertedToLambda
//# sourceMappingURL=orgAnice.navigation.js.map