﻿
///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />
///<reference path="orgAnice.statusBar.d.ts"/> />
///<reference path="orgAnice.pagingControl.d.ts"/> />

class PagingMoveEventArgs implements IPagingMoveEventArgs {
    // --- Events ---------------------------------------------------------
    // --- Fields ---------------------------------------------------------
	private targetPage: number;

    // --- Properties -----------------------------------------------------
	public get TargetPage(): number { return this.targetPage; }
	public set TargetPage(value: number) {
		this.TargetPage = value;
	}

    // --- Methods --------------------------------------------------------
    // --- Constructor ----------------------------------------------------
    // --- Implementation -------------------------------------------------
}

class PagingControl implements IPagingControl {
    // --- Events ---------------------------------------------------------
	public Move: orgAniceModule.orgAniceEvent<IPagingMoveEventArgs> = new orgAniceModule.orgAniceEvent<IPagingMoveEventArgs>();

	public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

    // --- Fields ---------------------------------------------------------
	private currentPage: KnockoutObservable<number> = ko.observable<number>();
	private currentRecord: number;
	private firstPage: KnockoutObservable<number> = ko.observable<number>(1);
	private firstOid: string;
	private gotoPositionCommand: (skipPages: number, count: number, oid?: string) => void = undefined;
	private lastPage: KnockoutObservable<number> = ko.observable<number>();
	private lastOid: string;
	private pageSize: number = 50;	// set default to 50
	private suspendPropertyChanged: boolean;
	private toplevel: JQuery;
	public KoTotalPages: KnockoutObservable<number> = ko.observable<number>();
	public KoTotalRecords: KnockoutObservable<number> = ko.observable<number>();

    // --- Properties -----------------------------------------------------
	public get CurrentPage(): number { return this.currentPage(); }

	public set CurrentPage(value: number) {
		this.currentPage(value);
		this.onPropertyChanged("CurrentPage");
	}

	public get CurrentRecord(): number { return this.currentRecord; }

	public set CurrentRecord(value: number) {
		this.currentRecord = value;
		this.onPropertyChanged("CurrentRecord");
	}

	public get FirstPage(): number { return this.firstPage(); }

	public set FirstPage(value: number) {
		this.firstPage(value);
		this.onPropertyChanged("FirstPage");
	}

	public get FirstOid(): string { return this.firstOid; }

	public set FirstOid(value: string) {
		this.firstOid = value;
		this.onPropertyChanged("FirstOid");
	}

	public get LastPage(): number { return this.lastPage(); }

	public set LastPage(value: number) {
		this.lastPage(value);
		this.onPropertyChanged("LastPage");
	}

	public get LastOid(): string { return this.lastOid; }

	public set LastOid(value: string) {
		this.lastOid = value;
		this.onPropertyChanged("LastOid");
	}

	public get PageSize(): number { return this.pageSize; }

	public set PageSize(value: number) {
		this.pageSize = value;
	}

	public get TopLevel(): JQuery { return this.toplevel; }

	public set TopLevel(value: JQuery) {
		this.toplevel = value;
	}

	public get TotalPages(): number { return this.KoTotalPages(); }

	public set TotalPages(value: number) {
		this.KoTotalPages(value);
		this.onPropertyChanged("TotalPages");
	}

	public get TotalRecords(): number { return this.KoTotalRecords(); }

	public set TotalRecords(value: number) {
		this.KoTotalRecords(value);
		this.onPropertyChanged("TotalRecords");
	}

	public get Width(): number {
		return (this.toplevel == undefined) ? 0 : this.toplevel.width();
	}

	public set Width(value: number) {
		if (this.toplevel != undefined)
			this.toplevel.width(value);
	}

	public get GotoPositionCommand(): (skipPages: number, count: number, oid?: string) => void { return this.gotoPositionCommand; }
	public set GotoPositionCommand(value: (skipPages: number, count: number, oid?: string) => void) { this.gotoPositionCommand = value; }


    // --- Methods --------------------------------------------------------
	public Initialize(firstPage: number, countPages: number): void {
	}

	public Update(total: number, current: number): void {
	}

    // --- Constructor ----------------------------------------------------
	constructor(options: {}) {
	}

    // --- Implementation -------------------------------------------------
	private onPropertyChanged(property: string) {
		if (this.suspendPropertyChanged) return;

		this.PropertyChanged.trigger(property);
	}
}

class DesktopPagingControl extends PagingControl {
	// --- Fields ---------------------------------------------------------
	private infoPaneLeft: JQuery;
	private infoPaneRight: JQuery;
	private pagingButtons: number = 5;	// number of paging buttons
	private viewModel: {
		pageButtons: KnockoutObservableArray<JQuery>;
		currentButton: JQuery;	// the currently selected button
	} = {
		pageButtons: ko.observableArray<JQuery>(),
		currentButton: undefined
	};

	// --- Properties -----------------------------------------------------

	// --- Methods --------------------------------------------------------
	public Initialize(firstPage: number, countPages: number) {
		super.Initialize(firstPage, countPages);

		this.reindexPageButtons();
		this.markCurrentPageButton();
	}

	public Update(total: number, current: number): void {
		this.CurrentRecord = current;
		this.TotalRecords = total;
		this.TotalPages = Math.ceil(total / this.PageSize); 
		this.CurrentPage = Math.ceil(current / this.PageSize);

		if (this.TotalPages < this.pagingButtons) {
			this.pagingButtons = this.TotalPages;
			this.FirstPage = 1;
			this.LastPage = this.TotalPages;
		} else {
			// calculate 1st page
			if (this.CurrentPage == 1) {
				this.FirstPage = 1;
				this.LastPage = this.pagingButtons;
			} else {
				var pos = Math.floor(this.pagingButtons / 2);
				while ((this.CurrentPage - pos) <= 0) {
					pos--;
				}
				this.FirstPage = this.CurrentPage - pos;
				this.LastPage = this.FirstPage + this.pagingButtons - 1;

				while (this.LastPage > this.TotalPages) {
					this.LastPage--;
					this.FirstPage--;
				}
			}
		}
		
		this.reindexPageButtons();
		this.markCurrentPageButton();
	}

	// --- Constructor ----------------------------------------------------
	constructor(options: {}) {
		super(options);
		if (options != undefined) {
			if (options.hasOwnProperty("pagingButtons"))
				this.pagingButtons = options["pagingButtons"];
		}

		this.KoTotalPages.subscribe((newvalue) => {
			this.infoPaneLeft.text(newvalue + (newvalue == 1 ? " Seite" : " Seiten"));
		});

		this.KoTotalRecords.subscribe((newValue) => {
			this.infoPaneRight.text(newValue + (newValue == 1 ? "Satz" : " Sätze"));
		});

		this.create();
	}

    // --- Implementation -------------------------------------------------
    private create() {
		this.TopLevel = $(document.createElement("div")).addClass("paging-control");

		this.infoPaneLeft = $(document.createElement("div")).addClass("paging-info-left");
	    this.TopLevel.append(this.infoPaneLeft);

		this.infoPaneRight = $(document.createElement("div")).addClass("paging-info-right");
	    this.TopLevel.append(this.infoPaneRight);

		var pagingButtons = $(document.createElement("div")).addClass("paging-buttons");
		var control = this;

		pagingButtons.append($(document.createElement("div")).attr("id", "toBegin").addClass("paging-button").text("<<").click(() => {
			if (//control.CurrentPage <= 1 ||
				control.GotoPositionCommand == undefined) return;

			control.GotoPositionCommand(-2147483648, this.PageSize);
		}));
		pagingButtons.append($(document.createElement("div")).attr("id", "previous").addClass("paging-button").text("<").click(() => {
			if (//control.CurrentPage <= 1 ||
				control.GotoPositionCommand == undefined) return;

			control.GotoPositionCommand(-1, this.PageSize, this.FirstOid);
		}));

		//var buttons = [];
		//var button: JQuery;
	    //var index = this.FirstPage, idx;
		//for (idx = this.FirstPage; idx <= this.pagingButtons; ++idx) {
		//	button = this.createPagingButton(idx);
		//	pagingButtons.append(button);
		//	buttons.push(button);
		//	index++;
		//}

	    //this.viewModel.pageButtons(buttons);

		pagingButtons.append($(document.createElement("div")).attr("id", "next").addClass("paging-button").text(">").click(() => {
			if (//control.CurrentPage >= control.TotalPages ||
				control.GotoPositionCommand == undefined) return;

			control.GotoPositionCommand(1, this.PageSize, this.LastOid);
		}));
		pagingButtons.append($(document.createElement("div")).attr("id", "toEnd").addClass("paging-button").text(">>").click(() => {
			if (//control.CurrentPage >= control.TotalPages ||
				control.GotoPositionCommand == undefined) return;

			control.GotoPositionCommand(2147483647, this.PageSize);
		}));

		this.TopLevel.append(pagingButtons);
	}

	private createPagingButton(index: number): JQuery {
		var control = this;
		var button = $(document.createElement("div"))
			.text(index)
			.attr("id", "page" + index.toString())
			.addClass("paging-button").text(index.toString())
			.click((eventTarget) => {
				if (eventTarget == undefined || eventTarget.target == undefined) return;
				try {
					var id = Number($(eventTarget.target).attr("id").match(/[0-9]/));
					var target = id - control.CurrentPage;

					var oid = target < 0 ? control.FirstOid : control.LastOid;

					control.GotoPositionCommand(target, control.PageSize, oid);
				} catch (e) {
				}
			});

		return button;
	}

	private markCurrentPageButton(): void {
		var id = "page" + this.CurrentPage;
		$(this.viewModel.pageButtons()).each((index, value) => {
			var jq = $(value);
			jq.removeClass("selected");
			var buttonId = jq.attr("id");
			if (buttonId == id) {
				jq.addClass("selected");
			}
		});

		var button = $(this.viewModel.pageButtons()).find(id);
	}

	private reindexPageButtons(): void {
		var index = this.FirstPage;
		$(this.viewModel.pageButtons()).each((position, button) => {
			$(button)
				.attr("id", "page" + index)
				.text(index);
			index++;
		});
	}
}

class MobilePagingControl extends PagingControl {
	// --- Fields ---------------------------------------------------------
	private infoPane: JQuery;
	private viewModel: {
		pageButtons: KnockoutObservableArray<JQuery>;
	} = {
		pageButtons: ko.observableArray<JQuery>(),
	};

	// --- Properties -----------------------------------------------------

	// --- Methods --------------------------------------------------------
	public Initialize(firstPage: number, countPages: number) {
		super.Initialize(firstPage, countPages);
	}

	public Update(total: number, current: number): void {
		this.CurrentRecord = current;
		this.TotalRecords = total;
		this.TotalPages = Math.ceil(total / this.PageSize);
		this.CurrentPage = Math.ceil(current / this.PageSize);
	}

	// --- Constructor ----------------------------------------------------
	constructor(options: {}) {
		super(options);

		this.KoTotalPages.subscribe((newvalue) => {
			this.infoPane.text(newvalue + (newvalue == 1 ? " Seite" : " Seiten"));
		});

		this.create();
	}

    // --- Implementation -------------------------------------------------
	private create(): void {
		this.TopLevel = $(document.createElement("div")).addClass("paging-control");

		this.infoPane = $(document.createElement("div")).addClass("paging-info").css("display", "none");
		this.TopLevel.append(this.infoPane);

		var control = this;
		var pagingButtons = $(document.createElement("div")).addClass("paging-buttons");

		pagingButtons.append($(document.createElement("div")).attr("id", "toBegin").addClass("paging-button").text("|<")
			.click((ev) => {
				if ( //control.CurrentPage <= 1 ||
					control.GotoPositionCommand == undefined) return;

				ev.stopImmediatePropagation();
				ev.stopPropagation();

				control.GotoPositionCommand(-2147483648, this.PageSize); // value == int.MinValue
			}));

		pagingButtons.append($(document.createElement("div")).attr("id", "previous").addClass("paging-button").text("<")
			.click((ev) => {
				if ( //control.CurrentPage <= 1 ||
					control.GotoPositionCommand == undefined) return;

				ev.stopImmediatePropagation();
				ev.stopPropagation();

				control.GotoPositionCommand(-1, this.PageSize, this.FirstOid);
			}));

		//var infoButton = $(document.createElement("div")).attr("id", "info").addClass("paging-button").text(" ");

		this.infoPane = $(document.createElement("div")).attr("id", "info").addClass("paging-button");

		pagingButtons.append(this.infoPane);

		//infoButton.on("taphold", () => {
		//	control.infoPane.attr("display", "block");
		//	return false;
		//});

		//infoButton
		//	.on("taphold", "#info", () => {
		//		this.infoPane.attr("display", "block");
		//	})
		//	.on("touchstart", "#info", () => {
		//		this.infoPane.attr("display", "block");
		//	})
		//	.on("vmousedown", "#info", () => {
		//		this.infoPane.attr("display", "block");
		//	})
		//	.on("touchend", "#info", () => {
		//		this.infoPane.attr("display", "none");
		//	})
		//	.on("vmouseup", "#info", () => {
		//		this.infoPane.attr("display", "none");
		//	});


			//.click((ev) => {
			//	if ( //control.CurrentPage <= 1 ||
			//		control.GotoPositionCommand == undefined) return;

			//	ev.stopImmediatePropagation();
			//	ev.stopPropagation();

			//	//control.GotoPositionCommand(-1, this.PageSize, this.FirstOid);
			//}));

		pagingButtons.append($(document.createElement("div")).attr("id", "next").addClass("paging-button")
			.text(">")
			.click((ev) => {
				if ( //control.CurrentPage >= control.TotalPages ||
					control.GotoPositionCommand == undefined) return;

				ev.stopImmediatePropagation();
				ev.stopPropagation();

				control.GotoPositionCommand(1, this.PageSize, this.LastOid);
			}));

		pagingButtons.append($(document.createElement("div")).attr("id", "toEnd").addClass("paging-button").text(">|")
			.click((ev) => {
				if ( //control.CurrentPage >= control.TotalPages ||
					control.GotoPositionCommand == undefined) return;

				ev.stopImmediatePropagation();
				ev.stopPropagation();

				control.GotoPositionCommand(2147483647, this.PageSize); // value == int.MaxValue
			}));

		this.TopLevel.append(pagingButtons);
	}
}