﻿
interface IPopupManager {
	// --- Events ---------------------------------------------------------
	// --- Properties -----------------------------------------------------
	Dialogs: { [index: string]: PopupItem };

	// --- Methods --------------------------------------------------------
	AddDialog(json: {}): PopupItem;
	Show(name: string): PopupItem

	// --- Constructor ----------------------------------------------------
	// --- Implementation -------------------------------------------------
}

interface IPopupItem {
	// --- Events ---------------------------------------------------------
	// --- Properties -----------------------------------------------------
	IsAllowScroll: boolean;

	/// <summary>Gets the buttons of the dialog.</summary>
	/// <value>The buttons of the dialog.</value>
	/// <remarks>The buttons will be rendered from left to right in the order they appear in the list.</remarks>
	Buttons: DialogButton[];

	/// <summary>Gets or sets the caption text.</summary>
	/// <value>The caption text.</value>
	CaptionText: string;

	/// <summary>Gets or sets the name of the content; see <b>Remarks</b> for details.</summary>
	/// <value>The name of the content.</value>
	/// <remarks>The <see cref="ContentName"/> is the name of a partial control, which may be rendered inside the dialog.
	/// 		 It will be displayed between the <see cref="Text"/> (if any) and the <see cref="Buttons"/> of the
	/// 		 dialog. If a <b>Content</b> is given, the <see cref="Data"/> will be passed to the partial control.</remarks>
	ContentName: string;
	
	Height: number;

	IsAutoOpen: boolean;

	/// <summary>Gets or sets a value indicating whether the dialog is modal.</summary>
	/// <value><c>true</c> if the dialog is modal, <c>false</c> if not.</value>
	IsModal: boolean;

	/// <summary>Gets or sets a value indicating whether to create a popup dialog.</summary>
	/// <value><c>true</c> if this is a popup dialog, <c>false</c> if not.</value>
	IsPopup: boolean;

	IsResizable: boolean;

	Name: string;

	/// <summary>Gets or sets the text.</summary>
	/// <value>The text.</value>
	Text: string;

	Width: number;

	// --- Methods --------------------------------------------------------
	FromJson(json: {}): PopupItem;

	Initialize();

	Show();
}

interface IDialogButton {
	// --- Events ---------------------------------------------------------
	// --- Properties -----------------------------------------------------
	Command: string;

	IsCloseDialog: boolean;

	IsDefaultButton: boolean;

	Name: string;

	Text: string;

	// --- Methods --------------------------------------------------------
	FromJson(json: {}): DialogButton;
} 