///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.popupManager.d.ts"/>
/// <summary>Manager for dialogs.</summary>
/// <seealso cref="T:IPopupManager"/>
var PopupManager = (function () {
    function PopupManager() {
    }
    Object.defineProperty(PopupManager.prototype, "Dialogs", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.dialogs; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    PopupManager.prototype.AddDialog = function (json) {
        if (json == undefined || !json.hasOwnProperty("Name"))
            return null;
        var name = json["Name"];
        var dialogItem = new PopupItem(name);
        dialogItem.FromJson(json);
        this.dialogs[name] = dialogItem;
        return dialogItem;
    };
    PopupManager.prototype.Show = function (name) {
        var item = this.dialogs[name];
        if (item != undefined)
            item.Show();
        return item;
    };
    return PopupManager;
}());
var PopupItem = (function () {
    // --- Constructor ----------------------------------------------------
    function PopupItem(name, jsonInit) {
        if (jsonInit === void 0) { jsonInit = null; }
        // --- Events ---------------------------------------------------------
        // --- Fields ---------------------------------------------------------
        this.buttons = [];
        this.name = name;
        if (jsonInit != null)
            this.FromJson(jsonInit);
    }
    Object.defineProperty(PopupItem.prototype, "IsAllowScroll", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.isAllowScroll != undefined && this.isAllowScroll; },
        set: function (value) { this.isAllowScroll = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Buttons", {
        /// <summary>Gets the buttons of the dialog.</summary>
        /// <value>The buttons of the dialog.</value>
        /// <remarks>The buttons will be rendered from left to right in the order they appear in the list.</remarks>
        get: function () { return this.buttons; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "CaptionText", {
        /// <summary>Gets or sets the caption text.</summary>
        /// <value>The caption text.</value>
        get: function () { return this.captionText; },
        set: function (text) { this.captionText = text; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "ContentName", {
        /// <summary>Gets or sets the name of the content; see <b>Remarks</b> for details.</summary>
        /// <value>The name of the content.</value>
        /// <remarks>The <see cref="ContentName"/> is the name of a partial control, which may be rendered inside the dialog.
        /// 		 It will be displayed between the <see cref="Text"/> (if any) and the <see cref="Buttons"/> of the
        /// 		 dialog. If a <b>Content</b> is given, the <see cref="Data"/> will be passed to the partial control.</remarks>
        get: function () { return this.contentName; },
        set: function (name) { this.contentName = name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Height", {
        get: function () { return this.height; },
        set: function (height) { this.height = height; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Invoker", {
        get: function () {
            return $.fn[this.invoker];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "IsAutoOpen", {
        get: function () { return this.isAutoOpen != undefined && this.isAutoOpen; },
        set: function (value) { this.isAutoOpen = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "IsModal", {
        /// <summary>Gets or sets a value indicating whether the dialog is modal.</summary>
        /// <value><c>true</c> if the dialog is modal, <c>false</c> if not.</value>
        get: function () { return this.isModal; },
        set: function (isModal) { this.isModal = isModal; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "IsPopup", {
        /// <summary>Gets or sets a value indicating whether to create a popup dialog.</summary>
        /// <value><c>true</c> if this is a popup dialog, <c>false</c> if not.</value>
        get: function () { return this.isPopup; },
        set: function (isPopup) { this.isPopup = isPopup; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "IsResizable", {
        get: function () { return this.isResizable; },
        set: function (value) { this.isResizable = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Name", {
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Text", {
        /// <summary>Gets or sets the text.</summary>
        /// <value>The text.</value>
        get: function () { return this.text; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PopupItem.prototype, "Width", {
        get: function () { return this.width; },
        set: function (width) { this.width = width; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    PopupItem.prototype.FromJson = function (json) {
        var _this = this;
        if (json.hasOwnProperty("CaptionText"))
            this.captionText = json["CaptionText"];
        if (json.hasOwnProperty("ContentName"))
            this.contentName = json["ContentName"];
        if (json.hasOwnProperty("IsModal"))
            this.isModal = json["IsModal"];
        if (json.hasOwnProperty("IsPopup"))
            this.isModal = json["IsPopup"];
        if (json.hasOwnProperty("Name"))
            this.name = json["Name"];
        if (json.hasOwnProperty("Text"))
            this.text = json["Text"];
        if (json.hasOwnProperty("Height")) {
            var height = json["Height"];
            this.height = height != undefined && height > 0 ? height : undefined;
        }
        if (json.hasOwnProperty("Width")) {
            var width = json["Width"];
            this.width = width != undefined && width > 0 ? width : undefined;
        }
        if (json.hasOwnProperty("Buttons")) {
            $.each(json["Buttons"], function (index, value) {
                var button = new DialogButton();
                button.FromJson(value);
                _this.buttons.push(button);
            });
        }
        return this;
    };
    PopupItem.prototype.Initialize = function () {
        var _this = this;
        var root = $("#" + this.name);
        if (root == undefined) {
            //orgAnice.reportError("Dialog " + this.name + " nicht gefunden");
            return;
        }
        var buttons = [];
        if (this.buttons != undefined && this.buttons.length > 0) {
            $.each(this.buttons, function (index, button) {
                var name = "popupButton_" + _this.name + "_" + button.Name;
                var btn = {
                    text: button.Text,
                    click: function () {
                        $(this).dialog("close");
                        if ($.fn[name] == undefined && button.Command != undefined)
                            $.fn[name] = button.Command;
                    }
                };
                buttons.push(btn);
            });
        }
        root.dialog(this.createOptions(buttons));
        //root.dialog({
        //	autoOpen: this.IsAutoOpen,
        //	resizable: true, // this.IsResizable != undefined && this.IsResizable,
        //	modal: this.IsModal == undefined || this.isModal
        //});
        this.invoker = "$.fn.openPopup_" + this.name;
        $.fn[this.invoker] = function () { eval("$(\"# + this.name\").dialog(\"open\");"); };
    };
    PopupItem.prototype.Show = function () {
        eval("$(\"#" + this.name + "\").dialog(\"open\");");
    };
    // --- Implementation -------------------------------------------------
    PopupItem.prototype.createOptions = function (buttons) {
        var _this = this;
        var options = {};
        options["autoOpen"] = this.IsAutoOpen;
        options["resizable"] = true;
        if (this.height != undefined && this.height > 0)
            options["height"] = this.height;
        if (this.width != undefined && this.width > 0)
            options["width"] = this.width;
        if (this.buttons.length > 0)
            options["buttons"] = buttons;
        if (!this.IsAllowScroll) {
            options["open"] = function () {
                $("#" + _this.Name).css("overflow", "hidden");
            };
        }
        return options;
    };
    PopupItem.prototype.toLowerCamelCase = function (value) {
        if (value == undefined)
            return value;
        var result = value[0].toLowerCase();
        result += value.substring(1);
        return result;
    };
    return PopupItem;
}());
var DialogButton = (function () {
    function DialogButton() {
    }
    Object.defineProperty(DialogButton.prototype, "Command", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.command; },
        set: function (cmd) { this.command = cmd; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogButton.prototype, "IsCloseDialog", {
        get: function () { return this.isCloseDialog != undefined && this.isCloseDialog; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogButton.prototype, "IsDefaultButton", {
        get: function () { return this.isDefaultButton != undefined && this.isDefaultButton; },
        set: function (value) { this.isDefaultButton = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogButton.prototype, "Name", {
        get: function () {
            var name = this.name;
            if (name == undefined || name.length == 0)
                this.name = name = this.Text || "__X__";
            return name.replace(" ", "_");
        },
        set: function (value) { this.name = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DialogButton.prototype, "Text", {
        get: function () { return this.text; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    DialogButton.prototype.FromJson = function (json) {
        if (json.hasOwnProperty("Text"))
            this.text = json["Text"];
        if (json.hasOwnProperty("IsCloseDialog"))
            this.isCloseDialog = json["IsCloseDialog"];
        if (json.hasOwnProperty("IsDefaultButton"))
            this.isDefaultButton = json["IsDefaultButton"];
        return this;
    };
    return DialogButton;
}());
//# sourceMappingURL=orgAnice.popupManager.js.map