﻿
interface IRibbonItemOptions {
    // --- Properties -----------------------------------------------------
    DisplayText: string;
    EventType?: string;
    ImagePath?: string;
	ImagePathDisabled?: string;
	IgnoreFocusLost: boolean;
    IsActive: boolean;
    IsEnabled: boolean;
    IsVisible: boolean;
    Name: string;
    Tooltip?: string;
}

interface IRibbonGroupOptions {
    // --- Properties -----------------------------------------------------
    IsTextVisible: boolean;
    Items: IRibbonItemOptions[];
    Name: string;
}

interface IRibbonManagerOptions {
    Groups: IRibbonGroupOptions[];
}

interface IRibbonItem {
    // --- Properties -----------------------------------------------------
	ActivateOnClick: boolean;

    /// <summary>Name of the command, which will be fired, when the item is clicked.
    ///		This will follow the notation: <literal>_<parent name>_<name>, where <literal> is defined as 'ribbon',
    ///		and the entire string is converted to lower case. Thus, the RibbonItem with name 'Save' in the ribbon
    ///		group 'EditNavigation' will trigger the event: 'ribbon_editnavigation_save'.
    /// </summary>
    EventType: string;
    ///// <summary>Full pathname of the image file, which shall be deisplayed, if the item is disabled.</summary>
    //DisabledImagePath: string;
    /// <summary>The text to be displayed with the image.</summary>
    DisplayText: string;
    /// <summary><c>true</c> if this object has a <see href="ValidationCallback" /> method assigned.</summary>
	HasValidator: boolean;
	/// <summary>Gets or sets a value indicating whether focus lost event shall be ignored,
	/// 		 if fired by any origin when targeting this entity.</summary>
	IgnoreFocusLost: boolean;
    /// <summary><c>true</c> if this object is active; <c>false</c> otherwise.</summary>
    IsActive: boolean;
    /// <summary><c>true</c> if this object is disabled; <c>false</c> otherwise.</summary>
    IsDisabled: boolean;
    /// <summary><c>true</c> if this object is visible; <c>false</c> otherwise.</summary>
    IsVisible: boolean;
    /// <summary>The name of the instance.</summary>
    Name: string;

    ValidationCallback: (options?: any) => boolean;

    // --- Methods --------------------------------------------------------
	AttachEvent(options: {});

    SetProperty(values: {});
}


interface IRibbonGroup {
    // --- Events ---------------------------------------------------------
    // --- Properties -----------------------------------------------------
    IsDocumentModified: boolean;
    Items: { [index: string]: IRibbonItem };
    Name: string;
    TopLevel: JQuery;

    // --- Methods --------------------------------------------------------
	AttachEvent(itemNames: string, options: {});
    SetProperty(itemNames: string, values: {}): void;
}

/// <summary>A wrapper is used to insert an external control into the ribbon.
//           It extends IRibbonGroup and inserts a single IRibbonItem (itself).
///          </summary>
/// <remarks>The wrapper accepts a single HTML element and integrates this element into the ribbon.
///          It allows to define, where the element shall be inserted (position) and whether a separator itar shown.
///          See the search control.</remarks >
interface IRibbonWrapper extends IRibbonGroup {
    /// <summary>The element to be wrapped.</summary>
    Element: HTMLElement;
    /// <summary>The position in the ribbon, counted from left to right.</summary>
    Position: Number;
    /// <summary>Defines the separator, if any.</summary>
    /// <remarks>Possible values are: left, right, both, none</remarks>
    Separator: string;
}

interface IRibbonManager {
    // --- Events ---------------------------------------------------------
    PropertyChanged: orgAniceModule.orgAniceEvent<string>;
    Ready: orgAniceModule.orgAniceEvent<void>;

    // --- Properties -----------------------------------------------------
    IsModified: boolean;

    // --- Methods --------------------------------------------------------
    Add(name: string, imagePath: string): IRibbonGroup;
    AttachEvent(selector: string, options: {});
    DetachEvent(selecter: string, options: {});
    FindGroup(name: string): IRibbonGroup;
    //SetDisabledItemsByState(name: string, state: string): any;
    SetAction(arg2: string, arg3: {}): void;
    SetProperty(selector: string, values: {});
}
