///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.ribbon.d.ts"/>
var DialogState;
(function (DialogState) {
    DialogState[DialogState["List"] = 0] = "List";
    DialogState[DialogState["Display"] = 1] = "Display";
    DialogState[DialogState["New"] = 2] = "New";
    DialogState[DialogState["Edit"] = 3] = "Edit";
    DialogState[DialogState["Delete"] = 4] = "Delete";
})(DialogState || (DialogState = {}));
var RibbonManager = (function () {
    // --- Constructor ----------------------------------------------------
    function RibbonManager(topLevel, options) {
        // --- Events ---------------------------------------------------------
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.Ready = new orgAniceModule.orgAniceEvent();
        // --- Fields ---------------------------------------------------------
        this.ribbonGroups = [];
        this.isModified = false;
        this.isReadyFired = false;
        this.topLevel = this.create(topLevel, options);
        this.onReady();
    }
    Object.defineProperty(RibbonManager.prototype, "Groups", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.ribbonGroups; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonManager.prototype, "IsModified", {
        get: function () { return this.isModified; },
        set: function (value) {
            if (this.isModified === value)
                return;
            this.isModified = value;
            this.onPropertyChanged("IsModified");
        },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    RibbonManager.prototype.Add = function (name, imagePath) {
        var newGroup = new RibbonGroup(name, imagePath);
        this.ribbonGroups.push(newGroup);
        return newGroup;
    };
    RibbonManager.prototype.AttachEvent = function (selector, options) {
        var instance = this;
        if (selector == undefined || selector == "") {
            // set property on RibbonManager
            $.each(options, function (name, value) {
                switch (name.toLowerCase()) {
                    case "propertychanged":
                        instance.PropertyChanged.add(value);
                        break;
                    case "ready":
                        instance.Ready.add(value);
                        break;
                }
            });
            return;
        }
        var groupSelectors = selector.split(",");
        $.each(groupSelectors, function (index, groupSelector) {
            var sepIndex = groupSelector.indexOf(":");
            var groupName = sepIndex == -1 ? groupSelector : groupSelector.substr(0, sepIndex);
            var itemSelector = sepIndex == -1 ? "" : groupSelector.substr(sepIndex + 1).trim().replace("{", "").replace("}", "");
            var group = instance.FindGroup(groupName);
            if (group != undefined) {
                group.AttachEvent(itemSelector, options);
            }
        });
    };
    RibbonManager.prototype.DetachEvent = function (selecter, options) {
        throw "Not implemented, yet";
    };
    RibbonManager.prototype.FindGroup = function (name) {
        var foundGroup;
        $.each(this.ribbonGroups, function (index, group) {
            if (group.Name.toLowerCase() == name) {
                foundGroup = group;
                return;
            }
        });
        return foundGroup;
    };
    RibbonManager.GetNavItem = function (item) {
        if (item == undefined)
            return undefined;
        var data = undefined;
        var loop = 3;
        // ReSharper disable once QualifiedExpressionMaybeNull
        while (loop-- >= 0 && (data = item.data("ribbonItemData")) == undefined) {
            item = item.parent();
        }
        return data;
    };
    //public SetDisabledItemsByState(name: string, state: string): void {
    //	if (this.ribbonGroups.hasOwnProperty(name)) {
    //		this.ribbonGroups[name].SetDisabledItemsByState(state);
    //		return;
    //	}
    //	this.ribbonGroups.forEach(group => { group.SetDisabledItemsByState(state); });
    //}
    RibbonManager.prototype.SetProperty = function (selector, values) {
        // editnavigation:{save delete undo}, editNavigation
        var instance = this;
        if (selector == undefined || selector == "") {
            // set property on RibbonManager
            $.each(values, function (name, value) {
                switch (name.toLowerCase()) {
                    case "ismodified":
                        instance.IsModified = value;
                        break;
                }
            });
            return;
        }
        var groupSelectors = selector.split(",");
        $.each(groupSelectors, function (index, groupSelector) {
            var sepIndex = groupSelector.indexOf(":");
            var groupName = sepIndex == -1 ? groupSelector : groupSelector.substr(0, sepIndex);
            var itemSelector = sepIndex == -1 ? "" : groupSelector.substr(sepIndex + 1).trim().replace("{", "").replace("}", "");
            var group = instance.FindGroup(groupName);
            if (group != undefined) {
                group.SetProperty(itemSelector, values);
            }
        });
    };
    RibbonManager.prototype.SetAction = function (selector, options) {
        switch (selector.toLowerCase()) {
            case "add":
                this.addGroup(options);
                break;
        }
    };
    // --- Implementation -------------------------------------------------
    RibbonManager.prototype.addGroup = function (args) {
        var position = Number.MAX_VALUE, name, separator = "none", element = undefined;
        $.each(args, function (index, value) {
            switch (index.toLowerCase()) {
                case "element":
                    element = value;
                    break;
                case "name":
                    name = value;
                    break;
                case "position":
                    position = value;
                    break;
                case "separator":
                    separator = value;
                    break;
            }
        });
        // ReSharper disable once ConditionIsAlwaysConst
        if (element == undefined)
            return; // nothing to add
        var group = new RibbonWrapper({
            element: element,
            name: name,
            position: position,
            separator: separator
        });
        if (position >= this.ribbonGroups.length) {
            this.ribbonGroups.push(group);
            if (separator === "left" || separator === "both")
                this.topLevel.append($(document.createElement("div")).addClass("separator").text(" "));
            this.topLevel.append(group.TopLevel);
            if (separator === "right" || separator === "both")
                this.topLevel.append($(document.createElement("div")).addClass("separator").text(" "));
        }
        else if (position == 0) {
            this.ribbonGroups.unshift(group);
            if (separator === "right" || separator === "both")
                this.topLevel.prepend($(document.createElement("div")).addClass("separator").text(" "));
            this.topLevel.prepend(group.TopLevel);
            if (separator === "left" || separator === "both")
                this.topLevel.prepend($(document.createElement("div")).addClass("separator").text(" "));
        }
        else {
            this.ribbonGroups.splice(position, 0, group);
        }
    };
    RibbonManager.prototype.create = function (topLevel, options) {
        var instance = this;
        $.each(options.Groups, function (index, value) {
            var group = new RibbonGroup(value);
            instance.ribbonGroups.push(group);
            topLevel.append(group.TopLevel);
        });
        return topLevel;
    };
    RibbonManager.prototype.onPropertyChanged = function (property) {
        this.PropertyChanged.trigger(this, property);
    };
    RibbonManager.prototype.onReady = function () {
        this.isReadyFired = true;
        this.Ready.FireOnAdd(this.isReadyFired);
        this.Ready.trigger("");
    };
    RibbonManager.RibbonState = {
        Normal: "Normal",
        Modified: "Modified",
        New: "New",
    };
    return RibbonManager;
}());
var RibbonWrapper = (function () {
    // --- Constructor ----------------------------------------------------
    function RibbonWrapper(options) {
        this.position = Number.MAX_VALUE;
        this.separator = "none";
        var $this = this;
        $.each(options, function (index, value) {
            switch (index.toLowerCase()) {
                case "element":
                    $this.element = value;
                    break;
                case "name":
                    $this.name = value;
                    break;
                case "position":
                    $this.position = value;
                    break;
                case "separator":
                    $this.separator = value;
                    break;
            }
        });
        var itemOpts = {
            DisplayText: this.name,
            Name: this.name,
            IgnoreFocusLost: false,
            IsActive: true,
            IsEnabled: true,
            IsVisible: true,
        };
        this.item = new RibbonItem(itemOpts, this);
        this.toplevel = $(this.element);
        this.toplevel.addClass("ribbon-group");
    }
    Object.defineProperty(RibbonWrapper.prototype, "IsDocumentModified", {
        // --- Properties -----------------------------------------------------
        get: function () { return false; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "Element", {
        get: function () { return this.element; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "Items", {
        get: function () {
            var group = {};
            group[this.name] = this.item;
            return group;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "Name", {
        get: function () { return this.name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "Position", {
        get: function () { return this.position; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "Separator", {
        get: function () { return this.separator; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonWrapper.prototype, "TopLevel", {
        get: function () { return this.toplevel; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    RibbonWrapper.prototype.AttachEvent = function (itemNames, options) { };
    RibbonWrapper.prototype.SetProperty = function (itemNames, values) {
        if (this.item == undefined)
            return;
        if (itemNames == undefined || (typeof itemNames == "string" && (itemNames == "" || itemNames == this.name)))
            this.item.SetProperty(values);
    };
    return RibbonWrapper;
}());
var RibbonGroup = (function () {
    // --- Constructor ----------------------------------------------------
    function RibbonGroup(options, imageBasePath) {
        this.items = {};
        if (arguments.length > 1) {
            this.imageBasePath = imageBasePath;
            this.name = name;
        }
        else
            this.topLevel = this.create(options);
    }
    Object.defineProperty(RibbonGroup.prototype, "ImageBasePath", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.imageBasePath; },
        set: function (value) { this.imageBasePath = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonGroup.prototype, "Items", {
        get: function () { return this.items; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonGroup.prototype, "IsDocumentModified", {
        set: function (value) { this.isDocumentModified = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonGroup.prototype, "Name", {
        get: function () { return this.name; },
        set: function (value) { this.name = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonGroup.prototype, "TopLevel", {
        get: function () { return this.topLevel; },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    RibbonGroup.prototype.Add = function (name, item) {
        this.items[name] = item;
        item.Parent = this;
        return this;
    };
    RibbonGroup.prototype.AttachEvent = function (itemNames, options) {
        var names = itemNames.split(" ");
        var instance = this;
        $.each(names, function (index, value) {
            $.each(instance.items, function (name, item) {
                if (name.toLowerCase() == value)
                    item.AttachEvent(options);
            });
        });
    };
    RibbonGroup.prototype.CreateImagePath = function (imagePath) {
        return this.imageBasePath + "/" + imagePath;
    };
    RibbonGroup.prototype.SetProperty = function (itemNames, values) {
        var names = itemNames.split(" ");
        var instance = this;
        $.each(names, function (index, value) {
            $.each(instance.items, function (name, item) {
                if (name.toLowerCase() == value)
                    item.SetProperty(values);
            });
        });
    };
    // --- Implementation -------------------------------------------------
    RibbonGroup.prototype.create = function (options) {
        var _this = this;
        this.name = options.Name;
        var container = $(document.createElement("div"));
        container.addClass("ribbon-group");
        if (this.name != undefined)
            container.attr("id", this.name);
        $.each(options.Items, function (index, value) {
            var item = new RibbonItem(value, _this);
            _this.items[item.Name] = item;
            container.append(item.Container);
        });
        return container;
    };
    Object.defineProperty(RibbonGroup.prototype, "isDocumentModified", {
        set: function (isModified) {
            this.isModified = isModified;
            $.each(this.items, function (index, value) {
                value.IsEnabled = !(value.IsDisabledOnModified && isModified);
            });
        },
        enumerable: true,
        configurable: true
    });
    return RibbonGroup;
}());
var RibbonItem = (function () {
    // --- Constructor ----------------------------------------------------
    function RibbonItem(options, parent) {
        var _this = this;
        this.ignoreFocusLost = false;
        this.viewModel = {
            DisplayText: ko.observable(),
            IsActive: ko.observable(false),
            IsDisabled: ko.observable(false),
            IsVisible: ko.observable(true),
            Name: ko.observable(),
            OnClick: function () {
                _this.onClick();
            }
        };
        // --- Events ---------------------------------------------------------
        this.OnActivatedChanged = new TypedEvent;
        this.OnDisabledChanged = new TypedEvent;
        this.OnVisibleChanged = new TypedEvent;
        this.parent = parent;
        this.ignoreFocusLost = options.hasOwnProperty("IgnoreFocusLost") && options["IgnoreFocusLost"];
        this.viewModel.Name(options.Name);
        this.viewModel.DisplayText(options.DisplayText);
        //this.imagePath = options.ImagePath;
        //this.disabledImagePath = options.ImagePathDisabled;
        this.viewModel.IsActive(options.IsActive || false);
        this.viewModel.IsDisabled((options.IsEnabled != undefined && !options.IsEnabled) || false);
        this.viewModel.IsVisible(options.IsVisible || true);
        this.container = this.create(this.parent.Name);
        this.viewModel.IsVisible.subscribe(function (newvalue) {
            if (newvalue === false)
                _this.container.css("display", "none");
            else {
                _this.container.css("display", "inline-block");
            }
        });
        this.viewModel.IsActive.subscribe(function (newvalue) {
            _this.toggleVisualState(newvalue, "active", false);
            _this.OnActivatedChanged.trigger(newvalue);
        });
    }
    Object.defineProperty(RibbonItem.prototype, "ActivateOnClick", {
        // --- Properties -----------------------------------------------------
        get: function () { return this.isActivateOnClick; },
        set: function (value) {
            this.isActivateOnClick = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "CommandName", {
        get: function () {
            return ("ribbon_" + this.parent.Name + "_" + this.EventType).toLowerCase();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "Container", {
        get: function () { return this.container; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "DisplayText", {
        //public get DisabledImagePath() { return this.disabledImagePath; }
        //public set DisabledImagePath(value: string) { this.disabledImagePath = value; }
        get: function () {
            return (this.viewModel.DisplayText() == undefined || this.viewModel.DisplayText() == "") ? this.Name : this.viewModel.DisplayText();
        },
        set: function (value) {
            this.viewModel.DisplayText(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "EventType", {
        get: function () { return (this.eventType || this.Name).toLowerCase(); },
        set: function (value) { this.eventType = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "HasValidator", {
        get: function () { return this.validationCallback != undefined; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "IgnoreFocusLost", {
        get: function () { return this.ignoreFocusLost; },
        set: function (value) {
            this.ignoreFocusLost = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "IsActive", {
        get: function () { return this.viewModel.IsActive() || false; },
        set: function (value) {
            this.viewModel.IsActive(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "IsDisabled", {
        get: function () { return this.viewModel.IsDisabled() || false; },
        set: function (value) {
            this.viewModel.IsDisabled(value);
            this.toggleVisualState(value, "disabled", false);
            this.OnDisabledChanged.trigger(this.viewModel.IsDisabled());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "IsDisabledOnModified", {
        get: function () { return this.IsDisabledOnModified; },
        set: function (value) { this.IsDisabledOnModified = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "IsVisible", {
        get: function () { return this.viewModel.IsVisible() || true; },
        set: function (value) {
            this.viewModel.IsVisible(value);
            this.OnVisibleChanged.trigger(this.viewModel.IsVisible());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "Name", {
        get: function () { return this.viewModel.Name(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "Parent", {
        get: function () { return this.parent; },
        set: function (value) { this.parent = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RibbonItem.prototype, "ValidationCallback", {
        get: function () { return this.validationCallback; },
        set: function (value) {
            this.validationCallback = value;
        },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    RibbonItem.prototype.AttachEvent = function (options) {
        if (options.hasOwnProperty("OnActivatedChanged")) {
            this.OnActivatedChanged.add(options["OnActivatedChanged"]);
        }
    };
    RibbonItem.prototype.SetProperty = function (values) {
        var instance = this;
        $.each(values, function (index, value) {
            if (instance.viewModel.hasOwnProperty(index)) {
                if (typeof instance.viewModel[index] == "function")
                    instance.viewModel[index].call(instance, value);
                else
                    instance.viewModel[index] = value;
            }
            else if (instance.hasOwnProperty(index)) {
                instance[index] = value;
            }
            else {
                switch (index.toLowerCase()) {
                    case "activateonclick":
                        instance.ActivateOnClick = value;
                        break;
                    case "isactive":
                        instance.IsActive = value;
                        break;
                    case "isdisabled":
                        instance.IsDisabled = value;
                        break;
                    case "isenabled":
                        instance.IsDisabled = !value;
                        break;
                    case "isvisible":
                        instance.IsVisible = value;
                        break;
                    case "name":
                        instance.viewModel.Name(value);
                        break;
                    case "validationcallback":
                        instance.validationCallback = value;
                        break;
                }
            }
        });
    };
    // --- Implementation -------------------------------------------------
    //<div class="nav-item active checked" id = "List" data - bind = "visible: IsVisible, attr: { id: Name }, click: OnClick" >
    //	<div class="imgdiv active" >< / div >
    //	<div class="txtdiv active" data - bind = "text: DisplayText" > Liste </div >
    //</div >
    RibbonItem.prototype.create = function (groupName) {
        var _this = this;
        var container = $(document.createElement("div"));
        container.attr("id", groupName + "_" + this.Name);
        container.addClass("nav-item");
        if (this.ignoreFocusLost)
            container.addClass("ignoreFocusLost");
        container.click(function () {
            _this.onClick();
        });
        if (!this.IsVisible)
            container.css("display", "none");
        var inner = $(document.createElement("div"));
        inner.addClass("ribbon-item");
        this.domElement = inner;
        var imgdiv = $(document.createElement("div"));
        imgdiv.addClass("ribbon-item-image");
        var txtdiv = $(document.createElement("div"));
        txtdiv.addClass("ribbon-item-text");
        txtdiv.text(this.DisplayText);
        if (this.IsDisabled) {
            container.addClass("disabled");
            inner.addClass("disabled");
            imgdiv.addClass("disabled");
            txtdiv.addClass("disabled");
        }
        inner.append(imgdiv);
        inner.append(txtdiv);
        container.append(inner);
        container.data("ribbonItemData", this);
        return container;
    };
    RibbonItem.prototype.onClick = function () {
        if (this.IsDisabled)
            return;
        // if a validation is assigned, but fails -> do nothing
        if (this.HasValidator && !this.validationCallback())
            return;
        if (this.isActivateOnClick) {
            this.IsActive = !this.IsActive;
        }
        var command = this.CommandName;
        eval("orgAnice.trigger('" + command + "');");
        //TODO: "eval()" is not the way to go! Changed this to a reference to the "orgAnice" namespace
    };
    RibbonItem.prototype.toggleVisualState = function (toggle, selector, reverseProperty) {
        if ((toggle && !reverseProperty) || (!toggle && reverseProperty)) {
            this.domElement.addClass(selector);
            this.domElement.find(".ribbon-item-image").addClass(selector);
            this.domElement.find(".ribbon-item-text").addClass(selector);
        }
        else {
            this.domElement.removeClass(selector);
            this.domElement.find(".ribbon-item-image").removeClass(selector);
            this.domElement.find(".ribbon-item-text").removeClass(selector);
        }
    };
    return RibbonItem;
}());
// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
    $.fn.orgAniceRibbon = function (arg1, arg2, arg3) {
        if (!this.length)
            return this;
        var count = arguments.length;
        var retval = undefined;
        var fval = this.each(function () {
            var instance = this;
            var $instance = $(instance);
            var options;
            var control = $instance.data["orgAniceRibbon"];
            var target, selector, value;
            switch (count) {
                case 0:
                    return;
                case 1:
                    options = arg1;
                    if (control == undefined) {
                        control = new RibbonManager($instance, options);
                        $instance.data["orgAniceRibbon"] = control;
                    }
                    else {
                    }
                    break;
                case 2:
                    target = arg1;
                    selector = arg2;
                    switch (target.toLowerCase()) {
                        case "property":
                            switch (selector.toLowerCase()) {
                                case "ismodified":
                                    retval = control.IsModified;
                                    break;
                                case "isenabled":
                                    retval = control.IsModified;
                                    break;
                            }
                            break;
                    }
                    break;
                default:
                    target = arg1;
                    selector = arg2;
                    value = arg3;
                    if (typeof value == "string") {
                        switch (value.toLowerCase()) {
                            case "ismodified":
                                retval = control.IsModified;
                                break;
                        }
                    }
                    else {
                        switch (target.toLowerCase()) {
                            case "action":
                                control.SetAction(arg2, arg3);
                                break;
                            case "event":
                            case "event:attach":
                                control.AttachEvent(selector, value);
                                //switch (selector.toLowerCase()) {
                                //	case "ready":
                                //		control.Ready.add(value);
                                //		break;
                                //	case "":
                                //		break;
                                //}
                                break;
                            case "event:detach":
                                control.DetachEvent(selector, value);
                                //switch (selector.toLowerCase()) {
                                //	case "ready":
                                //		control.Ready.remove(value);
                                //		break;
                                //}
                                break;
                            case "property":
                                control.SetProperty(selector, value);
                                break;
                        }
                    }
                    break;
            }
        });
        return retval == undefined ? fval : retval;
    };
})(jQuery || $);
// ReSharper restore JsFunctionCanBeConvertedToLambda
//# sourceMappingURL=orgAnice.ribbon.js.map