﻿
///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.ribbon.d.ts"/>

enum DialogState {
	List = 0,
	Display = 1,
	New = 2,
	Edit = 3,
	Delete = 4
}

class RibbonManager implements IRibbonManager {
	// --- Events ---------------------------------------------------------
	public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();
	public Ready: orgAniceModule.orgAniceEvent<void> = new orgAniceModule.orgAniceEvent<void>();

	// --- Fields ---------------------------------------------------------
	private ribbonGroups: IRibbonGroup[] = [];
	private isModified: boolean = false;
	private isReadyFired: boolean = false;
	private topLevel: any;

	// --- Properties -----------------------------------------------------
	public get Groups() { return this.ribbonGroups; }

	public get IsModified(): boolean { return this.isModified; }

	public set IsModified(value: boolean) {
		if (this.isModified === value) return;

		this.isModified = value;
		this.onPropertyChanged("IsModified");
	}

	public static RibbonState: {} = {
		Normal: "Normal",
		Modified: "Modified",
		New: "New",
	}

	// --- Methods --------------------------------------------------------
	public Add(name: string, imagePath: string): IRibbonGroup {
		var newGroup = new RibbonGroup(name, imagePath);
		this.ribbonGroups.push(newGroup);

		return newGroup;
	}

	public AttachEvent(selector: string, options: {}): void {
		var instance = this;
		if (selector == undefined || selector == "") {
			// set property on RibbonManager
			$.each(options, (name, value) => {
				switch (name.toLowerCase()) {
					case "propertychanged":
						instance.PropertyChanged.add(value);
						break;
					case "ready":
						instance.Ready.add(value);
						break;
				}
			});
			return;
		}

		var groupSelectors = selector.split(",");
		$.each(groupSelectors, (index, groupSelector) => {
			var sepIndex = groupSelector.indexOf(":");
			var groupName = sepIndex == -1 ? groupSelector : groupSelector.substr(0, sepIndex);
			var itemSelector = sepIndex == -1 ? "" : groupSelector.substr(sepIndex + 1).trim().replace("{", "").replace("}", "");

			var group = instance.FindGroup(groupName);
			if (group != undefined) {
				group.AttachEvent(itemSelector, options);
			}
		});
	}

	public DetachEvent(selecter: string, options: {}): void {
		throw "Not implemented, yet";
	}

	public FindGroup(name: string): IRibbonGroup {
		var foundGroup: IRibbonGroup;
		$.each(this.ribbonGroups, (index, group) => {
			if (group.Name.toLowerCase() == name) {
				foundGroup = group;
				return;
			}
		});
		return foundGroup;
	}

	public static GetNavItem(item: JQuery): IRibbonItem {
		if (item == undefined) return undefined;

		var data: IRibbonItem = undefined;
		var loop = 3;

		// ReSharper disable once QualifiedExpressionMaybeNull
		while (loop-- >= 0 && (data = item.data("ribbonItemData")) == undefined) {
			item = item.parent();
		}

		return data;
	}

	//public SetDisabledItemsByState(name: string, state: string): void {
	//	if (this.ribbonGroups.hasOwnProperty(name)) {
	//		this.ribbonGroups[name].SetDisabledItemsByState(state);
	//		return;
	//	}
	//	this.ribbonGroups.forEach(group => { group.SetDisabledItemsByState(state); });
	//}

	public SetProperty(selector: string, values: {}): void {
		// editnavigation:{save delete undo}, editNavigation

		var instance = this;
		if (selector == undefined || selector == "") {
			// set property on RibbonManager
			$.each(values, (name, value) => {
				switch (name.toLowerCase()) {
					case "ismodified":
						instance.IsModified = value;
						break;
				}
			});
			return;
		}

		var groupSelectors = selector.split(",");
		$.each(groupSelectors, (index, groupSelector) => {
			var sepIndex = groupSelector.indexOf(":");
			var groupName = sepIndex == -1 ? groupSelector : groupSelector.substr(0, sepIndex);
			var itemSelector = sepIndex == -1 ? "" : groupSelector.substr(sepIndex + 1).trim().replace("{", "").replace("}", "");

			var group = instance.FindGroup(groupName);
			if (group != undefined) {
				group.SetProperty(itemSelector, values);
			}
		});
	}

    public SetAction(selector: string, options: {}): void {
        switch(selector.toLowerCase()) {
            case "add":
                this.addGroup(options);
                break;
        }
    }
	// --- Constructor ----------------------------------------------------
	constructor(topLevel?: any, options?: IRibbonManagerOptions) {
		this.topLevel = this.create(topLevel, options);
		this.onReady();
	}

	// --- Implementation -------------------------------------------------
    private addGroup(args: {}): void {
        var position: number = Number.MAX_VALUE,
            name: string,
            separator: string = "none",
            element = undefined;

        $.each(args, (index, value) => {
            switch (index.toLowerCase()) {
                case "element":
                    element = value;
                    break;
                case "name":
                    name = value;
                    break;
                case "position":
                    position = value;
                    break;
                case "separator":
                    separator = value;
                    break;
            }
        });

        // ReSharper disable once ConditionIsAlwaysConst
        if (element == undefined) return;    // nothing to add

        var group = new RibbonWrapper({
            element: element,
            name: name,
            position: position,
            separator: separator
        });

        if (position >= this.ribbonGroups.length) {
            this.ribbonGroups.push(group);
            if (separator === "left" || separator === "both")
                this.topLevel.append($(document.createElement("div")).addClass("separator").text(" "));
            this.topLevel.append(group.TopLevel);
            if (separator === "right" || separator === "both")
                this.topLevel.append($(document.createElement("div")).addClass("separator").text(" "));
        } else if (position == 0) {
            this.ribbonGroups.unshift(group);
            if (separator === "right" || separator === "both")
                this.topLevel.prepend($(document.createElement("div")).addClass("separator").text(" "));
            this.topLevel.prepend(group.TopLevel);
            if (separator === "left" || separator === "both")
                this.topLevel.prepend($(document.createElement("div")).addClass("separator").text(" "));
        } else {
            this.ribbonGroups.splice(position, 0, group);
        }
    }

	private create(topLevel: any, options?: IRibbonManagerOptions) {
		var instance = this;
		$.each(options.Groups, (index, value) => {
			var group = new RibbonGroup(value);
			instance.ribbonGroups.push(group);

			topLevel.append(group.TopLevel);
		});

		return topLevel;
	}

	private onPropertyChanged(property: string) {
		this.PropertyChanged.trigger(this, property);
	}

	private onReady() {
		this.isReadyFired = true;
		this.Ready.FireOnAdd(this.isReadyFired);
		this.Ready.trigger("");
	}
}

class RibbonWrapper implements IRibbonWrapper {
	// --- Events ---------------------------------------------------------
	// --- Fields ---------------------------------------------------------
    private element: HTMLElement;
    private item: IRibbonItem;
    private name: string;
    private position: Number = Number.MAX_VALUE;
    private separator: string = "none";
    private toplevel: JQuery;

	// --- Properties -----------------------------------------------------
    public get IsDocumentModified(): boolean { return false; }

    public get Element(): HTMLElement { return this.element; }

    public get Items(): { [index: string]: IRibbonItem } {
        var group: { [index: string]: IRibbonItem } = {};
        group[this.name] = this.item;
        return group;
    }

    public get Name(): string { return this.name; }

    public get Position(): Number { return this.position; }

    public get Separator(): string { return this.separator; }

    public get TopLevel(): JQuery { return this.toplevel; }

	// --- Methods --------------------------------------------------------
	public AttachEvent(itemNames: string, options: {}) { }

    public SetProperty(itemNames: string, values: {}): void {
        if (this.item == undefined) return;
        if (itemNames == undefined || (typeof itemNames == "string" && (itemNames == "" || itemNames == this.name)))
            this.item.SetProperty(values);
    }

// --- Constructor ----------------------------------------------------
    constructor(options: {}) {
        var $this = this;
        $.each(options, (index, value) => {
            switch (index.toLowerCase()) {
                case "element":
                    $this.element = value;
                    break;
                case "name":
                    $this.name = value;
                    break;
                case "position":
                    $this.position = value;
                    break;
                case "separator":
                    $this.separator = value;
                    break;
            }
        });

        var itemOpts: IRibbonItemOptions = {
            DisplayText: this.name,
			Name: this.name,
			IgnoreFocusLost: false,
            IsActive: true,
            IsEnabled: true,
            IsVisible: true,
        };

        this.item = new RibbonItem(itemOpts, this);
        this.toplevel = $(this.element);
        this.toplevel.addClass("ribbon-group");
    }

	// --- Implementation -------------------------------------------------
}

class RibbonGroup implements IRibbonGroup {
	// --- Events ---------------------------------------------------------
	//public NotifyCommand: IRoutedCommand = new TypedEvent();

	// --- Fields ---------------------------------------------------------
	private imageBasePath: string;
	private isModified: boolean;
	private items: { [index: string]: IRibbonItem } = {};
	private name: string;
	private topLevel: JQuery;

	// --- Properties -----------------------------------------------------
	public get ImageBasePath() { return this.imageBasePath; }

	public set ImageBasePath(value: string) { this.imageBasePath = value; }

	public get Items() { return this.items; }

	public set IsDocumentModified(value: boolean) { this.isDocumentModified = value; }

	public get Name() { return this.name; }

	public set Name(value: string) { this.name = value; }

	public get TopLevel(): JQuery { return this.topLevel; }

	// --- Methods --------------------------------------------------------
	public Add(name: string, item: RibbonItem): RibbonGroup {
		this.items[name] = item;
		item.Parent = this;

		return this;
	}

	public AttachEvent(itemNames: string, options: {}) {
		var names = itemNames.split(" ");
		var instance = this;
		$.each(names, (index, value) => {
			$.each(instance.items, (name, item) => {
				if (name.toLowerCase() == value)
					item.AttachEvent(options);
			});
		});
	}

	public CreateImagePath(imagePath: string): string {
		return this.imageBasePath + "/" + imagePath;
	}

	public SetProperty(itemNames: string, values: {}): void {
		var names = itemNames.split(" ");
		var instance = this;
		$.each(names, (index, value) => {
			$.each(instance.items, (name, item) => {
				if (name.toLowerCase() == value)
					item.SetProperty(values);
			});
		});
	}

	// --- Constructor ----------------------------------------------------
	constructor(options: any, imageBasePath?: string) {

		if (arguments.length > 1) {
			this.imageBasePath = imageBasePath;
			this.name = name;
		} else
			this.topLevel = this.create(options);
	}

	// --- Implementation -------------------------------------------------
	private create(options: IRibbonGroupOptions): JQuery {
		this.name = options.Name;

		var container = $(document.createElement("div"));
        container.addClass("ribbon-group");

	    if (this.name != undefined)
	        container.attr("id", this.name);

		$.each(options.Items, (index, value) => {
			var item = new RibbonItem(value, this);
			this.items[item.Name] = item;

			container.append(item.Container);
		});

		return container;
	}

	private set isDocumentModified(isModified: boolean) {
		this.isModified = isModified;

		$.each(this.items, (index, value) => {
			value.IsEnabled = !(value.IsDisabledOnModified && isModified);
		});
	}
}

class RibbonItem implements IRibbonItem {
    // --- Fields ---------------------------------------------------------
	private container: JQuery;
	private domElement: any;
	private eventType: string;
	private ignoreFocusLost: boolean = false;
	private isActivateOnClick: boolean;
	private isDisabledOnModified: boolean;
	private parent: IRibbonGroup;
	private validationCallback: (options?: any) => boolean;
	private viewModel: {
		DisplayText: KnockoutObservable<string>;
		IsActive: KnockoutObservable<boolean>;
		IsDisabled: KnockoutObservable<boolean>;
		IsVisible: KnockoutObservable<boolean>;
		Name: KnockoutObservable<string>;
		OnClick: Function;
	} = {
		DisplayText: ko.observable<string>(),
		IsActive: ko.observable<boolean>(false),
		IsDisabled: ko.observable<boolean>(false),
		IsVisible: ko.observable<boolean>(true),
		Name: ko.observable<string>(),
		OnClick: () => {
			this.onClick();
		}
	};

    // --- Events ---------------------------------------------------------
	public OnActivatedChanged: IBooleanEvent = new TypedEvent;
	public OnDisabledChanged: IBooleanEvent = new TypedEvent;
	public OnVisibleChanged: IBooleanEvent = new TypedEvent;

    // --- Properties -----------------------------------------------------
	public get ActivateOnClick(): boolean { return this.isActivateOnClick; }

	public set ActivateOnClick(value: boolean) {
		this.isActivateOnClick = value;
	}

	public get CommandName(): string {
		return ("ribbon_" + this.parent.Name + "_" + this.EventType).toLowerCase();
	}

	public get Container(): JQuery { return this.container; }

	//public get DisabledImagePath() { return this.disabledImagePath; }

	//public set DisabledImagePath(value: string) { this.disabledImagePath = value; }

    public get DisplayText(): string {
		return (this.viewModel.DisplayText() == undefined || this.viewModel.DisplayText() == "") ? this.Name : this.viewModel.DisplayText();
    }

	public set DisplayText(value: string) {
		this.viewModel.DisplayText(value);
	}

	public get EventType(): string { return (this.eventType || this.Name).toLowerCase(); }

	public set EventType(value: string) { this.eventType = value; }

	public get HasValidator(): boolean { return this.validationCallback != undefined; }

	public get IgnoreFocusLost(): boolean { return this.ignoreFocusLost; }

	public set IgnoreFocusLost(value: boolean) {
		this.ignoreFocusLost = value;
	}

	public get IsActive(): boolean { return this.viewModel.IsActive() || false; }

	public set IsActive(value: boolean) {
		this.viewModel.IsActive(value);
	}

	public get IsDisabled(): boolean { return this.viewModel.IsDisabled() || false; }

	public set IsDisabled(value: boolean) {
		this.viewModel.IsDisabled(value);
		this.toggleVisualState(value, "disabled", false);
		this.OnDisabledChanged.trigger(this.viewModel.IsDisabled());
	}

	public get IsDisabledOnModified() { return this.IsDisabledOnModified; }

	public set IsDisabledOnModified(value: boolean) { this.IsDisabledOnModified = value; }

	public get IsVisible(): boolean { return this.viewModel.IsVisible() || true; }

	public set IsVisible(value: boolean) {
		this.viewModel.IsVisible(value);
		this.OnVisibleChanged.trigger(this.viewModel.IsVisible());
	}

	public get Name(): string { return this.viewModel.Name(); }

	public get Parent(): IRibbonGroup { return this.parent; }

	public set Parent(value: IRibbonGroup) { this.parent = value; }

	public get ValidationCallback(): (options?: any) => boolean { return this.validationCallback; }

	public set ValidationCallback(value: (options?: any) => boolean) {
		this.validationCallback = value;
	}

    // --- Methods --------------------------------------------------------
	public AttachEvent(options: {}) {
		if (options.hasOwnProperty("OnActivatedChanged")) {
			this.OnActivatedChanged.add(options["OnActivatedChanged"]);
		}
	}

	public SetProperty(values: {}) {
		var instance = this;
		$.each(values, (index, value) => {
			if (instance.viewModel.hasOwnProperty(index)) {
				if (typeof instance.viewModel[index] == "function")
					instance.viewModel[index].call(instance, value);
				else
					instance.viewModel[index] = value;
			} else if (instance.hasOwnProperty(index)) {
				instance[index] = value;
			} else {
				switch (index.toLowerCase()) {
					case "activateonclick":
						instance.ActivateOnClick = value;
						break;
					case "isactive":
						instance.IsActive = value;
						break;
					case "isdisabled":
						instance.IsDisabled = value;
						break;
					case "isenabled":
						instance.IsDisabled = !value;
						break;
					case "isvisible":
						instance.IsVisible = value;
						break;
					case "name":
						instance.viewModel.Name(value);
						break;
					case "validationcallback":
						instance.validationCallback = value;
						break;
				}
			}
		});
	}

    // --- Constructor ----------------------------------------------------
	constructor(options: IRibbonItemOptions, parent: IRibbonGroup) {

		this.parent = parent;
		this.ignoreFocusLost = options.hasOwnProperty("IgnoreFocusLost") && options["IgnoreFocusLost"];
		this.viewModel.Name(options.Name);
		this.viewModel.DisplayText(options.DisplayText);
		//this.imagePath = options.ImagePath;
		//this.disabledImagePath = options.ImagePathDisabled;
		this.viewModel.IsActive(options.IsActive || false);
		this.viewModel.IsDisabled((options.IsEnabled != undefined && !options.IsEnabled) || false);
		this.viewModel.IsVisible(options.IsVisible || true);

        this.container = this.create(this.parent.Name);

	    this.viewModel.IsVisible.subscribe((newvalue) => {
	        if (newvalue === false)
                this.container.css("display", "none");
	        else {
	            this.container.css("display", "inline-block");
	        }
		});

		this.viewModel.IsActive.subscribe((newvalue) => {
			this.toggleVisualState(newvalue, "active", false);
			this.OnActivatedChanged.trigger(newvalue);
		});
	}

    // --- Implementation -------------------------------------------------
	//<div class="nav-item active checked" id = "List" data - bind = "visible: IsVisible, attr: { id: Name }, click: OnClick" >
	//	<div class="imgdiv active" >< / div >
	//	<div class="txtdiv active" data - bind = "text: DisplayText" > Liste </div >
	//</div >
	private create(groupName: string): JQuery {
		var container = $(document.createElement("div"));
		container.attr("id", groupName + "_" + this.Name);
		container.addClass("nav-item");
		if (this.ignoreFocusLost)
			container.addClass("ignoreFocusLost");
		container.click(() => {
			this.onClick();
		});

		if (!this.IsVisible)
			container.css("display", "none");

		var inner = $(document.createElement("div"));
		inner.addClass("ribbon-item");

		this.domElement = inner;

		var imgdiv = $(document.createElement("div"));
		imgdiv.addClass("ribbon-item-image");

		var txtdiv = $(document.createElement("div"));
		txtdiv.addClass("ribbon-item-text");
		txtdiv.text(this.DisplayText);

		if (this.IsDisabled) {
			container.addClass("disabled");
			inner.addClass("disabled");
			imgdiv.addClass("disabled");
			txtdiv.addClass("disabled");
		}

		inner.append(imgdiv);
		inner.append(txtdiv);

		container.append(inner);

		container.data("ribbonItemData", this);

		return container;
	}

	private onClick() {
		if (this.IsDisabled) return;

		// if a validation is assigned, but fails -> do nothing
		if (this.HasValidator && !this.validationCallback()) return;

		if (this.isActivateOnClick) {
			this.IsActive = !this.IsActive;
		}

		var command = this.CommandName;
		eval("orgAnice.trigger('" + command + "');");

		//TODO: "eval()" is not the way to go! Changed this to a reference to the "orgAnice" namespace
	}

	private toggleVisualState(toggle: boolean, selector: string, reverseProperty: boolean) {

		if ((toggle && !reverseProperty) || (!toggle && reverseProperty)) {
			this.domElement.addClass(selector);
			this.domElement.find(".ribbon-item-image").addClass(selector);
			this.domElement.find(".ribbon-item-text").addClass(selector);
		} else {
			this.domElement.removeClass(selector);
			this.domElement.find(".ribbon-item-image").removeClass(selector);
			this.domElement.find(".ribbon-item-text").removeClass(selector);
		}
	}
}

// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
	$.fn.orgAniceRibbon = function (arg1: any, arg2?: any, arg3?: any) {

		if (!this.length) return this;

		var count = arguments.length;
		var retval = undefined;

		var fval = this.each(function () {
			var instance = this;
			var $instance = $(instance);
			var options: IRibbonManagerOptions;
			var control: IRibbonManager = $instance.data["orgAniceRibbon"];
			var target: string,
				selector: string,
				value: any;

			switch (count) {
				case 0:
					return;

				case 1:
					options = arg1;

					if (control == undefined) {
						control = new RibbonManager($instance, options);
						$instance.data["orgAniceRibbon"] = control;
					} else {
						//control.Initialize(options);
					}
					break;

				case 2:	// value requested?
					target = arg1;
					selector = arg2;

					switch (target.toLowerCase()) {
						case "property":
							switch (selector.toLowerCase()) {
								case "ismodified":
									retval = control.IsModified;
									break;
								case "isenabled":
									retval = control.IsModified;
									break;
							}
							break;
					}
					break;

				default:	// arguments.length > 1
					target = arg1;
					selector = arg2;
					value = arg3;

					if (typeof value == "string") {
						switch (value.toLowerCase()) {
							case "ismodified":
								retval = control.IsModified;
								break;
						}
					} else {
                        switch (target.toLowerCase()) {
                            case "action":
                                control.SetAction(arg2, arg3);
                                break;
							case "event":
							case "event:attach":
								control.AttachEvent(selector, value);
								//switch (selector.toLowerCase()) {
								//	case "ready":
								//		control.Ready.add(value);
								//		break;
								//	case "":
								//		break;
								//}
								break;
							case "event:detach":
								control.DetachEvent(selector, value);
								//switch (selector.toLowerCase()) {
								//	case "ready":
								//		control.Ready.remove(value);
								//		break;
								//}
								break;

							case "property":
								control.SetProperty(selector, value);
								break;
						}
					}
					break;
			}

		});

		return retval == undefined ? fval : retval;
	};
})(jQuery || $);
// ReSharper restore JsFunctionCanBeConvertedToLambda
