﻿
///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.d.ts"/>
///<reference path="orgAnice.data.d.ts"/>

interface ISearchControlViewModel {
	Active: KnockoutObservable<IIndexDescriptor>;
    ButtonEnabled: KnockoutComputed<boolean>;
	Controller: KnockoutObservable<string>;
	Indexes: KnockoutObservableArray<IIndexDescriptor>;
	OldIndex: IIndexDescriptor;
	TableName: KnockoutObservable<string>;
	Value: KnockoutObservable<string>;
}

interface ISearchControl {
	// --- Events ---------------------------------------------------------
	/// <summary>Raised, if a new index has been selected from the index drop-down.</summary>
	IndexChanged: orgAniceModule.IorgAniceEvent<IIndexChangedEventArgs>;

	/// <summary>Raised, if any property on the item has changed.</summary>
	PropertyChanged: orgAniceModule.IorgAniceEvent<string>;

	/// <summary>Raised, when the component is ready for service.</summary>
	Ready: orgAniceModule.orgAniceEvent<void>;

	/// <summary>Raised, when the search returns with or w/o results.</summary>
	SearchResult: orgAniceModule.orgAniceEvent<IListQueryResult>;

	// --- Properties -----------------------------------------------------
    /// <summary>The data endpoint from which the indexes are requested.</summary>
    DataEndpoint: string;

    /// <summary>Determines, whether typing into the value field will automatically invoke a server query.</summary>
    /// <value>if <i>true</i>, typing into the value field will automatically invoke a server query</value>
    IsAutoQuery: boolean;

	/// <summary><c>true</c> to show, <c>false</c> to hide the index search button; default: <c>true</c>.</summary>
	ShowIndexSearchButton: boolean;

    /// <summary>The top level control element.</summary>
    TopLevel: JQuery;

    /// <summary>The view model.</summary>
    //ViewModel: {
    //    Active: KnockoutObservable<string>;
    //    Indexes: KnockoutObservableArray<string>;
    //    TableName: string;
    //    Value: KnockoutObservable<string>;
    //};
	ViewModel: ISearchControlViewModel;

	// --- Methods --------------------------------------------------------
	QueryIndexes(endPoint: string, controller: string): void;

	/// <summary>Resets the active index to its previous value. </summary>
	/// <param name="requery">(Optional) if <c>true</c>, the <see href="IndexChanged" /> event will be fired; defaults to <c>false</c>. </param>
	ResetIndex(requery?: boolean): void;

	/// <summary>Resume index changed event. </summary>
	ResumeIndexChangedEvent(): void;

	SelectIndex(indexName: string, suppressEvent: boolean): void;

	SetSearchResult(result: ISearchResult): void;

	/// <summary>Instructs the control to ignore index changed events, until <see href="ResumeIndexChangedEvent" /> has been invoked. </summary>
	SuppressIndexChangedEvent(): void;
}

/// <summary>Interface for an item, which represents the search properties of a OrgData table. </summary>
interface ISearchControlItem {
	// --- Properties -----------------------------------------------------
	ActiveIndex: string;
	Indexes: IIndexDescriptor[];
	TableName: string;
}

interface ISearchArgs {
	value: string;
	index: string;
	table: string;
	byIndex: boolean;
	count: number;
}

interface ISearchResult {
	index: string;
	success: boolean;
}

