///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.d.ts"/>
///<reference path="orgAnice.searchControl.d.ts"/>
var SearchControl = (function () {
    // --- Constructor ----------------------------------------------------
    function SearchControl(options) {
        var _this = this;
        // --- Events ---------------------------------------------------------
        // ReSharper disable InconsistentNaming
        this.IndexChanged = new orgAniceModule.orgAniceEvent();
        // ReSharper restore InconsistentNaming
        this.PropertyChanged = new orgAniceModule.orgAniceEvent();
        this.Ready = new orgAniceModule.orgAniceEvent();
        this.SearchResult = new orgAniceModule.orgAniceEvent();
        this.showIndexSearchButton = true;
        this.suppressIndexChanged = false;
        this.viewModel = new SearchControlViewModel();
        this.buttonEnabled = ko.computed(function () {
            return _this.viewModel.Indexes() != undefined && _this.viewModel.Indexes().length > 0 &&
                _this.viewModel.Value() != undefined && _this.viewModel.Value() !== "";
        });
        if (options == undefined)
            throw new Error("Invalid argument");
        if (options.hasOwnProperty("DataEndpoint")) {
            this.dataEndpoint = options["DataEndpoint"];
        }
        if (options.hasOwnProperty("SearchResult") && typeof options["SearchResult"] == "function") {
            this.SearchResult.add(options["SearchResult"]);
        }
        this.viewModel.Active.subscribe(function (newValue) {
            _this.onIndexChanged(newValue);
        });
        this.create();
        this.buttonEnabled.subscribe(function (newValue) {
            if (newValue) {
                _this.searchButton.removeClass("disabled");
                if (_this.indexButton != undefined)
                    _this.indexButton.removeClass("disabled");
            }
            else {
                _this.searchButton.addClass("disabled");
                if (_this.indexButton != undefined)
                    _this.indexButton.addClass("disabled");
            }
        });
    }
    Object.defineProperty(SearchControl.prototype, "DataEndpoint", {
        get: function () {
            return this.dataEndpoint;
        },
        set: function (value) {
            if (this.dataEndpoint === value)
                return;
            this.dataEndpoint = value;
            this.onPropertyChanged("DataEndpoint");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControl.prototype, "IsAutoQuery", {
        get: function () { return this.isAutoQuery; },
        set: function (value) {
            if (this.isAutoQuery == value)
                return;
            this.IsAutoQuery = value;
            this.onPropertyChanged("IsAutoQuery");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControl.prototype, "ShowIndexSearchButton", {
        get: function () { return this.showIndexSearchButton; },
        set: function (value) {
            this.showIndexSearchButton = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControl.prototype, "TableName", {
        get: function () {
            return this.viewModel.TableName();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControl.prototype, "TopLevel", {
        get: function () {
            return this.topLevel;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControl.prototype, "ViewModel", {
        get: function () {
            return this.viewModel;
        },
        enumerable: true,
        configurable: true
    });
    // --- Methods --------------------------------------------------------
    SearchControl.prototype.QueryIndexes = function (endPoint, controller) {
        var _this = this;
        var control = this;
        this.ViewModel.Controller(controller);
        $.ajax({
            url: endPoint,
            contentType: "application/json; charset=utf-8",
            dataType: "json",
            data: {},
            type: "GET",
            cache: false
        })
            .done(function (result) {
            _this.SuppressIndexChangedEvent();
            control.viewModel.Indexes(result.Data);
            var dictionary = result.Dictionary;
            if (dictionary.hasOwnProperty("active"))
                control.viewModel.Active(dictionary["active"]);
            if (dictionary.hasOwnProperty("table"))
                control.viewModel.TableName(dictionary["table"]);
            _this.ResumeIndexChangedEvent();
        })
            .fail(function (xhr) {
            window.orgAnice.dialogManager.HandleAjaxError(xhr);
        });
    };
    SearchControl.prototype.ResetIndex = function (requery) {
        requery = requery || false;
        if (!requery)
            this.SuppressIndexChangedEvent();
        this.viewModel.Active(this.viewModel.OldIndex);
        if (!requery)
            this.ResumeIndexChangedEvent();
    };
    SearchControl.prototype.ResumeIndexChangedEvent = function () {
        this.suppressIndexChanged = false;
    };
    SearchControl.prototype.SelectIndex = function (indexName, suppressEvent) {
        var descriptor = undefined;
        $.each(this.viewModel.Indexes(), function (idx, indexDescriptor) {
            if (indexDescriptor.DisplayName === indexName) {
                descriptor = indexDescriptor;
                return false;
            }
            return true;
        });
        // ReSharper disable once ConditionIsAlwaysConst
        // ReSharper disable once HeuristicallyUnreachableCode
        if (descriptor != undefined) {
            if (suppressEvent)
                this.SuppressIndexChangedEvent();
            this.viewModel.Active(descriptor);
            if (suppressEvent)
                this.ResumeIndexChangedEvent();
        }
    };
    SearchControl.prototype.SetSearchResult = function (result) {
        if (!result.success) {
            this.valueInput.addClass("failed").removeClass("success");
            $(document).statusBar().AddMessage("Suche hat nichts gefunden", StatusBarMessageSeverity.Success);
        }
        else {
            this.valueInput.addClass("success").removeClass("failed");
            var active = undefined;
            $.each(this.viewModel.Indexes(), function (idx, value) {
                if (value.Name == result.index) {
                    active = value;
                    return false;
                }
                else
                    return true;
            });
            this.viewModel.Active(active);
        }
    };
    SearchControl.prototype.SuppressIndexChangedEvent = function () {
        this.suppressIndexChanged = true;
    };
    // --- Implementation -------------------------------------------------
    SearchControl.prototype.create = function () {
        var _this = this;
        this.topLevel = $(document.createElement("div")).attr("id", "searchDialog");
        var container = $(document.createElement("div")).addClass("searchControls");
        var keyLabel = $(document.createElement("div")).addClass("label search-key").text(" ");
        var keyBox = $(document.createElement("div")).addClass("search-key-box");
        var keySelect = $(document.createElement("select"))
            .attr("id", "key-selector")
            .attr("data-bind", "options: Indexes, optionsText: 'DisplayName', value: Active");
        keyBox.append(keyLabel).append(keySelect);
        if (this.showIndexSearchButton) {
            this.indexButton = $(document.createElement("div")).addClass("indexButton")
                .addClass("disabled")
                .click(function () {
                _this.executeSearch(true);
            });
            keyBox.append(this.indexButton);
        }
        var valBox = $(document.createElement("div")).addClass("search-value-box");
        var valLabel = $(document.createElement("div")).addClass("label search-value").text(" ");
        this.valueInput = $(document.createElement("input"))
            .attr("type", "textbox")
            .attr("data-bind", "textInput: Value")
            .addClass("search-value");
        this.searchButton = $(document.createElement("div")).addClass("searchButton").attr("type", "submit")
            .addClass("disabled")
            .click(function () {
            _this.executeSearch(false);
        });
        this.valueInput.keypress(function (e) {
            if (e.keyCode == 13) {
                _this.executeSearch(false);
            }
        });
        valBox.append(valLabel).append(this.valueInput).append(this.searchButton);
        container.append(valBox);
        container.append(keyBox);
        this.topLevel.append(container);
        this.topLevel.focusout(function () {
            _this.valueInput.removeClass("success").removeClass("failed");
        });
        ko.applyBindings(this.viewModel, this.topLevel[0]);
        this.Ready.trigger("");
    };
    SearchControl.prototype.executeSearch = function (searchByIndex) {
        if (!this.buttonEnabled())
            return;
        var dialogManager = window.orgAnice.dialogManager;
        if (dialogManager == undefined || dialogManager.CurrentDialog == undefined)
            return;
        var options = {
            value: this.viewModel.Value() || "",
            index: (this.viewModel.Active() != undefined ? this.viewModel.Active().Name : "") || "",
            table: this.viewModel.TableName() || "",
            byIndex: searchByIndex,
            count: -1
        };
        dialogManager.CurrentDialog.Search(options);
    };
    SearchControl.prototype.onIndexChanged = function (newIndex) {
        if (this.suppressIndexChanged)
            return;
        var oldIndex = this.ViewModel.OldIndex == undefined ? "" : this.ViewModel.OldIndex.Name;
        if (oldIndex === newIndex.Name)
            return;
        var dialogManager = window.orgAnice.dialogManager;
        if (this.tempOid == undefined)
            this.tempOid = dialogManager.CurrentDialog.CurrentOid;
        var eventArgs = {
            Controller: this.ViewModel.Controller(),
            Index: newIndex.Name,
            Key: this.tempOid,
            OldIndex: this.ViewModel.OldIndex == undefined ? "" : this.ViewModel.OldIndex.Name
        };
        this.tempOid = undefined;
        this.IndexChanged.trigger(eventArgs);
    };
    SearchControl.prototype.onPropertyChanged = function (property) {
        this.PropertyChanged.trigger(this, property);
    };
    SearchControl.prototype.onReady = function () {
        this.isReadyFired = true;
        this.Ready.FireOnAdd(this.isReadyFired);
        this.Ready.trigger("");
    };
    // --- Properties -----------------------------------------------------
    SearchControl.SearchSuccessEventname = "search_success";
    SearchControl.SearchFailedEventname = "search_failed";
    return SearchControl;
}());
var SearchControlViewModel = (function () {
    function SearchControlViewModel() {
        var _this = this;
        this.activeIndex = ko.observable();
        this.controller = ko.observable();
        this.indexes = ko.observableArray();
        this.searchValue = ko.observable();
        this.tableName = ko.observable();
        this.buttonEnabled = ko.computed(function () {
            return _this.indexes() != undefined && _this.indexes().length > 0 &&
                _this.searchValue() != undefined && _this.searchValue().length > 0;
        }).extend({ update: "always" });
        this.activeIndex.subscribe(function (oldIndex) {
            _this.oldIndex = oldIndex;
        }, null, "beforeChange");
    }
    Object.defineProperty(SearchControlViewModel.prototype, "Active", {
        get: function () {
            return this.activeIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "ButtonEnabled", {
        get: function () {
            return this.buttonEnabled;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "Controller", {
        get: function () {
            return this.controller;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "Indexes", {
        get: function () {
            return this.indexes;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "OldIndex", {
        get: function () {
            return this.oldIndex;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "Value", {
        get: function () {
            return this.searchValue;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchControlViewModel.prototype, "TableName", {
        get: function () {
            return this.tableName;
        },
        enumerable: true,
        configurable: true
    });
    return SearchControlViewModel;
}());
var SearchResult = (function () {
    function SearchResult() {
    }
    Object.defineProperty(SearchResult.prototype, "CountRecords", {
        get: function () { return this.countPages; },
        set: function (value) { this.countPages = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "CurrentRecord", {
        get: function () { return this.currentPage; },
        set: function (value) { this.currentPage = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Error", {
        get: function () { return this.error; },
        set: function (value) { this.error = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Index", {
        get: function () { return this.index; },
        set: function (value) { this.index = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Oid", {
        get: function () { return this.oid; },
        set: function (value) { this.oid = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Success", {
        get: function () { return this.success; },
        set: function (value) { this.success = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Table", {
        get: function () { return this.table; },
        set: function (value) { this.table = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "Value", {
        get: function () { return this.value; },
        set: function (value) { this.value = value; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SearchResult.prototype, "List", {
        get: function () { return this.list; },
        set: function (value) { this.list = value; },
        enumerable: true,
        configurable: true
    });
    return SearchResult;
}());
//# sourceMappingURL=orgAnice.searchControl.js.map