﻿
///<reference path="../typings/jquery/jQuery.d.ts"/>
///<reference path="orgAnice.d.ts"/>
///<reference path="orgAnice.searchControl.d.ts"/>

class SearchControl implements ISearchControl {
	// --- Events ---------------------------------------------------------
// ReSharper disable InconsistentNaming
	public IndexChanged: orgAniceModule.IorgAniceEvent<IIndexChangedEventArgs> = new orgAniceModule.orgAniceEvent<IIndexChangedEventArgs>();
// ReSharper restore InconsistentNaming

	public PropertyChanged: orgAniceModule.IorgAniceEvent<{}> = new orgAniceModule.orgAniceEvent<{}>();

	public Ready: orgAniceModule.orgAniceEvent<void> = new orgAniceModule.orgAniceEvent<void>();

	public SearchResult: orgAniceModule.orgAniceEvent<IListQueryResult> = new orgAniceModule.orgAniceEvent<IListQueryResult>();

	// --- Fields ---------------------------------------------------------
	private dataEndpoint: string;
	private indexButton: JQuery;
	private isAutoQuery: boolean;
	private isReadyFired: boolean;
	private searchButton: JQuery;
	private showIndexSearchButton: boolean = true;
	private suppressIndexChanged: boolean = false;
	private tempOid: string;
	private topLevel: any;
	private valueInput: JQuery;
	private viewModel: ISearchControlViewModel = new SearchControlViewModel();

	private buttonEnabled: KnockoutComputed<boolean> = ko.computed (() => {
		return this.viewModel.Indexes() != undefined && this.viewModel.Indexes().length > 0 &&
			this.viewModel.Value() != undefined && this.viewModel.Value() !== "";
	});

	// --- Properties -----------------------------------------------------
	public static SearchSuccessEventname: string = "search_success";
	public static SearchFailedEventname: string = "search_failed";

	public get DataEndpoint(): string {
		return this.dataEndpoint;
	}
	public set DataEndpoint(value: string) {
		if (this.dataEndpoint === value) return;
		this.dataEndpoint = value;
		this.onPropertyChanged("DataEndpoint");
	}

	public get IsAutoQuery(): boolean { return this.isAutoQuery; }

	public set IsAutoQuery(value: boolean) {
		if (this.isAutoQuery == value) return;
		this.IsAutoQuery = value;
		this.onPropertyChanged("IsAutoQuery");
	}

	public get ShowIndexSearchButton(): boolean { return this.showIndexSearchButton; }

	public set ShowIndexSearchButton(value: boolean) {
		this.showIndexSearchButton = value;
	}

	public get TableName(): string {
		return this.viewModel.TableName();
	}

	public get TopLevel(): any {
		return this.topLevel;
	}

	public get ViewModel(): ISearchControlViewModel {
		return this.viewModel;
	}

	// --- Methods --------------------------------------------------------
	public QueryIndexes(endPoint: string, controller: string) {

		var control = this;
		this.ViewModel.Controller(controller);

		$.ajax({
				url: endPoint,
				contentType: "application/json; charset=utf-8",
				dataType: "json",
				data: {},
				type: "GET",
				cache: false
			})
			.done((result: IAjaxQueryResult) => {
				this.SuppressIndexChangedEvent();
				control.viewModel.Indexes(result.Data);

				var dictionary = result.Dictionary;
				if (dictionary.hasOwnProperty("active"))
					control.viewModel.Active(dictionary["active"]);
				if (dictionary.hasOwnProperty("table"))
					control.viewModel.TableName(dictionary["table"]);

				this.ResumeIndexChangedEvent();
			})
			.fail((xhr) => {
				window.orgAnice.dialogManager.HandleAjaxError(xhr);
			});
	}

	public ResetIndex(requery?: boolean): void {
		requery = requery || false;
		if (!requery)
			this.SuppressIndexChangedEvent();

		this.viewModel.Active(this.viewModel.OldIndex);

		if (!requery)
			this.ResumeIndexChangedEvent();
	}

	public ResumeIndexChangedEvent(): void {
		this.suppressIndexChanged = false;
	}

	public SelectIndex(indexName: string, suppressEvent: boolean): void {
		var descriptor: IIndexDescriptor = undefined;

		$.each(this.viewModel.Indexes(), (idx: number, indexDescriptor: IIndexDescriptor) => {
			if (indexDescriptor.DisplayName === indexName) {
				descriptor = indexDescriptor;
				return false;
			}
			return true;
		});
		// ReSharper disable once ConditionIsAlwaysConst
		// ReSharper disable once HeuristicallyUnreachableCode
		if (descriptor != undefined) {
			if (suppressEvent)
				this.SuppressIndexChangedEvent();
			this.viewModel.Active(descriptor);
			if (suppressEvent)
				this.ResumeIndexChangedEvent();
		}
	}

	public SetSearchResult(result: ISearchResult): void {
		if (!result.success) {
			this.valueInput.addClass("failed").removeClass("success");
			$(document).statusBar().AddMessage("Suche hat nichts gefunden", StatusBarMessageSeverity.Success);
		} else {
			this.valueInput.addClass("success").removeClass("failed");
			var active: IIndexDescriptor = undefined;
			$.each(this.viewModel.Indexes(), (idx, value: IIndexDescriptor) => {
				if (value.Name == result.index) {
					active = value;
					return false;
				} else return true;
			});
			this.viewModel.Active(active);
		}
	}

	public SuppressIndexChangedEvent(): void {
		this.suppressIndexChanged = true;
	}

	// --- Constructor ----------------------------------------------------
	constructor(options: {}) {
		if (options == undefined)
			throw new Error("Invalid argument");

		if (options.hasOwnProperty("DataEndpoint")) {
			this.dataEndpoint = options["DataEndpoint"];
		}
		if (options.hasOwnProperty("SearchResult") && typeof options["SearchResult"] == "function") {
			this.SearchResult.add(options["SearchResult"]);
		}

		this.viewModel.Active.subscribe((newValue: IIndexDescriptor) => {
			this.onIndexChanged(newValue);
		});

		this.create();

		this.buttonEnabled.subscribe((newValue) => {
			if (newValue) {
				this.searchButton.removeClass("disabled");
				if (this.indexButton != undefined)
					this.indexButton.removeClass("disabled");
			} else {
				this.searchButton.addClass("disabled");
				if (this.indexButton != undefined)
					this.indexButton.addClass("disabled");
			}
		});
	}

	// --- Implementation -------------------------------------------------
	private create(): void {

		this.topLevel = $(document.createElement("div")).attr("id", "searchDialog");

		var container = $(document.createElement("div")).addClass("searchControls");
		var keyLabel = $(document.createElement("div")).addClass("label search-key").text(" ");
		var keyBox = $(document.createElement("div")).addClass("search-key-box");
		var keySelect = $(document.createElement("select"))
			.attr("id", "key-selector")
			.attr("data-bind", "options: Indexes, optionsText: 'DisplayName', value: Active");

		keyBox.append(keyLabel).append(keySelect);

		if (this.showIndexSearchButton) {
			this.indexButton = $(document.createElement("div")).addClass("indexButton")
				.addClass("disabled")
				.click(() => {
					this.executeSearch(true);
				});
			keyBox.append(this.indexButton);
		}

		var valBox = $(document.createElement("div")).addClass("search-value-box");
		var valLabel = $(document.createElement("div")).addClass("label search-value").text(" ");
		this.valueInput = $(document.createElement("input"))
			.attr("type", "textbox")
			.attr("data-bind", "textInput: Value")
			.addClass("search-value");

		this.searchButton = $(document.createElement("div")).addClass("searchButton").attr("type", "submit")
			.addClass("disabled")
			.click(() => {
				this.executeSearch(false);
			});

        this.valueInput.keypress((e) => {
            if (e.keyCode == 13) { this.executeSearch(false); }
        });

		valBox.append(valLabel).append(this.valueInput).append(this.searchButton);

		container.append(valBox);
		container.append(keyBox);

		this.topLevel.append(container);

		this.topLevel.focusout(() => {
			this.valueInput.removeClass("success").removeClass("failed");
		});

		ko.applyBindings(this.viewModel, this.topLevel[0]);

		this.Ready.trigger("");
	}

	private executeSearch(searchByIndex: boolean) {
		if (!this.buttonEnabled()) return;

		var dialogManager = window.orgAnice.dialogManager;
		if (dialogManager == undefined || dialogManager.CurrentDialog == undefined) return;

		var options: ISearchArgs = {
			value: this.viewModel.Value() || "",
			index: (this.viewModel.Active() != undefined ? this.viewModel.Active().Name : "") || "",
			table: this.viewModel.TableName() || "",
			byIndex: searchByIndex,
			count: -1
		};

		dialogManager.CurrentDialog.Search(options);
	}

	private onIndexChanged(newIndex: IIndexDescriptor): void {
		if (this.suppressIndexChanged) return;
		var oldIndex = this.ViewModel.OldIndex == undefined ? "" : this.ViewModel.OldIndex.Name;
		if (oldIndex === newIndex.Name) return;

		var dialogManager = window.orgAnice.dialogManager;
		if (this.tempOid == undefined)
			this.tempOid = dialogManager.CurrentDialog.CurrentOid;

		var eventArgs: IIndexChangedEventArgs = {
			Controller: this.ViewModel.Controller(),
			Index: newIndex.Name,
			Key: this.tempOid,
			OldIndex: this.ViewModel.OldIndex == undefined ? "" : this.ViewModel.OldIndex.Name
		}
		this.tempOid = undefined;

		this.IndexChanged.trigger(eventArgs);
	}

	private onPropertyChanged(property: string) {
		this.PropertyChanged.trigger(this, property);
	}

	private onReady() {
		this.isReadyFired = true;
		this.Ready.FireOnAdd(this.isReadyFired);
		this.Ready.trigger("");
	}
}

class SearchControlViewModel implements ISearchControlViewModel {
	private activeIndex: KnockoutObservable<IIndexDescriptor> = ko.observable<IIndexDescriptor>();
	private controller: KnockoutObservable<string> = ko.observable<string>();
	private indexes: KnockoutObservableArray<IIndexDescriptor> = ko.observableArray<IIndexDescriptor>();
	private oldIndex: IIndexDescriptor;
	private searchValue: KnockoutObservable<string> = ko.observable<string>();
	private tableName: KnockoutObservable<string> = ko.observable<string>();

	private buttonEnabled: KnockoutComputed<boolean> = ko.computed<boolean>(() => {
		return this.indexes() != undefined && this.indexes().length > 0 &&
			this.searchValue() != undefined && this.searchValue().length > 0;
	}).extend({ update: "always" });

	public get Active(): KnockoutObservable<IIndexDescriptor> {
		return this.activeIndex;
	}

	public get ButtonEnabled(): KnockoutComputed<boolean> {
		return this.buttonEnabled;
	}

	public get Controller(): KnockoutObservable<string> {
		return this.controller;
	}

	public get Indexes(): KnockoutObservableArray<IIndexDescriptor> {
		return this.indexes;
	}

	public get OldIndex(): IIndexDescriptor {
		return this.oldIndex;
	}

	public get Value(): KnockoutObservable<string> {
		return this.searchValue;
	}

	public get TableName(): KnockoutObservable<string> {
		return this.tableName;
	}

	constructor() {
		this.activeIndex.subscribe((oldIndex) => {
			this.oldIndex = oldIndex;
		}, null, "beforeChange");
	}
}

class SearchResult implements IListQueryResult {
	
	private countPages: number;
	private currentPage: number;
	private error: string;
	private index: string;
	private oid: string;
	private success: boolean;
	private table: string;
	private value: string;
	private list: {}[];

	public get CountRecords(): number { return this.countPages; }
	public set CountRecords(value: number) { this.countPages = value; }

	public get CurrentRecord(): number { return this.currentPage; }
	public set CurrentRecord(value: number) { this.currentPage = value; }

	public get Error(): string { return this.error; }
	public set Error(value: string) { this.error = value; }

	public get Index(): string { return this.index; }
	public set Index(value: string) { this.index = value; }

	public get Oid(): string { return this.oid; }
	public set Oid(value: string) { this.oid = value; }

	public get Success(): boolean { return this.success; }
	public set Success(value: boolean) { this.success = value; }

	public get Table(): string { return this.table; }
	public set Table(value: string) { this.table = value; }

	public get Value(): string { return this.value; }
	public set Value(value: string) { this.value = value; }

	public get List(): {}[] { return this.list; }
	public set List(value: {}[]) { this.list = value; }
}

