﻿
interface IShortcutControlOptions {
	/// <summary>Name od the class to instantiate. </summary>
	instanceClass: string;
	/// <summary>The service endpoint for the query; will be extended by the <see href="ShortcutEndpoint" />
	///			 property, or by the literal <c>GetListShortcuts</c>, if not set. </summary>
	serviceEndpoint: string;
	/// <summary>The text to use for the <c>all</c> button; if not set, no <c>all</c> button will be displayed. </summary>
	shortcutAllSymbol: string;
	/// <summary>The shortcut column; the list column for which to retrieve the shortcuts. </summary>
	shortcutColumn: string;
	/// <summary>The placement of the control; required for choosing the correct layout. </summary>
	placement: Placement;
}

interface IShortcutControl {
	// --- Events ---------------------------------------------------------

	/// <summary>Raised if a property of the control has changed.</summary>
	PropertyChanged: orgAniceModule.orgAniceEvent<string>;

	/// <summary>Raised after new shortcuts have been retrieved.</summary>
	QueryShortcutsFinished: orgAniceModule.orgAniceEvent<string>;

	/// <summary>Raised if the selected shortcut changes; usually in effect of a user action on the shortcut UI. </summary>
	ShortcutChangedEvent: orgAniceModule.orgAniceEvent<string>

	// --- Properties -----------------------------------------------------
	AllSymbol: string
	CurrentShortcut: string;
	IncludeAll: boolean;
	Height: number;
	ServiceEndpoint: string;
	ShortcutColumn: string;
	Toplevel: JQuery;
	ViewModel: IShortcutViewModel;
	Width: number;

	// --- Methods --------------------------------------------------------
	QueryShortcuts(keyColumn?: string): void;
	Show(show: boolean): void;
	UpdateShortcuts(shortcuts: string[], currentShortcut?: string): void;
} 