﻿
///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/knockout-3.2.0.d.ts"/>
///<reference path="orgAnice.browserExtensions.d.ts"/> />
///<reference path="orgAnice.d.ts"/> />
///<reference path="orgAnice.data.d.ts"/> />
///<reference path="orgAnice.statusBar.d.ts"/> />
///<reference path="orgAnice.shortcutControl.d.ts"/> />

// ReSharper disable RedundantVariableTypeSpecification

class ShortcutControl implements IShortcutControl {
	// --- Events ---------------------------------------------------------
	public PropertyChanged: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	public QueryShortcutsFinished: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	public ShortcutChangedEvent: orgAniceModule.orgAniceEvent<string> = new orgAniceModule.orgAniceEvent<string>();

	// --- Fields ---------------------------------------------------------
	private isBound: boolean = false;
	private currentShortcut: string;
	private displayStatus: string;
	private height: KnockoutObservable<number> = ko.observable<number>();
	private includeAllButton: boolean;
	private instanceClass: string;
	private serviceEndpoint: string;
	private shortcutAllSymbol: string;
	private shortcutColumn: string;			// the field in the list to query for shortcuts
	private toplevel: JQuery;
	private width: KnockoutObservable<number> = ko.observable<number>();

	// --- Properties -----------------------------------------------------
	public get AllSymbol(): string { return this.shortcutAllSymbol || "**"; }

	public set AllSymbol(value: string) {
		this.shortcutAllSymbol = value;
		this.onPropertyChanged("AllSymbol");
	}

	public get CurrentShortcut(): string { return this.currentShortcut; }

	public set CurrentShortcut(value: string) {
		this.currentShortcut = value;
		this.onPropertyChanged("CurrentShortcut");
	}

	public get Height(): number { return this.height(); }

	public set Height(value: number) {
		if (value == undefined) return;

		value = Math.floor(value);
		if (value === this.height()) return;

		this.height(value);
		this.onPropertyChanged("Height");
	}

	public get HeightProperty(): KnockoutObservable<number> { return this.height; }

	public get IncludeAll(): boolean { return this.includeAllButton || false; }

	public set IncludeAll(value: boolean) {
		this.includeAllButton = value;
		this.onPropertyChanged("IncludeAll");
	}

	public get ShortcutColumn(): string { return this.shortcutColumn; }

	public set ShortcutColumn(value: string) {
		this.shortcutColumn = value;
		this.onPropertyChanged("ShortcutColumn");
	}

	public get ServiceEndpoint(): string { return this.serviceEndpoint; }

	public set ServiceEndpoint(value: string) {
		this.serviceEndpoint = value;
		this.onPropertyChanged("ServiceEndpoint");
	}

	public get Toplevel(): JQuery {
		return this.toplevel;
	}

	public set Toplevel(value: JQuery) {
		this.toplevel = value;
		this.onPropertyChanged("Toplevel");
	}

	public get ViewModel(): IShortcutViewModel { return undefined; }

	public get Width(): number { return this.width(); }

	public set Width(value: number) {
		if (value == undefined) return;

		value = Math.floor(value);
		if (value === this.width()) return;

		this.width(value);
		this.onPropertyChanged("Width");
	}

	public get WidthProperty(): KnockoutObservable<number> { return this.width; }

	// --- Methods --------------------------------------------------------
	public static Create(options: IShortcutControlOptions): IShortcutControl {
		switch (options.instanceClass.toLowerCase()) {
			case "mobileshortcutcontrol":
				return new MobileShortcutControl(options);

			case "desktopshortcutcontrol":
			default:
				return new DesktopShortcutControl(options);
		}
	}


	public QueryShortcuts(keyColumn?: string): void {
		if (this.serviceEndpoint == undefined || this.serviceEndpoint === "") return;

		var control = this;

		$.ajax({
				url: this.serviceEndpoint + "/GetListShortcuts",
				contentType: "application/json; charset=utf-8",
				dataType: "json",
				data: { index: this.shortcutColumn }, // if undefined, the active index will be used
				//data: { index: keyColumn, shortcutColumn: this.shortcutColumn }, // if undefined, the active index will be used
				cache: false,
				type: "GET"
			})
			.done((result: IAjaxQueryResult) => {
				if (result.Success) {
					var results = [];

					if (control.includeAllButton)
						results.push(control.shortcutAllSymbol);

					$.each(result.Data, (index, val) => { results.push(val); });

					control.ViewModel.shortcuts(results);
				} else {
					$(document).statusBar().AddMessage("Fehler beim Laden der Schnellanwahl!", StatusBarMessageSeverity.Error);
				}
				this.QueryShortcutsFinished.trigger(control);
			})
			.fail((xhr) => {
				window.orgAnice.dialogManager.HandleAjaxError(xhr);
			});
	}

	public Show(show: boolean): void {
		if (this.toplevel == undefined) return;

		if (show === false) {
			this.displayStatus = this.toplevel.attr("display");
			this.toplevel.attr("display", "none");
		} else {
			this.toplevel.attr("display", this.displayStatus || "block");
		}
	}

	public UpdateShortcuts(shortcuts: string[], currentShortcut?: string): void {
		this.ViewModel.shortcuts(shortcuts);
		this.ViewModel.currentShortcut(currentShortcut);
	}

	// --- Constructor(s) -------------------------------------------------
	constructor(options: IShortcutControlOptions) {
		this.instanceClass = options.instanceClass || "DesktopShortcustControl";
		this.serviceEndpoint = options.serviceEndpoint;
		this.shortcutColumn = options.shortcutColumn;
	}

	// --- Implementation -------------------------------------------------
	private onPropertyChanged(propertyName: string): void {
		this.PropertyChanged.trigger(this, propertyName);
	}

	private onShortcutsChanged(newShortcut: string) {
		this.ShortcutChangedEvent.trigger(this, newShortcut);
	}
}

class DesktopShortcutControl extends ShortcutControl {
	// --- Events ---------------------------------------------------------
	// --- Fields ---------------------------------------------------------
	private resetShortcuts: JQuery;
	private viewmodel: IShortcutViewModel = {
		currentShortcut: ko.observable<string>(),
		shortcuts: ko.observableArray([]),
		onClick: (data, event) => {
			var id = $(event.currentTarget).attr("id");
			var item = $(event.currentTarget);
			if (item != undefined) {
				var parent = item.parent();
				$(parent).find("li").removeClass("selected");
				item.addClass("selected");
			}
			this.viewmodel.currentShortcut(id);
		}
	};

	// --- Properties -----------------------------------------------------
	public get ViewModel(): IShortcutViewModel { return this.viewmodel; }

	// --- Methods --------------------------------------------------------
	public UpdateShortcuts(shortcuts: string[], currentShortcut?: string): void {
		this.viewmodel.shortcuts(shortcuts);
		this.viewmodel.currentShortcut(currentShortcut);
	}

	// --- Constructor(s) -------------------------------------------------
	constructor(options: IShortcutControlOptions) {
		super(options);
		this.create(options);
	}

	// --- Implementation -------------------------------------------------
	private create(options: IShortcutControlOptions): any {
		var pane = $(document.createElement("div"));
		pane.attr("id", name);
		pane.addClass("shortcut-pane");

		var ul = $(document.createElement("ul"));
		ul.attr("data-bind", "foreach: shortcuts");

		var span = $(document.createElement("span"));
		span.attr("data-bind", "text: $data");

		var li = $(document.createElement("li"));
		li.attr("data-bind",
			"attr: { id: $data }," +
			"click: $parent.onClick");

		li.append(span);
		ul.append(li);
		pane.append(ul);

		this.Toplevel = pane;

		var control = this;

		this.resetShortcuts = $(document.createElement("div"))
			.attr("id", "reset")
            .addClass("reset")
			.click(() => {
				control.Toplevel.find("li").removeClass("selected");
				control.viewmodel.currentShortcut('');
			});

		this.Toplevel.append(this.resetShortcuts);

		this.viewmodel.currentShortcut.subscribe((newValue) => {
			control.CurrentShortcut = newValue;
			control.ShortcutChangedEvent.trigger(control, newValue);
		});

		ko.applyBindings(this.viewmodel, pane[0]);

		return pane;
	}

}

class MobileShortcutControl extends ShortcutControl {
	// --- Events ---------------------------------------------------------
	// --- Fields ---------------------------------------------------------
	private resetShortcuts: JQuery;
	private viewmodel: IShortcutViewModel = {
		currentShortcut: ko.observable<string>(),
		shortcuts: ko.observableArray([]),
		onClick: (data, event) => {
			var id = $(event.currentTarget).attr("id");
			var item = $(event.currentTarget);
			if (item != undefined) {
				var parent = item.parent();
				$(parent).find("li").removeClass("selected");
				item.addClass("selected");
			}
			this.viewmodel.currentShortcut(id);
		}
	};

	// --- Properties -----------------------------------------------------
	public get ViewModel(): IShortcutViewModel { return this.viewmodel; }

	// --- Methods --------------------------------------------------------
	// --- Constructor(s) -------------------------------------------------
	constructor(options: IShortcutControlOptions) {
		super(options);
		this.create(options);

		var control = this;
		this.viewmodel.shortcuts.subscribe(() => {
			var length = control.viewmodel.shortcuts() == undefined ? 0 : control.viewmodel.shortcuts().length;
			this.Toplevel.attr("count-shortcuts", length);
			this.layout({ doHeight: true });
		});

		this.HeightProperty.subscribe(() => {
			this.layout({ doHeight: true });
		});
		this.WidthProperty.subscribe(() => {
			this.layout({ doWidth: true });
		});
	}

	// --- Implementation -------------------------------------------------
	private create(options: IShortcutControlOptions): void {
		var control = this;

		this.Toplevel = $(document.createElement("div"));
		this.Toplevel.attr("id", name);
		this.Toplevel.addClass("shortcut-pane");

		this.resetShortcuts = $(document.createElement("div"))
			.attr("id", "reset")
			.addClass("reset")
			.click(() => {
				control.Toplevel.find("li").removeClass("selected");
				control.viewmodel.currentShortcut('');
			});

		this.Toplevel.append(this.resetShortcuts);

		var ul = $(document.createElement("ul")).attr("id", "shortcuts").attr("data-bind", "foreach: shortcuts");

		var span = $(document.createElement("span")).attr("data-bind", "text: $data");

		var li = $(document.createElement("li")).attr("data-bind", "attr: { id: $data }, click: $parent.onClick");

		li.append(span);
		ul.append(li);
		this.Toplevel.append(ul);

		this.viewmodel.currentShortcut.subscribe((newValue) => {
			control.CurrentShortcut = newValue;
			control.ShortcutChangedEvent.trigger(control, newValue);
		});

		ko.applyBindings(this.viewmodel, this.Toplevel[0]);
	}

	private layout(options?: {}): void {
		var doHeight = options != undefined && options.hasOwnProperty("doHeight") && options["doHeight"],
			doWidth = options != undefined && options.hasOwnProperty("doWidth") && options["doWidth"];

		if (doHeight) {
			let height = this.Height;
			let toplevelHeight = height;

			height -= this.resetShortcuts.height();

			var items = this.Toplevel.find("li");
			if (items != undefined && items.length > 0) {
				var fontSize: number = 14;
				var itemHeight = Math.floor(height / items.length);
				var size = $(items[0]).css("font-size");
				if (size != undefined) {
					fontSize = window.orgAnice.ConvertUnits(size);
				}
				var calcSize = Math.floor(itemHeight / 4 * 3);
				calcSize = Math.min(calcSize, 14);	// limit to 16px

				toplevelHeight = items.length * itemHeight + this.resetShortcuts.height() + 18;

				$.each(items, (index, item) => {
					var $item = $(item);
					if (fontSize !== calcSize)
						$item.css("font-size", calcSize + "px");
					$item.height(itemHeight);
				});
			}

			this.Toplevel.height(toplevelHeight);
		}
		if (doWidth) {
			var width = this.Width;
			this.Toplevel.width(width);
		}
	}
}

// ReSharper restore RedundantVariableTypeSpecification
