﻿
///<reference path="orgAnice.statusBar.d.ts"/>

enum StatusBarMessageSeverity {
	Normal,
	Success,
	Warning,
	Error
}

class DataOperationError implements IDataOperationError {
	private comError: number;
	private exceptionTypename: string;
	private fieldname: string;
	private message: string;
	private messageKey: string;
	private tablename: string;

	public get ComError(): number { return this.ComError; }
	public set ComError(value: number) {
		this.ComError = value;
	}

	public get ExceptionTypename(): string { return this.exceptionTypename; }
	public set ExceptionTypename(value: string) {
		this.exceptionTypename = value;
	}

	public get Fieldname(): string { return this.fieldname; }
	public set Fieldname(value: string) {
		this.fieldname = value;
	}

	public get Message(): string { return this.message; }
	public set Message(value: string) {
		this.message = value;
	}

	public get MessageKey(): string { return this.messageKey; }
	public set MessageKey(value: string) {
		this.messageKey = value;
	}

	public get Tablename(): string { return this.tablename; }
	public set Tablename(value: string) {
		this.tablename = value;
	}
}


class StatusBarMessage {
	// --- Fields ---------------------------------------------------------
	//public static Normal: number = 0;
	//public static Success: number = 10;
	//public static Warning: number = 20;
	//public static Error: number = 30;
	public static DefaultSeverity = StatusBarMessageSeverity.Normal;

	private duration: number;
	private message: string;
	private severity: StatusBarMessageSeverity = StatusBarMessage.DefaultSeverity;

	// --- Fields ---------------------------------------------------------
	public get Duration(): number { return this.duration; }

	public get Message(): string { return this.message; }

	public get Severity(): number { return this.severity; }

	// --- Constructor ----------------------------------------------------
	constructor(options: {}) {
		this.duration = (options.hasOwnProperty("duration") && options["duration"]) || StatusBar.DefaultDuration;
		this.message = (options.hasOwnProperty("message") && options["message"]) || "";
		this.severity = (options.hasOwnProperty("severity") && options["severity"]) || StatusBarMessage.DefaultSeverity;
	}
}

class StatusBar implements IStatusBar {
	// --- Fields ---------------------------------------------------------
	public static DefaultDuration: number = 2000;
	public static DefaultTransition: number = 200;

	private defaultOptions: {} = {
		duration: StatusBar.DefaultDuration,
		severity: StatusBarMessageSeverity.Normal
	};
	private messages: KnockoutObservableArray<StatusBarMessage> = ko.observableArray<StatusBarMessage>([]);
	private topLevelNode: any;

	// --- Events ---------------------------------------------------------
	// --- Properties -----------------------------------------------------
	public get TopLevelNode(): any {
		return this.topLevelNode;
	}

	// --- Methods --------------------------------------------------------
	public AddMessage(message: any, severity?: StatusBarMessageSeverity, duration?: number, errors?: IDataOperationError[]): void {
		var options = {};
		if (typeof message == "string") {
			if (errors != undefined && errors.length > 0) {
				message += "<br>";
				var error = errors[0];
				if (error.Fieldname != undefined) {
					message += error.Fieldname + ": ";
				}
				if (error.Message != undefined)
					message += error.Message;
			}
			$.extend(options, { message: message }, this.defaultOptions);
		} else if (typeof message == "object") {
			$.extend(options, message, this.defaultOptions);
		}
		if (severity != undefined) {
			options["severity"] = severity;
		}
		if (duration != undefined && !isNaN(duration)) {
			options["duration"] = duration;
		}

		//if (arguments.length > 2 && !isNaN(arguments[2]))
		//	options["severity"] = arguments[2];
		//if (arguments.length > 1 && !isNaN(arguments[1]))
		//	options["duration"] = arguments[1];


		const msg = new StatusBarMessage(options);
		this.messages.push(msg);
		this.show(this, msg);
	}

	public Clear(): void {
		this.topLevelNode.text("");
	}

	public Hide(): void {
		const transition = window.orgAnice.IsTouch ? 0 : StatusBar.DefaultTransition;
		this.topLevelNode.hide(transition);
	}

	public Show(): void {
		
	}

	// --- Constructor ----------------------------------------------------
	constructor(options?: {}) {
		if (options == undefined) options = {};
		$.extend(options, {
			displayAsOverlay: true,
			id: "orgAniceStatusBar"
		});

		this.topLevelNode = $(document.createElement("div"));
		this.topLevelNode.attr("id", options["id"]);
		this.topLevelNode.css("display", "none");

		var text = $(document.createElement("div")).attr("id", "messages");
		var button = $(document.createElement("button"));

		var queue = this.messages;
		this.messages.subscribe((newValue) => {
			if (newValue == undefined || (typeof newValue == "object" && newValue.length == 0)) return;	// shift?
			//this.processQueue(queue);
		});
	}

    // --- Implementation -------------------------------------------------
    private processQueue(queue: KnockoutObservableArray<StatusBarMessage>): void {
		if (queue.length == 0) return;

		var message = queue.shift();

	}

	private show(caller: StatusBar, message: StatusBarMessage): void {

		//var entry = $(document.createElement("div")).addClass("status-message");
		if (message.Severity != undefined) {
            this.TopLevelNode.removeClass().addClass(StatusBarMessageSeverity[message.Severity].toString().toLocaleLowerCase());
			//entry.addClass(message.Severity.toString().toLocaleLowerCase());
		}

		this.topLevelNode.text(message.Message);

		this.topLevelNode.show(0);

		if (message.Duration > 0) {
			setTimeout(() => {
				var c: IStatusBar = document.statusBar;
				if (c != undefined) {
					c.Hide();
					c.Clear();
				}
			}, message.Duration, caller);
		}
	}
}

// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
	var control: StatusBar;
	$.fn.statusBar = (arg1: any, arg2?: any, arg3?: any): StatusBar => {

		var instance = this;
		var $instance = $(instance);
		control = $instance.data["orgAniceStatusBar"];

		if (control == undefined) {
			control = new StatusBar(arg1);

			$instance.data["orgAniceStatusBar"] = control;

			document.statusBar = control;
			$(document.body).append(control.TopLevelNode);
		}

		return control;
	};
})(jQuery || $);
// ReSharper restore JsFunctionCanBeConvertedToLambda
