﻿
interface ITapControlOptions {
	
}

interface ITapControlCommand {
	CoControl: JQuery;
	Type: string;
	Text: string;

	// --- Methods --------------------------------------------------------
	Invoke(control: JQuery): void;
}

class TapControlCommand implements ITapControlCommand {
	// --- Fields ---------------------------------------------------------
	private coControl: JQuery;
	private command: string;
	private text: string;
	private type: string;

	// --- Properties -----------------------------------------------------
	public get CoControl(): JQuery { return this.coControl; }
	public set CoControl(value: JQuery) { this.coControl = value; }

	public get Command(): string { return this.command; }
	public set Command(value: string) { this.command = value; }

	public get Text(): string { return this.text; }
	public set Text(value: string) { this.text = value; }

	public get Type(): string { return this.type; }
	public set Type(value: string) { this.type = value; }

	// --- Methods --------------------------------------------------------
	public Invoke(control: JQuery): void {
		if (control == undefined) return;
		const content = this.parseContent(this.type, control).replace(/[\(\) -]/g, "");
		const command = this.command + ":" + content;

		//window.open(command);
		this.coControl.attr("href", command);
		this.coControl[0].click();
	}

	// --- Constructor ----------------------------------------------------
	constructor(coControl: JQuery, command: string, type: string) {
		this.coControl = coControl;
		this.command = command;
		this.type = type;
	}

	// --- Implementation -------------------------------------------------
	private parseContent(arg: string, control: JQuery): string {
		if (arg == undefined || arg === "") return "";
		switch (arg.trim()) {
			case "html":
				return control.html();

			case "text":
				return control.text();

			case "value":
				const dial = control.attr("data-dialer");
				return dial || control.val();

			default:
				return control.attr(arg);
		}
	}
}

interface ITapControl {
	
}

class TapControl {
	// --- Events ---------------------------------------------------------
	// --- Fields ---------------------------------------------------------
	private attributes: string;
	private name: string;
	private topLevel: HTMLElement;

	private commands: ITapControlCommand[] = [];

	// --- Properties -----------------------------------------------------
	// --- Methods --------------------------------------------------------
	// --- Constructor ----------------------------------------------------
	constructor(args: HTMLElement) {
		if (args == undefined) return;

		this.topLevel = args;
		var $topLevel = $(this.topLevel);

		//$topLevel.css("-ms-touch-action", "none");
		//$topLevel.click((event) => {
		//	event.stopImmediatePropagation();
		//	event.preventDefault();
		//	return false;
		//});
		//$topLevel.bind("ontouchstart", (event) => {
		//	event.stopImmediatePropagation();
		//	event.preventDefault();
		//	return false;
		//});
		//$topLevel.bind("ontouchend", (event) => {
		//	event.stopImmediatePropagation();
		//	event.preventDefault();
		//	return false;
		//});

		this.name = args.id;
		this.attributes = $(args).attr("orgAnice-touch");

		if (this.attributes != undefined && this.attributes !== "") {
			this.parse(this.attributes);
		}

		var $this = this;
		$topLevel.on("taphold", () => {
			$this.commands[0].Invoke($(this.topLevel));
			return false;
		});
	}

	// --- Implementation -------------------------------------------------
	// tel: html, sms: html || email: html || http: html|text ; mutually exclusive!
	private parse(args: string): void {
		const commands = args.split(/[,]/);
		var $this = this;
		var ctrl = $(this.topLevel).parent("div");

		$.each(commands, (index, value) => {
			var parts = value.split(/[:]/);
			var coControl = $(document.createElement("a"));

			var cmd = new TapControlCommand(coControl, parts[0].trim(), parts[1].trim());
			$this.commands.push(cmd);

			//$this.coControl.attr("display", "none");
			ctrl.append(coControl);
		});
	}

	private parseContent(arg: string, control: JQuery): string {
		if (arg == undefined || arg === "") return "";
		switch (arg.trim()) {
			case "html":
				return control.html();

			case "text":
				return control.text();

			default:
				return control.attr(arg);
		}
	}
}

// ReSharper disable JsFunctionCanBeConvertedToLambda
(function ($) {
	$.fn.tapControl = function() {
		if (!this.length) return this;

		return this.each(function() {
			var $instance = $(this);
			var options: HTMLElement = $instance[0];
			var control: ITapControl = $instance.data("tabControl");

			if (control == undefined) {
				control = new TapControl(options);
				$instance.data("tabControl", control);
			}

			return control;
		});
	}

})(jQuery || $)
// ReSharper restore JsFunctionCanBeConvertedToLambda
