﻿// ReSharper disable InconsistentNaming

///<reference path="../typings/jquery/jquery.d.ts"/>
///<reference path="../typings/jquery/jquery.cookie.d.ts"/>
///<reference path="../typings/moment.d.ts"/>
///<reference path="orgAnice.d.ts"/>

module orgAniceModule {

	export enum DeviceType {
		Desktop,
		Tablet,
		Smartphone
	}

	export enum Orientation {
		Horizontal,
		Vertical
	}

	export var OrgAniceWebTags = {
		orgAniceDomain: "orgAniceDomain",
		orgAniceProvider: "orgAniceProvider",
		orgAniceUser: "orgAniceUser",
		orgAniceAgentCaps: "orgAniceAgentCaps",
		orgAniceContinuation: "orgAniceContinuation",
		orgAniceQuery: "orgAniceQuery",
		orgAniceSession: "orgAniceSession"
	}

    export class orgAnice implements IorgAnice {
        // --- Events ---------------------------------------------------------
        public on(eventType, handler) { $(document).on(eventType, handler); }

        public trigger(eventType, data) { $(document).trigger(eventType, data); }

        // --- Properties -----------------------------------------------------
		private _dialogManager: DialogCommandManager;
        private _popupManager: PopupManager;
		private _ribbonManager: RibbonManager;
		private isSmartphone: boolean = false;
		private isTablet: boolean = false;
		private isTouch: boolean = false;
		private searchControl: ISearchControl;

		// fallback values taken from 'de-DE'
		private cultureInfo: ICultureInfo = {
			DateTimeFormat: {
				AbbreviatedDayNames: ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"],
				AbbreviatedMonthNames: [],
				AMDesignator: "",
				DateSeparator: ".",
				DayNames: [],
				FullDateTimePattern: "dddd, d. MMMM.yyyy HH:mm:ss",
				LongDatePattern: "dddd, d. MMMM yyyy",
				LongTimePattern: "HH:mm:ss",
				MonthDayPattern: "d. MMMM",
				MonthNames: [],
				PMDesignator: "",
				ShortDatePattern: "dd.MM.yyyy",
				ShortTimePattern: "HH:mm",
				SortableDateTimePattern: "yyyy-MM-ddTHH:mm:ss",
				TimeSeparator: ":",
				UniversalSortableDateTimePattern: "yyyy-MM-dd HH:mm:ssZ",
				YearMonthPattern: "MMMM yyyy",
			},
			DisplayName: "Deutsch",
			Name: "de-DE",
			NativeName: "Deutsch",
			NumberFormat: undefined,
			TwoLetterISOLanguageName: "de"
		};

		/// <summary>Current session identifier. </summary>
		/// <returns>A string holding a GUID. </returns>
		/// <remarks>Usually, the server stores a unique id to a cookie when a user is logged on and a session is created for her/him.
		/// 		 We need this unique id in order to distinguish 2 or more sessions the same user might have opened to
		/// 		 the web service. </remarks>
	    public get CurrentSessionId(): string {
		    var guid = $.cookie(orgAniceModule.OrgAniceWebTags.orgAniceSession);

		    // in the (most unlikely) case there was no id written by the server
		    if (guid == undefined || guid === "") {
			    guid = window.orgAnice.CreateGuid();
			    $.cookie(orgAniceModule.OrgAniceWebTags.orgAniceSession, guid, { path: "/" });
		    }
		    // ReSharper disable once QualifiedExpressionMaybeNull
		    return guid.substr(0, 8);
	    }

	    public MomentShortDatePattern(): string {
			return "DD.MM.YY"; // this.cultureInfo.DateTimeFormat.ShortDatePattern.replace(/yyyy/, "yy");
		}

		public coreConst: {} = {};
		public deviceType: DeviceType = DeviceType.Desktop;

        // --- Properties -----------------------------------------------------
		//public get DeviceType(): DeviceType { return this.deviceType; }
		//public set DeviceType(value: DeviceType) {
		//	this.deviceType = value;
		//}

		public get dialogManager(): DialogCommandManager { return this._dialogManager; }

		public get popupManager(): PopupManager { return this._popupManager; }

		public get ribbonManager(): RibbonManager { return this._ribbonManager; }
		public set ribbonManager(value: RibbonManager) { this._ribbonManager = value; }

		public get CurrentCulture(): ICultureInfo { return this.cultureInfo; }
		public set CurrentCulture(value: ICultureInfo) { this.cultureInfo = value; }

		public GetEmSize: (element?: HTMLElement) => number;
		public GetFontSize: (element?: HTMLElement) => number;

		public get IsTouch(): boolean { return this.isTouch; }
		public set IsTouch(value: boolean) { this.isTouch = value; }

		public get SearchControl(): ISearchControl { return this.searchControl; }
		public set SearchControl(value: ISearchControl) {
			this.searchControl = value;
		}

		public static EmptyGuid: string = "00000000-0000-0000-0000-000000000000";
		public static EmptyRequestGuid: string = "00000000-0000-0000-0000-000000000001";


        // --- Methods --------------------------------------------------------
		public ConvertUnits(valueToConvert: string, element?: HTMLElement): number {
			return window.orgAnice.ConvertUnits(valueToConvert, element);
		}

        public CombinePath(path: string, part: string): string {
            if (path == undefined) {
                return (part == undefined) ? "" : part;
            }
            if (part == undefined) return path;

            if (path[path.length - 1] != "/") path += "/";
            if (part[0] == "/") part = part.substr(1);

            return path + part;
		}

		public CreateGuid(): string  {
			var d = new Date().getTime();
			var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, c => {
				var r = (d + Math.random() * 16) % 16 | 0;
				d = Math.floor(d / 16);
				return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
			});
			return uuid;
		}

        public DiffDateFromJsonDates(start: string, end: string): Date {
            var fallback = new Date(0);

            if (start == undefined || end == undefined) return fallback;

            var results = /Date\(([^)]+)\)/.exec(start);
            if (results == null) return fallback;

            var startTicks = parseFloat(results[1]);
            if (isNaN(startTicks)) return fallback;

            results = /Date\(([^)]+)\)/.exec(end);
            if (results == null) return fallback;

            var endTicks = parseFloat(results[1]);
            if (isNaN(endTicks)) return fallback;

            var diff = endTicks - startTicks;
            var diffDate = new Date(diff);

            return diffDate;
        }

	    public IsEmptyGuid(value: any): boolean {
		    if (value == undefined || typeof value != "string" || value === "") return true;
		    value = value.replace(/^[\{\}]/g, "");
			return value != orgAnice.EmptyGuid;
	    }

	    public IsEmptyRequestGuid(value: any): boolean {
		    if (value == undefined || typeof value != "string" || value === "") return false;
		    value = value.replace(/^[\{\}]/g, "");
		    return value != orgAnice.EmptyRequestGuid;
	    }

		public IsDesktop(): boolean { return this.deviceType == orgAniceModule.DeviceType.Desktop; }

		public IsSmartphone(): boolean { return this.deviceType == orgAniceModule.DeviceType.Smartphone; }

		public IsTablet(): boolean { return this.deviceType == orgAniceModule.DeviceType.Tablet; }

	    public IsValidGuid(value: any): boolean {
		    if (value == undefined || typeof value != "string" || value === "") return false;
		    return /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/.test(value);
	    }

	    JsonToShortDate: (value: string) => Date;
        JsonToShortTime: (value: string) => Date;
		JsonToDateTime: (value: string) => Date;

		public MakeStorageKey(name: string): string {
			return ("__" + this.CurrentSessionId + "_" + name).toLowerCase();
		}

		public ShowStatusMessage(value: string, duration: number): void {
		}

        public reportError(text: string): void {

        }

        //selectImageFromList: (selector: any, selectors: any[], images: string[]) => string;

        events: Event;


		static minDate: any = moment("1899-12-30T00:00:00");	// orgAnice CRM does not use UTC
	}

	export class orgAnicePath {
		private static imageBasePath: string;

		public CombinePath(path: string, part: string): string {
			if (path == undefined) {
				return (part == undefined) ? "" : part;
			}
			if (part == undefined) return path;

			if (path[path.length - 1] != "/") path += "/";
			if (part[0] == "/") part = part.substr(1);

			return path + part;
		}
	}

	class orgAniceRoot {
		private commonDialogManager: DialogCommandManager;
		private editRibbonManager: RibbonManager;
		private popupManager: PopupManager;
	}

	export interface IorgAniceEvent<T> {
		add(handler: { (args?: T): void });
		remove(handler: { (args?: T): void });
		trigger(caller: any, args?: T);
		trigger(args?: T);
		FireOnAdd(value: boolean): void;
	}

	export class orgAniceEvent<T> implements IorgAniceEvent<T> {

		private handlers: { (args?: T): void }[] = [];
		private isFireOnAdd: boolean = false;

		public add(handler: (caller: any, args?: T) => void);
		public add(handler: (args?: T) => void) {
			this.handlers.push(handler);
			if (this.isFireOnAdd) {
				var context = arguments.length == 2 ? arguments[0] : this;
				var arg = [];
				arg.push(arguments[arguments.length == 2 ? 1 : 0]);

				handler.call(context, arg);
			}
		}

		public IsFireOnAdd(): boolean { return this.isFireOnAdd; }
		public FireOnAdd(value: boolean) { this.isFireOnAdd = value; }

		public remove(handler: (args?: T) => void) {
			if (typeof handler === "function") {
				for (var i = 0, l = this.handlers.length; i < l; i++) {
					if (this.handlers[i] === handler) {
						this.handlers.splice(i, 1);
						break;
					}
				}
			}
		}

		public trigger(caller: any, arg?: T);
		public trigger(arg?: T) {

			var context = arguments.length == 2 ? arguments[0] : this;
			var handlers = this.handlers; //.splice(0);

			var args = [];
			if (arguments.length == 2)
				args.push(arguments[1]);
			else
				args.push(arg);

			for (var i = 0, l = handlers.length; i < l; i++) {
				handlers[i].call(context, args);
			}
		}
	}

	export var IsEmptyGuid = (value: any): boolean => {
		if (value == undefined || typeof value != "string" || value === "") return true;
		value = value.replace(/^[\{\}]/g, "");
		return value === "00000000-0000-0000-0000-000000000000";
    }

    export var IsEmptyRequestGuid = (value: any): boolean => {
        if (value == undefined || typeof value != "string" || value === "") return false;
        value = value.replace(/^[\{\}]/g, "");
        return value === "00000000-0000-0000-0000-000000000001";
    }

	export var IsValidGuid = (value: any): boolean => {
		if (value == undefined || typeof value != "string" || value === "") return false;
		return /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/.test(value);
	}

}

// ReSharper disable once JsFunctionCanBeConvertedToLambda
(function(window) {

	var orgAniceRoot = {

		// path used by the theming system
		imageBasePath: "/content/themes/orgAnice/images/",

		// combines 2 path segments into 1
		combinePath: (path: string, part: string) => {
			if (path == undefined) {
				return (part == undefined) ? "" : part;
			}
			if (part == undefined) {
				return "";
			}

            if (path[path.length - 1] != "/")
                path = path.concat("/");
            if (part[0] == "/")
                part = part.substr(1);

			return path + part;
		},
		coreConst: {
			minDate: moment("1899-12-30") // minDate as defined in orgAnice core
		}
	};

	window.orgAnice = orgAniceRoot;

	$.extend(orgAniceRoot, {
		on: (eventType, handler) => { $(document).on(eventType, handler); },

		trigger: (eventType, data) => { $(document).trigger(eventType, data); },

		deviceType: orgAniceModule.DeviceType.Desktop,

		dialogManager: new DialogCommandManager(() => confirm("Möchten Sie den geänderten Datensatz speichern?")),

		reportError: (text: string) => {
			var message = "Es ist ein Fehler aufgetreten!\n\n";
			if (text != undefined)
				message += text;

			message += "\n\n";
			message += "Die Anwendung wird fortgesetzt. Aus Sicherheitsgründen, " +
				"und um die Integrität Ihrer Daten zu wahren, sollten Sie die Anwendung jedoch neu starten " +
				"indem Sie sich ab- und neu anmelden!";

			alert(message);
		},

		ConvertUnits: (valueToConvert: string, element?: HTMLElement): number => {
			if (valueToConvert == undefined) {
				return 0;
			}
			var units = valueToConvert.replace(/[0-9]*/, '');
			var value = parseFloat(valueToConvert);
			switch (units) {
				case "px":
					return value;
				case "em":
					var emSize = parseFloat(fontSize) || 16;	// fallback
					try {
						if (element == undefined)
							element = document.documentElement;

						var cfs = getComputedStyle(element, 'fontSize');
						var fontSize = cfs.height;
						emSize = parseFloat(fontSize) || 16;
					} catch (e) {}

					return value * emSize;
				default:
					return value;
			}
		},

		CreateGuid: () => {
			var d = new Date().getTime();
			var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, c => {
				var r = (d + Math.random() * 16) % 16 | 0;
				d = Math.floor(d / 16);
				return (c == 'x' ? r : (r & 0x3 | 0x8)).toString(16);
			});
			return uuid;
		},

		CurrentSessionId: string => {
			var guid = $.cookie(orgAniceModule.OrgAniceWebTags.orgAniceSession);

			// in the (most unlikely) case there was no id written by the server
			if (guid == undefined || guid === "") {
				guid = window.orgAnice.CreateGuid();
				$.cookie(orgAniceModule.OrgAniceWebTags.orgAniceSession, guid, { path: "/" });
			}
			// ReSharper disable once QualifiedExpressionMaybeNull
			return guid.substr(0, 8);
		},

		GetEmSize: (element?: HTMLElement): number => {
			if (element == undefined) {
				element = document.documentElement;
			}
			var cfs = getComputedStyle(element, 'fontSize');
			var fontSize = cfs.height;
			return parseFloat(fontSize) || 16;
		},

		GetFontSize: (element?: HTMLElement): number => {
			if (element == undefined) {
				element = document.documentElement;
			}
			var cfs = getComputedStyle(element, 'fontSize');
			var fontSize = window.orgAnice.ConvertUnits(cfs.fontSize);
			return parseFloat(fontSize) || 16;
		},

		IsDesktop: () => { return window.orgAnice.deviceType == orgAniceModule.DeviceType.Desktop; },
		IsSmartphone: () => { return window.orgAnice.deviceType == orgAniceModule.DeviceType.Smartphone; },
		IsTablet: () => { return window.orgAnice.deviceType == orgAniceModule.DeviceType.Tablet; },

		MakeStorageKey: (name: string) => {
			return ("__" + this.CurrentSessionId + "_" + name).toLowerCase();
		},

		SetDeviceType: (device: string) => {
			device = device || "desktop";
			window.orgAnice.IsTouch = false;
			switch (device.toLowerCase()) {
				case "desktop":
					window.orgAnice.deviceType = orgAniceModule.DeviceType.Desktop;
					break;
				case "smartphone":
					window.orgAnice.deviceType = orgAniceModule.DeviceType.Smartphone;
					window.orgAnice.IsTouch = true;
					break;
				case "tablet":
					window.orgAnice.deviceType = orgAniceModule.DeviceType.Tablet;
					window.orgAnice.IsTouch = true;
					break;
			}
			$.cookie("DeviceType", window.orgAnice.deviceType, { expires: 30, path: "/" });
		},

		//ribbonManager: orgAniceRoot.editRibbonManager,

		JsonToShortDate: value => {
			try {
				if (value == undefined) return "";
				var results = /Date\(([^)]+)\)/.exec(value);
				var ticks = parseFloat(results[1]);
				if (isNaN(ticks)) return "";

				return (new Date(ticks)).toLocaleDateString();
			} catch (ex) {
				return "";
			}
		},
		JsonToShortTime: value => {
			try {
				if (value == undefined) return "";
				var results = /Date\(([^)]+)\)/.exec(value);
				var ticks = parseFloat(results[1]);
				if (isNaN(ticks)) return "";

				return (new Date(ticks)).toLocaleTimeString();
			} catch (ex) {
				return "";
			}
		},
		JsonToDateTime: value => {
			try {
				if (value == undefined) return "";
				var results = /Date\(([^)]+)\)/.exec(value);
				var ticks = parseFloat(results[1]);
				if (isNaN(ticks)) return "";

				return (new Date(ticks)).toLocaleString();
			} catch (ex) {
				return "";
			}
		},
		DiffDateFromJsonDates: (start, end) => {
			var fallback = new Date(0);

			if (start == undefined || end == undefined) return fallback;

			var results = /Date\(([^)]+)\)/.exec(start);
			if (results == null) return fallback;

			var startTicks = parseFloat(results[1]);
			if (isNaN(startTicks)) return fallback;

			results = /Date\(([^)]+)\)/.exec(end);
			if (results == null) return fallback;

			var endTicks = parseFloat(results[1]);
			if (isNaN(endTicks)) return fallback;

			var diff = endTicks - startTicks;
			var diffDate = new Date(diff);

			return diffDate;
		},

		//selectImageFromList: (selector: any, selectors: any[], images: string[]): string => {
		//	if (selector == undefined || selectors == undefined || images == undefined) return "";
		//	var pos = selectors.indexOf(selector);
		//	if (pos == -1 || pos >= images.length) return "";
		//	var image = images[pos];

		//	return (image[0] == "/" || image[0] == "~") ? image : orgAniceRoot.combinePath(orgAniceRoot.imageBasePath, image);
		//},

		toNumber: (value: any) => {
			if (value == undefined) return 0;

			try {
				var result = value.replace(/[^-\d\.]/g, "");
				return isNaN(result) ? 0 : result;
			} catch (e) {
				return 0;
			}
		},

		isEmptyGuid: (value: any) => {
			if (value == undefined || typeof value != "string") return true;

			value = value.replace(/^[\{\}]/g, "");

			return value === "00000000-0000-0000-0000-000000000000";
		},

		emptyGuid: "00000000-0000-0000-0000-000000000000",
		emptyRequestGuid: "00000000-0000-0000-0000-000000000001",

	    isEmptyRequestGuid: (value: any): boolean => {
			if (value == undefined || typeof value != "string" || value === "") return false;
			value = value.replace(/^[\{\}]/g, "");
			return value === "00000000-0000-0000-0000-000000000001";
		},

	    isValidGuid: (value: any): boolean => {
			if (value == undefined || typeof value != "string" || value === "") return false;
			return /^[0-9a-fA-F]{8}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{4}-[0-9a-fA-F]{12}$/.test(value);
		},


		getCookie: (name: string) => {
			return $.cookie(name, { path: "/" });
		},

		getUserCookie: (name: string) => {
			var cookie = $.cookie("orgAniceUser");
			var fullName;
			if (cookie == undefined || typeof cookie != "string") {
				fullName = "user";
			} else {
				fullName = cookie;
			}
			fullName += name;

			return $.cookie(fullName);
		},

		getUsernameFromCookie: (): string => {
			var cookie = $.cookie("orgAniceUser");

			return (cookie == undefined || typeof cookie != "string") ? "unknownUser" : cookie;
		},

		pageSize: 50,

		setCookie: (name: string, value: string) => {
			$.removeCookie(name, { path: "/" });
			$.cookie(name, value, { path: "/" });
		},

		setUserCookie: (name: string, value: string) => {
			var cookie = $.cookie("orgAniceUser");
			var fullName;
			if (cookie == undefined || typeof cookie != "object") {
				fullName = "user";
			} else {
				fullName = cookie.Value;
			}
			fullName += name;

			return $.cookie(fullName, value, { path: "/" });
		},

		removeCookie: (name: string) => {
			$.removeCookie(name, { path: "/" });
		}
	});

	($ => {
		$.each(['show', 'hide'], (i, ev) => {
			var el = $.fn[ev];
			$.fn[ev] = function() {
				this.trigger(ev);
				return el.apply(this, arguments);
			};
		});
	})(jQuery || $);

})(window);

// TODO: this is ugly! re-define this in a more best-practices manner (i.e. in the context of the global orgAnice module)
var addressCookieName = "AddressOid";
var contactCookieName = "ContactOid";
var processCookieName = "ProcessOid";
var activityCookieName = "ActivityOid";

var addressFullnameName = "_AddressFullname";
var contactFullnameName = "_ContactFullname";

var EmptyGuid: string = "00000000-0000-0000-0000-000000000000";
var EmptyRequestGuid: string = "00000000-0000-0000-0000-000000000001";


// ReSharper restore InconsistentNaming
